@@ -1,4 +1,4 @@
- /* $Id: hook.h,v 1.3 2000/06/28 09:11:38 edwards Exp $ */
+ /* $Id: hook.h,v 1.7 2000/07/17 17:04:17 edwards Exp $ */
  
 /*
  * hook.h: header for hook.c
@@ -16,7 +16,6 @@ enum HOOK_TYPES {
 	BROWSE_BEGIN_LIST,
 	BROWSE_END_LIST,
 	CHANNEL_JOIN_LIST,
-	CHANNEL_NICK_LIST,
 	CONNECT_LIST,
 	DISCONNECT_LIST,
 	ERROR_LIST,
@@ -35,24 +34,28 @@ enum HOOK_TYPES {
 	KILL_LIST,
 	LEAVE_LIST,
 	LIST_LIST,
+	MP3_LIST,
 	MSG_LIST,
 	NAMES_LIST,
+	NAP_LIST,
 	NAPERROR_LIST,
 	NAPFINISH_LIST,
 	NAPGET_LIST,
 	NAPREQUEST_LIST,
 	NAPSEND_LIST,
+	PASTE_LIST,
 	PONG_LIST,
 	PUBLIC_LIST,
 	PUBLIC_OTHER_LIST,
 	RAW_IRC_LIST,
-	REDIRECT_LIST,
 	SEARCH_BEGIN_LIST,
 	SEARCH_END_LIST,
+	SEND_ACTION_LIST,
 	SEND_MSG_LIST,
 	SEND_OPS_LIST,
 	SEND_PUBLIC_LIST,
 	SERVERMSG_LIST,
+	SIGUSR1_LIST,
 	STATUS_UPDATE_LIST,
 	SWITCH_CHANNELS_LIST,
 	SWITCH_WINDOWS_LIST,
