@@ -6,7 +6,7 @@
  * Copyright(c) 1990, 1995 Michael Sandroff and others 
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
- * $Id: alias.c,v 1.4 2000/07/01 14:03:03 edwards Exp $
+ * $Id: alias.c,v 1.7 2000/08/27 06:15:44 cvs Exp $
  */
 
 #define _cs_alist_hash_
@@ -28,6 +28,7 @@
 #include "window.h"
 #include "ircterm.h"
 #include "server.h"
+#include "stack.h"
 #include "list.h"
 #include "keys.h"
 #include "newio.h"
@@ -389,7 +390,7 @@ BUILT_IN_COMMAND(assigncmd)
 	 * Register the variable
 	 */
 	else
-		add_var_alias(real_name, args);
+		add_var_alias(real_name, args, 1);
 
 	new_free(&real_name);
 	return;
@@ -501,7 +502,7 @@ BUILT_IN_COMMAND(localcmd)
 			*next_name++ = 0;
 
 		real_name = remove_brackets(name, NULL, NULL);
-		add_local_alias(real_name, args);
+		add_local_alias(real_name, args, 1);
 		new_free(&real_name);
 		name = next_name;
 	}
@@ -698,7 +699,7 @@ void	prepare_alias_call (void *al, char **stuff)
 		}
 
 		/* Add the local variable */
-		add_local_alias(args->vars[i], next_val);
+		add_local_alias(args->vars[i], next_val, 0);
 		if (expanded)
 			new_free(&expanded);
 	}
@@ -808,7 +809,7 @@ Alias *make_new_Alias (char *name)
  * If ``name'' refers to an already created local variable, that
  * local variable is used (invisibly)
  */
-void	add_var_alias	(char *name, char *stuff)
+void	add_var_alias	(char *name, char *stuff, int noisy)
 {
 	char *ptr;
 	Alias *tmp = NULL;
@@ -827,7 +828,7 @@ void	add_var_alias	(char *name, char *stuff)
 	 * Weed out invalid variable names
 	 */
 	ptr = after_expando(name, 1, NULL);
-	if (*ptr)
+	if (*ptr && noisy)
   		error("ASSIGN names may not contain '%c' (You asked for [%s])", *ptr, name);
 		
 
@@ -835,13 +836,13 @@ void	add_var_alias	(char *name, char *stuff)
 	 * Weed out FUNCTION_RETURN (die die die)
 	 */
 	else if (!strcmp(name, "FUNCTION_RETURN"))
-		add_local_alias(name, stuff);
+		add_local_alias(name, stuff, noisy);
 
 	/*
 	 * Pass the buck on local variables
 	 */
 	else if ((local == 1) || ((local == 0) && find_local_alias(name, NULL)))
-		add_local_alias(name, stuff);
+		add_local_alias(name, stuff, noisy);
 
 	else if (stuff && *stuff)
 	{
@@ -871,16 +872,17 @@ void	add_var_alias	(char *name, char *stuff)
 		/*
 		 * And tell the user.
 		 */
-		say("Assign %s added [%s]", name, stuff);
+		if (noisy)
+			say("Assign %s added [%s]", name, stuff);
 	}
 	else
-		delete_var_alias(name, window_display);
+		delete_var_alias(name, noisy);
 
 	new_free(&save);
 	return;
 }
 
-void	add_local_alias	(char *name, char *stuff)
+void	add_local_alias	(char *name, char *stuff, int noisy)
 {
 	char *ptr;
 	Alias *tmp = NULL;
@@ -895,7 +897,8 @@ void	add_local_alias	(char *name, char *stuff)
 	ptr = after_expando(name, 1, NULL);
 	if (*ptr)
 	{
-		error("LOCAL names may not contain '%c' (You asked for [%s])", *ptr, name);	
+		if (noisy)
+			error("LOCAL names may not contain '%c' (You asked for [%s])", *ptr, name);	
 		new_free(&name);
 		return;
 	}
@@ -915,7 +918,8 @@ void	add_local_alias	(char *name, char *stuff)
 	malloc_strcpy(&(tmp->stuff), stuff);
 	alias_total_allocated++;
 	alias_total_bytes_allocated += strlen(tmp->stuff);
-	say("Assign %s (local) added [%s]", name, stuff);
+	if (noisy)
+		say("Assign %s (local) added [%s]", name, stuff);
 
 	new_free(&name);
 	return;
@@ -1800,7 +1804,7 @@ char *	parse_line_alias_special (char *name, char *what, char *args, int d1, int
 	if (function)
 	{
 		last_function_call_level = wind_index;
-		add_local_alias("FUNCTION_RETURN", empty_string);
+		add_local_alias("FUNCTION_RETURN", empty_string, 0);
 	}
 	window_display = old_window_display;
 
@@ -2069,7 +2073,7 @@ void 	panic_dump_call_stack 	(void)
  */
 void 	destroy_call_stack 	(void)
 {
-	wind_index = 0;
+	wind_index = -1;
 	new_free((char **)&call_stack);
 }
 
@@ -2140,9 +2144,9 @@ BUILT_IN_FUNCTION(aliasctl)
 		{
 			upper(listc);
 			if (list == VAR_ALIAS_LOCAL)
-				add_local_alias(listc, input);
+				add_local_alias(listc, input, 0);
 			else if (list == VAR_ALIAS)
-				add_var_alias(listc, input);
+				add_var_alias(listc, input, 0);
 			else
 				add_cmd_alias(listc, NULL, input);
 
@@ -2211,7 +2215,6 @@ BUILT_IN_FUNCTION(aliasctl)
 	RETURN_EMPTY;
 }
 
-#if 0
 /*************************** stacks **************************************/
 typedef	struct	aliasstacklist
 {
@@ -2329,4 +2332,3 @@ void	do_stack_alias (int type, char *args, int which)
 	say("Unknown STACK type ??");
 }
 
-#endif
\ No newline at end of file
