@@ -1,5 +1,5 @@
 
- /* $Id: commands.c,v 1.9 2000/07/01 06:40:21 edwards Exp $ */
+ /* $Id: commands.c,v 1.42 2000/09/09 06:21:21 cvs Exp $ */
  
 #include "teknap.h"
 #include "struct.h"
@@ -17,6 +17,7 @@
 #include "vars.h"
 #include "window.h"
 #include "status.h"
+#include "stack.h"
 #include "napster.h"
 #include "commands.h"
 #include "timer.h"
@@ -25,10 +26,16 @@
 #include "tgtk.h"
 #include "whois.h"
 #include "ptest.h"
+#include "irchandler.h"
+
 #ifdef WANT_CD
 #include "cdrom.h"
 #endif
 
+#ifdef WANT_MP3PLAYER
+#include "mp3control.h"
+#endif
+
 #include <unistd.h>
 #include <sys/stat.h>
 
@@ -98,9 +105,19 @@ BUILT_IN_COMMAND(invite_cmd);
 BUILT_IN_COMMAND(setuser);
 BUILT_IN_COMMAND(variablecmd);
 BUILT_IN_COMMAND(url_grabber);
+BUILT_IN_COMMAND(pastecmd);
+BUILT_IN_COMMAND(stackcmd);
+BUILT_IN_COMMAND(pop_cmd);
+BUILT_IN_COMMAND(push_cmd);
+BUILT_IN_COMMAND(setenvcmd);
+BUILT_IN_COMMAND(xtypecmd);
+BUILT_IN_COMMAND(irccmd);
+BUILT_IN_COMMAND(shiftcmd);
+BUILT_IN_COMMAND(unshiftcmd);
 
 int load_depth = -1;
 int loading_savefile = 0;
+int initial_load = 0;
 extern int current_numeric;
 
 PingStruct *ping_time = NULL;
@@ -110,6 +127,12 @@ static IgnoreStruct *ignores = NULL;
 static IgnoreStruct *signores = NULL;
 static IgnoreStruct *cignores = NULL;
 
+void send_text(const char *, const char *, char *, int, int);
+
+char *recv_nick = NULL;
+char *sent_nick = NULL;
+
+
 typedef	struct	WaitCmdstru
 {
 	char	*stuff;
@@ -205,13 +228,14 @@ IrcCommand irc_command[] = {
 { "INPUT",	"INPUT",	inputcmd,	HELP_INPUT, 0, 0 },
 { "INPUT_CHAR",	"INPUT_CHAR",	inputcmd,	HELP_INPUTCHAR, 0, 0 },
 { "INVITE",	NULL,		invite_cmd,	HELP_INVITE, SERVERREQ, 0 },
+{ "IRC",	NULL,		irccmd,		HELP_IRCCMD, 0, 0 },
 { "JOIN",	NULL,		joincmd,	HELP_JOIN, 0, 0 },
 { "KICK",	NULL,		send_kick,	HELP_KICK, 0, CMDS_KICK },
 { "KILL",	NULL,		send_kill,	HELP_KILL, SERVERREQ, CMDS_KILLUSER },
 { "L",		NULL,		partcmd,	HELP_L, SERVERREQ, 0 },
 { "LASTLOG",	NULL,		lastlog,	HELP_LASTLOG, 0, 0 },
 { "LEAVE",	NULL,		partcmd,	HELP_LEAVE, SERVERREQ, 0 },
-{ "LIST",	NULL,		send_comm,	HELP_LIST, SERVERREQ, CMDS_SHOWALLCHANNELS },
+{ "LIST",	"LIST",		send_comm,	HELP_LIST, SERVERREQ, CMDS_SHOWALLCHANNELS },
 { "LOAD",	NULL,		load,		HELP_LOAD, 0, 0 },
 { "LOCAL",	NULL,		localcmd,	HELP_LOCAL, 0, 0 },
 #ifndef WINNT
@@ -221,6 +245,9 @@ IrcCommand irc_command[] = {
 { "ME",		NULL,		send_me,	HELP_ME, 0, CMDS_SENDME },
 { "MODE",	NULL,		mode_cmd,	HELP_MODE, SERVERREQ, CMDS_CHANNELMODE },
 { "MOTD",	NULL,		send_comm,	HELP_MOTD, SERVERREQ, CMDR_MOTDS },
+#if defined(WANT_MP3PLAYER) && defined(WANT_THREAD)
+{ "MP3",	NULL,		mp3play,	NULL, 0, 0 },
+#endif
 { "MSG",	"MSG",		privmsg,	HELP_MSG, SERVERREQ, 0 },
 { "MUZZLE",	NULL,		send_muzzle,	HELP_MUZZLE, SERVERREQ, CMDS_MUZZLE },
 { "NAMES",	NULL,		send_names,	HELP_NAMES, SERVERREQ, CMDS_NAME },
@@ -232,14 +259,16 @@ IrcCommand irc_command[] = {
 { "OPSAY",	NULL,		opsay_cmd,	HELP_OPSAY, SERVERREQ, CMDS_OPWALL },
 { "PARSEKEY",	NULL,		parsekeycmd,	HELP_PARSEKEY, 0, 0 },
 { "PART",	NULL,		partcmd,	HELP_PART, SERVERREQ, 0 },
-
+{ "PASTE",	NULL,		pastecmd,	HELP_PASTE, SERVERREQ, 0 },
 { "PAUSE",	NULL,		e_pause,	HELP_PAUSE, 0, 0 },
 
 { "PING",	NULL,		ping,		HELP_PING, SERVERREQ, 0 },
+{ "POP",	NULL,		pop_cmd,	HELP_POP, 0, 0 },
 { "PRINT",	NULL,		print_napster,	HELP_PRINT, 0, 0 },
 { "PS",		"ps",		exec_cmd,	HELP_PS, 0, 0 },
 { "PTEST",	NULL,		ptestcmd,	HELP_PTEST, 0, 0 },
 { "PURGE",	NULL,		purge,		HELP_PURGE, 0, 0 },
+{ "PUSH",	NULL,		push_cmd,	HELP_PUSH, 0, 0 },
 { "QUERY",	NULL,		query,		HELP_QUERY, 0, 0 },
 { "QUEUE",	NULL,		queuecmd,	HELP_QUEUE, 0, 0 },
 { "QUIT",	NULL,		napquit,	HELP_QUITNAP, 0, 0 },
@@ -277,24 +306,29 @@ IrcCommand irc_command[] = {
 { "SETCHANLIMIT", NULL,		setuser,	HELP_SETCHANNELLIMIT, SERVERREQ, CMDS_SETCHANNELLIMIT },
 
 { "SETDATAPORT",  NULL,		setuser,	HELP_SETDATAPORT, SERVERREQ, CMDR_SETDATAPORT },
+{ "SETENV",	  NULL,		setenvcmd,	HELP_SETENV,	0, 0 },
 { "SETLINESPEED", NULL,		setuser,	HELP_SETLINESPEED, SERVERREQ, CMDS_SETLINESPEED },
 { "SETPASSWORD",  NULL,		setuser,	HELP_SETPASSWORD, SERVERREQ, CMDS_SETPASSWORD },
 { "SETUSERLEVEL", NULL,		setuser,	HELP_SETUSERLEVEL, SERVERREQ, CMDS_SETUSERLEVEL },
 
 { "SHARE",	NULL,		share_napster,	HELP_SHARE, 0, 0 },
+{ "SHIFT",	NULL,		shiftcmd,	HELP_SHIFT, 0, 0 },
 { "SIGNORE",	NULL,		server_ignore,	HELP_SIGNORE, 0, 0 },
 { "SLEEP",	NULL,		sleepcmd,	HELP_SLEEP, 0, 0 },
 { "SOUNDEX",	"SOUNDEX",	nap_search,	HELP_SOUNDEX, SERVERREQ, 0 },
 { "SPING",	NULL,		spingcmd,	HELP_SPING, SERVERREQ, 0 },
+{ "STACK",	NULL,		stackcmd,	HELP_STACKCMD, 0, 0 },
 { "STATS",	NULL,		nap_stats,	HELP_STATS, 0, 0 },
 { "STUB",	NULL,		stubcmd,	HELP_STUB, 0, 0 },
 { "SWITCH",	NULL,		switchcmd,	HELP_SWITCH, 0, 0 },
+{ "TBAN",	"TBAN",		bancmd,		HELP_TBANCMD, 0, 0 },
 { "TIMER",	NULL,		timercmd,	HELP_TIMER, 0, 0 },
 { "TOPIC",	NULL,		topic,		HELP_TOPIC, SERVERREQ, 0 },
 { "TYPE",	NULL,		typecmd,	HELP_TYPE, 0, 0 },
 { "UNBLOCK",	NULL,		send_1comm,	HELP_UNBLOCK, SERVERREQ, CMDS_UNBLOCK },
 { "UNLESS",	"UNLESS",	ifcmd,		HELP_UNLESS, 0, 0 },
 { "UNMUZZLE",	NULL,		send_unmuzzle,	HELP_UNMUZZLE, SERVERREQ, CMDS_UNMUZZLE },
+{ "UNSHIFT",	NULL,		unshiftcmd,	HELP_UNSHIFT, 0, 0 },
 { "UNVOICE",	NULL,		variablecmd,	HELP_UNVOICE, SERVERREQ, CMDS_CHANNELUNVOICE },
 { "URL",	NULL,		url_grabber,	HELP_URL, 0, 0 },
 { "USLEEP",	NULL,		usleepcmd,	HELP_USLEEP, 0, 0 },
@@ -303,10 +337,12 @@ IrcCommand irc_command[] = {
 { "W",		NULL,		whois,		HELP_W, SERVERREQ, 0 },
 { "WAIT",	NULL,		waitcmd,	HELP_WAIT, 0, 0 },
 { "WALLOP",	NULL,		send_allcomm,	HELP_WALLOP, SERVERREQ, CMDS_OPSAY },
+{ "WHICH",	"WHICH",	load,		HELP_WHICH, 0, 0 },
 { "WHILE",	"WHILE",	whilecmd,	HELP_WHILE, 0, 0 },
 { "WHOIS",	NULL,		whois,		HELP_WHOIS, SERVERREQ, 0 },
 { "WINDOW",	NULL,		windowcmd,	HELP_WINDOW, INTERNAL_HELP, 0 },
 { "XECHO",	"XECHO",	echocmd,	HELP_XECHO, 0, 0 },
+{ "XTYPE",	"XTYPE",	xtypecmd,	HELP_XTYPE, 0, 0 },
 { NULL,		NULL,		NULL,		NULL, 0, 0 }
 };
 
@@ -374,7 +410,12 @@ struct utsname buf;
 		}
 	}
 	if (arg)		
-		send_ncommand(cmd, "%s %s", arg, version_buf);
+	{
+		if (get_server_ircmode(from_server))
+			send_text(arg, version_buf, command, window_display, 1);
+		else
+			send_ncommand(cmd, "%s %s", arg, version_buf);
+	}
 	else
 		say("%s", version_buf);
 	new_free(&version_buf);
@@ -382,7 +423,10 @@ struct utsname buf;
 
 BUILT_IN_COMMAND(send_comm)
 {
-	send_ncommand(numeric, NULL);
+	if (get_server_ircmode(from_server) && command)
+		send_to_server("%s", command);
+	else
+		send_ncommand(numeric, NULL);
 }
 
 BUILT_IN_COMMAND(send_1comm)
@@ -462,7 +506,17 @@ NickStruct *n;
 BUILT_IN_COMMAND(send_me)
 {
 	if (args && *args && current_window->current_channel)
-		send_ncommand(numeric, "%s \"%s\"", current_window->current_channel, args);
+	{
+		do_hook(SEND_ACTION_LIST, "%s %s", current_window->current_channel, args);
+		if (get_server_ircmode(current_window->server))
+		{
+			char *target = next_arg(args, &args);
+			if (args && *args)
+				send_to_server("PRIVMSG %s :%cACTION %s%c", target, CTCP_DELIM_CHAR, args, CTCP_DELIM_CHAR);
+		}
+		else
+			send_ncommand(numeric, "%s \"%s\"", current_window->current_channel, args);
+	}
 }
 
 BUILT_IN_COMMAND(send_kill)
@@ -536,16 +590,25 @@ char *nick, *n;
 BUILT_IN_COMMAND(send_kick)
 {
 char *nick = NULL, *n, *chan = NULL;
+Window *tmp = NULL;
 	if ((chan = next_arg(args, &args)))
 	{
-		if (!find_in_list((List **)&current_window->nchannels, chan, 0))
+		while (traverse_all_windows(&tmp))
+		{
+			if (tmp->server != from_server)
+				continue;
+			if (find_in_list((List **)&tmp->nchannels, chan, 0))
+			{
+				nick = next_arg(args, &args);
+				break;
+			}
+		}
+		if (!nick)
 		{
 			nick = chan;
 			chan = current_window->current_channel;
 		}
 	}
-	else
-		nick = next_arg(args, &args);
 	if (chan && nick)
 	{
 		while ((n = next_in_comma_list(nick, &nick)))
@@ -574,8 +637,9 @@ BUILT_IN_COMMAND(exec_cmd)
 BUILT_IN_COMMAND(nap_scan)
 {
 ChannelStruct *ch;
-char *chan = NULL;
+char *chan = NULL, *pattern = NULL;
 int sort_type = NORMAL_SORT;
+
 	if (!args || !*args)
 		chan = current_window->current_channel;
 	else
@@ -591,31 +655,46 @@ int sort_type = NORMAL_SORT;
 				else if (!my_stricmp(arg, "speed"))
 					sort_type = SPEED_SORT;
 			}
-			else
+			else if (*arg == '#')
 				chan = arg;
+			else 
+				pattern = arg;
 		}
 	}
 	if (!chan || !*chan)
-		return;
+	{
+		chan = current_window->current_channel;;
+		if (!chan)
+			return;
+	}
 	if ((ch = (ChannelStruct *)find_in_list((List **)&current_window->nchannels, chan, 0)))
-		name_print(chan, ch->nicks, 0, sort_type);
+		name_print(chan, ch->nicks, 0, sort_type, pattern);
 }
 
 BUILT_IN_COMMAND(nap_search)
 {
 char buff[BIG_BUFFER_SIZE+1];
 char s_buff[BIG_BUFFER_SIZE+1];
+char x_buff[BIG_BUFFER_SIZE+1];
 int n = 0;
 
-int bitrate = 0;
+unsigned int size = 0;
+unsigned int duration = 0;
+unsigned int bitrate = 0;
 unsigned int freq = 0;
-int linespeed = 0;
+unsigned int linespeed = 0;
 int bit_int = -1;
 int freq_int = -1;
 int line_int = -1;
+int dur_int = -1;
+int size_int = -1;
+
 int do_type = 0;
 int buf_len = 0;
 int local = 0;
+int search_flags = 0;
+int exclude = 0;
+
 char *search_param[] = { "EQUAL TO", "AT BEST", "AT LEAST", ""};
 int soundex = 0;
 char *sound_ex[] = { "FILENAME", "SOUNDEX", "" };
@@ -624,19 +703,24 @@ char *type = NULL;
 
 	if (!args || !*args)
 	{
-		display_list(file_search);
+		FileStruct **f;
+		f = get_search_head(from_server);
+		display_list(*f);
 		return;
 	}
 	if (get_server_search(from_server))
 	{
 		if (args && !my_strnicmp(args, "-end", 4))
-			set_server_search(from_server, 0);
+			set_server_search(from_server, search_flags);
 		else
 			say("Already a search in progress");
 		return;
 	}
 	if (command && !my_stricmp(command, "soundex"))
 		soundex++;
+
+	*x_buff = 0;
+
 	while (args && *args == '-')
 	{
 		char *cmd, *val;
@@ -647,8 +731,20 @@ char *type = NULL;
 			local = 1;
 			continue;
 		}
-		val = next_arg(args, &args);
+		else if (!my_strnicmp(cmd, "-any", 4))
+		{
+			type = any;
+			do_type = 1;
+			continue;
+		}
+		else if (!my_strnicmp(cmd, "-fullpath", 4))
+		{
+			search_flags |= SEARCH_FULLPATH;
+			continue;
+		}
+		val = new_next_arg(args, &args);
 		value = my_atol(val);
+		
 		if (!my_strnicmp(cmd, "-type", 4))
 		{
 			if (!(type = val))
@@ -659,10 +755,10 @@ char *type = NULL;
 			do_type = 1;
 			continue;
 		}
-		else if (!my_strnicmp(cmd, "-any", 4))
+		if (!my_strnicmp(cmd, "-exclude", 4) && val)
 		{
-			type = any;
-			do_type = 1;
+			sprintf(x_buff+exclude, "%s ", val);
+			exclude += strlen(x_buff);
 			continue;
 		}
 		if (!my_strnicmp(cmd, "-maxresults", 4))
@@ -689,11 +785,29 @@ char *type = NULL;
 			}
 			if (!my_strnicmp(cmd, "-bitrate", 4))
 				bitrate = value, bit_int = 0;
-			else if (!my_strnicmp(cmd, "-minbitrate", 4))
+			else if (!my_strnicmp(cmd, "-minbitrate", 6))
 				bitrate = value, bit_int = 2;
-			else if (!my_strnicmp(cmd, "-maxbitrate", 4))
+			else if (!my_strnicmp(cmd, "-maxbitrate", 6))
 				bitrate = value, bit_int = 1;
 		} 
+		else if (strstr(cmd, "duration"))
+		{
+			if (!my_strnicmp(cmd, "-duration", 4))
+				duration = value, dur_int = 0;
+			else if (!my_strnicmp(cmd, "-minduration", 6))
+				bitrate = value, dur_int = 2;
+			else if (!my_strnicmp(cmd, "-maxduration", 6))
+				duration = value, dur_int = 1;
+		} 
+		else if (strstr(cmd, "size"))
+		{
+			if (!my_strnicmp(cmd, "-size", 4))
+				size = value, size_int = 0;
+			else if (!my_strnicmp(cmd, "-minsize", 6))
+				size = value, size_int = 2;
+			else if (!my_strnicmp(cmd, "-maxsize", 6))
+				size = value, size_int = 1;
+		} 
 		else if (strstr(cmd, "freq"))
 		{
 			long fr[] = {8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000, -1};
@@ -708,9 +822,9 @@ char *type = NULL;
 			}
 			if (!my_strnicmp(cmd, "-maxfreq", 4))
 				freq = value, freq_int = 1;
-			else if (!my_strnicmp(cmd, "-minfreq", 4))
+			else if (!my_strnicmp(cmd, "-minfreq", 6))
 				freq = value, freq_int = 2;
-			else if (!my_strnicmp(cmd, "-freq", 4))
+			else if (!my_strnicmp(cmd, "-freq", 6))
 				freq = value, freq_int = 0;
 		}
 		else if (strstr(cmd, "line"))
@@ -722,23 +836,31 @@ char *type = NULL;
 			}
 			if (!my_strnicmp(cmd, "-maxlinespeed", 4))
 				linespeed = value, line_int = 1;
-			else if (!my_strnicmp(cmd, "-minlinespeed", 4))
+			else if (!my_strnicmp(cmd, "-minlinespeed", 6))
 				linespeed = value, line_int = 2;
-			else if (!my_strnicmp(cmd, "-linespeed", 4))
+			else if (!my_strnicmp(cmd, "-linespeed", 6))
 				linespeed = value, line_int = 0;
 		}
+#if 0
 		else if (strstr(cmd, "-end"))
 		{
-			set_server_search(from_server, 0);
+			set_server_search(from_server, SEARCH_FINISH);
 			return;
 		}
+#endif
 	}
 	
 
 	if (!args || !*args)
 		return;
 
-	clear_filelist(&file_search);
+	*buff = 0;
+	
+	if (get_icmp_socket(from_server) > -1)
+		new_close(get_icmp_socket(from_server));
+	set_icmp_socket(from_server, open_icmp());
+	
+	clear_filelist(get_search_head(from_server));
 	if (soundex)
 		compute_soundex(s_buff, sizeof(s_buff), args);
 
@@ -747,17 +869,26 @@ char *type = NULL;
 		sprintf(buff, "TYPE %s ", type);
 		buf_len = strlen(buff);
 	}	
-
+	
 	if ((n = get_int_var(MAX_RESULTS_VAR)))
-		sprintf(buff + buf_len, "%s CONTAINS \"%s\" MAX_RESULTS %d", sound_ex[soundex], soundex ? s_buff : lower(args), n);
+		sprintf(buff + buf_len, "%s CONTAINS \"%s\" MAX_RESULTS %d ", sound_ex[soundex], soundex ? s_buff : lower(args), n);
 	else
-		sprintf(buff + buf_len, "%s CONTAINS \"%s\"", sound_ex[soundex], soundex ? s_buff : lower(args));
+		sprintf(buff + buf_len, "%s CONTAINS \"%s\" ", sound_ex[soundex], soundex ? s_buff : lower(args));
+
+	buf_len = strlen(buff);
+	
 	if (local)
 	{
-		strcat(buff, "LOCAL ");
+		strcat(buff, "LOCAL_ONLY ");
 		buf_len = strlen(buff);
 	}
 
+	if (exclude)
+	{
+		sprintf(buff + buf_len, "FILENAME EXCLUDES \"%s\" ", x_buff);
+		buf_len = strlen(buff);
+	}
+	
 	if (!do_type && !type)
 	{
 		if (bitrate && (bit_int != -1))
@@ -766,17 +897,21 @@ char *type = NULL;
 			strmopencat(buff, BIG_BUFFER_SIZE, " FREQ \"", search_param[freq_int], "\" \"", ltoa(freq), "\"", NULL);
 		if (linespeed && (line_int != -1))
 			strmopencat(buff, BIG_BUFFER_SIZE, " LINESPEED \"", search_param[line_int], "\" ", ltoa(linespeed), NULL);
+		if (duration && (dur_int != -1))
+			strmopencat(buff, BIG_BUFFER_SIZE, " DURATION \"", search_param[dur_int], "\" ", ltoa(duration), NULL);
+		if (size && (size_int != -1))
+			strmopencat(buff, BIG_BUFFER_SIZE, " SIZE \"", search_param[size_int], "\" ", ltoa(size), NULL);
 	}
 	if (do_type)	
 	{
 		if (do_hook(SEARCH_BEGIN_LIST, "%s", buff))
-			say("* Searching for type [%s] %s %s", type, args, soundex ? s_buff: "");
+			say("* Searching for type [%s] %s %s excluding [%s]", type, args, soundex ? s_buff: "", x_buff);
 	}
 	else
 		if (do_hook(SEARCH_BEGIN_LIST, "%s", buff))
-			say("* Searching for %s %s", args, soundex ? s_buff: "");
+			say("* Searching for %s %s excluding [%s]", args, soundex ? s_buff: "", x_buff);
 	send_ncommand(CMDS_SEARCH, "%s", buff);
-	set_server_search(from_server, 1);
+	set_server_search(from_server, search_flags | SEARCH_START);
 }
 
 
@@ -817,10 +952,10 @@ IgnoreStruct **todo = &ignores;
 	}
 	while ((nick = new_next_arg(args, &args)))
 	{
-		if (*nick == '-')
+		if (!my_stricmp("-REMOVE", nick))
 		{
-			nick++;
-			if (!*nick)
+			nick = next_arg(args, &args);;
+			if (!nick || !*nick)
 				continue;
 			if ((new = (IgnoreStruct *)remove_from_list((List **)todo, nick)))
 			{
@@ -828,6 +963,8 @@ IgnoreStruct **todo = &ignores;
 				new_free(&new);
 				say("Removed %s from ignore list", nick);
 			}
+			else
+				say("%s not found on ignore list", nick);
 		}
 		else
 		{
@@ -906,19 +1043,22 @@ BUILT_IN_COMMAND(partcmd)
 {
 char *cmd;
 ChannelStruct *ch = NULL;
+char *parting = NULL;
 	cmd = next_arg(args, &args);
 	if (cmd)
 	{
 		if ((ch = (ChannelStruct *)remove_from_list((List **)&current_window->nchannels, cmd)))
-			send_ncommand(CMDS_PART, "%s", cmd);
+			parting = cmd;
 		else
 		{
 			Window *tmp = NULL;
 			while (traverse_all_windows(&tmp))
 			{
+				if (tmp->server != from_server)
+					continue;
 				if ((ch = (ChannelStruct *)remove_from_list((List **)&tmp->nchannels, cmd)))
 				{
-					send_ncommand(CMDS_PART, "%s", cmd);
+					parting = cmd;
 					break;
 				}
 			}
@@ -927,7 +1067,15 @@ ChannelStruct *ch = NULL;
 	else if (current_window->current_channel)
 	{
 		if ((ch = (ChannelStruct *)remove_from_list((List **)&current_window->nchannels, current_window->current_channel)))
-			send_ncommand(CMDS_PART, "%s", current_window->current_channel);
+			parting = current_window->current_channel;
+	} else
+		parting = cmd;
+	if (parting)
+	{
+		if (get_server_ircmode(from_server))
+			send_to_server("PART %s", parting);
+		else
+			send_ncommand(CMDS_PART, "%s", parting);
 	}
 	if (ch)
 	{
@@ -963,11 +1111,17 @@ char *ch;
 			if (!ch || !*ch)
 				break;
 			while (traverse_all_windows(&tmp))
-				if ((df = (ChannelStruct *)find_in_list((List **)&tmp->nchannels, ch, 0)))
-					break;
+			{
+				if (tmp->server == from_server)
+					if ((df = (ChannelStruct *)find_in_list((List **)&tmp->nchannels, ch, 0)))
+						break;
+			}
 			if (!df)
 			{
-				send_ncommand(CMDS_JOIN, "%s", ch);
+				if (get_server_ircmode(from_server))
+					send_to_server("JOIN %s", ch);
+				else
+					send_ncommand(CMDS_JOIN, "%s", ch);
 				add_waiting_channel(current_window, ch);
 			} 
 			else if (!tmp->bind_channel || my_stricmp(tmp->bind_channel, df->channel))
@@ -1048,7 +1202,12 @@ struct target_type target[2] =
 		else if (!text || !*text)
 			;
 		else if (*current_nick == '"')
-			send_ncommand(CMDS_SEND, "%s %s", current_window->current_channel, text);
+		{
+			if (get_server_ircmode(from_server))
+				send_to_server("PRIVMSG %s :%s", current_window->current_channel, text);
+			else
+				send_ncommand(CMDS_SEND, "%s %s", current_window->current_channel, text);
+		}
 #if 0
 		else if (*current_nick == '/')
 		{
@@ -1059,23 +1218,17 @@ struct target_type target[2] =
 #endif
 		else
 		{
-			if (!command && find_in_list((List **)&current_window->nchannels, current_nick, 0))
-			{
+			if ((*current_nick == '#') || (!command && find_in_list((List **)&current_window->nchannels, current_nick, 0)))
 				hooked = 1;
-#if 0
-				do_hook(target[i].hook, "%s %s", current_nick, text);
-#endif
-			}
 			else
 			{
 				hooked = 0;
 				addtabkey(current_nick, "msg", (char *)text);
 			}
-			send_ncommand(target[hooked].hook_type, "%s %s", current_nick, text);
-#if 0
-			if (!i)
-				do_hook(target[hooked].hook, "%s %s", current_nick, text);
-#endif
+			if (get_server_ircmode(from_server))
+				send_to_server("PRIVMSG %s :%s", current_nick, text);
+			else
+				send_ncommand(target[hooked].hook_type, "%s %s", current_nick, text);
 			m_s3cat(&target[hooked].nick_list, ",", current_nick);
 			target[hooked].message = (char *)text;
 		}
@@ -1090,9 +1243,24 @@ struct target_type target[2] =
 		set_display_target(target[0].nick_list, target[0].level);
 		if (do_hook(target[0].hook, "%s %s", target[0].nick_list, target[0].message))
 			put_it(target[0].format, target[0].nick_list, target[0].message);
+		malloc_strcpy(&sent_nick, target[0].nick_list);
 		new_free(&target[0].nick_list);
 		target[0].message = NULL;
 		restore_display_target(old_mf, old_ml);
+	} 
+	else if (get_server_ircmode(from_server) && target[1].nick_list && target[1].message)
+	{
+		const char *old_mf;
+		unsigned long old_ml;
+		int is_current;				
+		save_display_target(&old_mf, &old_ml);
+		set_display_target(target[1].nick_list, target[1].level);
+		is_current = is_current_channel(target[1].nick_list, from_server, 0);
+		if (do_hook(target[1].hook, "%s %s", target[1].nick_list, target[1].message))
+			put_it(target[1].format, is_current ? get_server_nickname(from_server) : target[1].nick_list, target[1].message);
+		new_free(&target[1].nick_list);
+		target[1].message = NULL;
+		restore_display_target(old_mf, old_ml);
 	}
 	window_display = old_window_display;
 }
@@ -1109,7 +1277,33 @@ BUILT_IN_COMMAND(privmsg)
 {
 char *nick;
 	if ((nick = next_arg(args, &args)))
+	{
+		if (!strcmp(nick, ","))
+		{
+			if (!(nick = recv_nick))
+			{
+				say("You have not recieved a msg yet");
+				return;
+			}
+		}
+		else if (!strcmp(nick, "."))
+		{
+			if (!(nick = sent_nick))
+			{
+				say("You have not sent a msg yet");
+				return;
+			}
+		}
+		else if (!strcmp(nick, "*"))
+		{
+			if (!(nick = current_window->current_channel))
+			{
+				say("You are not in a current channel");
+				return;
+			}
+		}
 		send_text(nick, args, command, window_display, 1);
+	}
 }
 
 BUILT_IN_COMMAND(do_send_text)
@@ -1664,14 +1858,20 @@ BUILT_IN_COMMAND(load)
 	
 	if (!(irc_path = get_string_var(LOAD_PATH_VAR)))
 	{
+		int owc = window_display;
+		window_display = 1;
 		bitchsay("LOAD_PATH has not been set");
+		window_display = owc;
 		return;
 	}
 
 	if (++load_depth == MAX_LOAD_DEPTH)
 	{
+		int owc = window_display;
+		window_display = 1;
 		load_depth--;
 		bitchsay("No more than %d levels of LOADs allowed", MAX_LOAD_DEPTH);
+		window_display = owc;
 		return;
 	}
 
@@ -1696,19 +1896,13 @@ BUILT_IN_COMMAND(load)
 			flag = 1;
 			continue;
 		}
-#if 0
-		if (!end_strcmp(filename, ".tcl", 4))
-		{
-#ifdef WANT_TCL
-			if (Tcl_EvalFile(tcl_interp, filename) != TCL_OK)
-				error("Unable to load filename %s[%s]", filename, tcl_interp->result);
-#endif
-			continue;
-		}
-#endif
 		if (!(expanded = expand_twiddle(filename)))
 		{
-			error("Unknown user for file %s", filename);
+			int owc = window_display;
+			window_display = 1;
+			if (!initial_load)
+				error("Unknown user for file %s", filename);
+			window_display = owc;
 			continue;
 		}
 
@@ -1718,17 +1912,8 @@ BUILT_IN_COMMAND(load)
 			/* uzfopen emits an error if the file
 			 * is not found, so we dont have to. */
 			window_display = 1;
-#if 0
-#ifdef WANT_DLL
-			if (expanded)
-			{
-				dll_load(NULL, expanded, NULL, NULL);
-				new_free(&expanded);
-				window_display = owc;
-				continue;
-			}
-#endif
-#endif
+			if (!initial_load)
+				say("Filename not found [%s]", filename);
 			new_free(&expanded);
 			window_display = owc;
 			continue;
@@ -1736,10 +1921,13 @@ BUILT_IN_COMMAND(load)
 
 		if (command && *command == 'W')
 		{
+			if (!window_display)
+				window_display = 1;
 			yell ("%s", expanded);
 			if (fp)
 				fclose (fp);
 			new_free(&expanded);
+			window_display = 0;
 			continue;
 		}
 /* Reformatted by jfn */
@@ -1916,7 +2104,12 @@ case '}' :
 	if (in_comment)
 		break;
         if (!paste_level)
+	{
+		int owc = window_display;
+		window_display = 1;
 		error("Unexpected } in %s, line %d", expanded, load_level[load_depth].line);
+		window_display = owc;
+	}
 	else 
 	{
         	--paste_level;
@@ -1957,12 +2150,22 @@ case ';' :
 			}
 		}
 		if (in_comment)
+		{
+			int owc = window_display;
+			window_display = 1;
 			error("File %s ended with an unterminated comment in line %d", expanded, comment_line);
+			window_display = owc;
+		}
 		if (current_row)
 		{
 			if (paste_level)
+			{
+				int owc = window_display;
+				window_display = 1;
 				error("Unexpected EOF in %s trying to match '[' at line %d",
 						expanded, paste_line);
+				window_display = owc;
+			}
 			else
 				parse_line(NULL,current_row, flag ? args: get_int_var(INPUT_ALIASES_VAR) ? empty_string : NULL,0,0, 1);
 			new_free(&current_row);
@@ -2016,6 +2219,7 @@ void load_scripts(void)
 	if (!done++)
 	{
 		window_display = 0;
+		initial_load = 1;
 #ifdef WINNT
 		if (!access(bircrc_file, R_OK))
 			load("LOAD", bircrc_file, empty_string, NULL, 0);
@@ -2042,6 +2246,7 @@ void load_scripts(void)
 		loading_savefile--;
 		window_display = old_display;
 	}
+	initial_load = 0;
 }
 
 BUILT_IN_COMMAND(whois)
@@ -2073,7 +2278,7 @@ char *nick;
 	if (!args || !*args)
 	{
 		say("Your Hotlist:");
-		name_print(NULL, nap_hotlist, 1, 0);
+		name_print(NULL, nap_hotlist, 1, 0, NULL);
 		return;
 	}
 	while ((nick = next_arg(args, &args)))
@@ -2484,7 +2689,7 @@ BUILT_IN_COMMAND(beepcmd)
 
 BUILT_IN_COMMAND(nslookup)
 {
-#if defined(THREAD) && defined(WANT_NSLOOKUP)
+#if defined(WANT_THREAD) && defined(WANT_NSLOOKUP)
 char *host;
 char *cmd = NULL;
 	while ((host = next_arg(args, &args)))
@@ -2513,7 +2718,7 @@ char *cmd = NULL;
 
 BUILT_IN_COMMAND(ptestcmd)
 {
-#if defined(THREAD) && defined(WANT_PTEST)
+#if defined(WANT_THREAD) && defined(WANT_PTEST)
 char *arg = NULL, *host= NULL, *port = NULL;
 char *cmd = NULL;
 	if (!get_int_var(PTEST_VAR))
@@ -2703,12 +2908,7 @@ BUILT_IN_COMMAND(returncmd)
 	else
 	{
 		if (args && *args)
-		{
-			int o_w_d = window_display;
-			window_display = 0;
-			add_local_alias("FUNCTION_RETURN", args);
-			window_display = o_w_d;
-		}
+			add_local_alias("FUNCTION_RETURN", args, 0);
 		return_exception++;
 	}
 }
@@ -2809,7 +3009,9 @@ BUILT_IN_COMMAND(waitcmd)
 			}
 			else
 			{
+#if 0
 				set_input(empty_string);
+#endif
 				lock_stack_frame();
 				while (process_is_running(ctl_arg))
 					io("wait %proc");
@@ -2905,17 +3107,17 @@ char *chan;
 		say("Specify a channel as you are not on a channel");
 		return;
 	}
-	send_ncommand(CMDS_LISTOPS, "%s", chan);
+	send_ncommand(CMDS_CREATEOP, "%s", chan);
 }
 
 BUILT_IN_COMMAND(createop)
 {
 char *chan;
 	chan = next_arg(args, &args);
-	if (args && *args)
-		send_ncommand(CMDS_CREATEOP, "%s %s", chan, args);
-	else
-		say("Who's your Daddy?");
+	if (!chan)
+		chan = current_window->current_channel;
+	if (chan)
+		send_ncommand(CMDS_CREATEOP, "%s %s", chan, args && *args ? args : empty_string);
 }
 
 BUILT_IN_COMMAND(delop)
@@ -2931,9 +3133,10 @@ char *chan;
 BUILT_IN_COMMAND(variablecmd)
 {
 	char *chan;
-	chan = next_arg(args, &args);
-	if (args && *args)
-		send_ncommand(numeric, "%s %s", chan, args);
+	if (!(chan = next_arg(args, &args)))
+		chan = current_window->current_channel;
+	if (chan)
+		send_ncommand(numeric, "%s %s", chan, args && *args ? args : empty_string);
 }
 
 BUILT_IN_COMMAND(nadmin)
@@ -2969,10 +3172,11 @@ Nadmin admin_comm[] = {
 	{ "email",	CMDS_CHANGEEMAIL,	1,	2, NULL },
 	{ "dataport",	CMDS_CHANGEDATA,	1,	2, NULL },
 	{ "clearchannel",CMDS_CLEARCHANNEL,	2,	2, NULL },
-	{ "stats",	CMDS_SERVERUSAGE,	0,	2, NULL },
+	{ "stats",	CMDS_SERVERUSAGE,	-1,	2, NULL },
 	{ "register",	CMDS_ADMINREGISTER,	2,	3, NULL },
+	{ "rehash",	CMDS_SERVERREHASH,	0,	3, NULL },
+	{ "drop",	CMDS_DROPCHANNEL,	1,	3, NULL },
 #if 0
-	{ "announce",	CMDR_ANNOUNCE,		-1,	1, NULL },
 	{ "setpassword",CMDS_SETPASSWORD,	-1,	4, NULL },
 	{ "setchanlevel",CMDS_SETCHANLEVEL,	2,	9, NULL },
 	{ "setdataport",CMDR_SETDATAPORT,	2,	4, NULL }, 
@@ -3024,7 +3228,7 @@ Nadmin admin_comm[] = {
 				case 0:
 				{
 					send_ncommand(admin_comm[i].cmd, NULL);
-					break;
+					return;
 				}
 				case 1:
 				{
@@ -3032,7 +3236,7 @@ Nadmin admin_comm[] = {
 						send_ncommand(admin_comm[i].cmd, "%s", user);
 					else
 						say("Nothing to send for %s", admin_comm[i].command);
-					break;
+					return;
 				}
 				case 2:
 				{
@@ -3041,7 +3245,7 @@ Nadmin admin_comm[] = {
 						send_ncommand(admin_comm[i].cmd, "%s \"%s\"", user, args);
 					else
 						send_ncommand(admin_comm[i].cmd, "%s", user);
-					break;
+					return;
 				}
 				case -1:
 				{
@@ -3049,8 +3253,10 @@ Nadmin admin_comm[] = {
 						send_ncommand(admin_comm[i].cmd, "%s", args);
 					else
 						send_ncommand(admin_comm[i].cmd, NULL);
+					return;
 				}
 			}
+			say("Unknown /admin command %s", comm);
 			return;
 		}
 	}
@@ -3058,16 +3264,29 @@ Nadmin admin_comm[] = {
 
 BUILT_IN_COMMAND(bancmd)
 {
-char *nick, *n;
+char *nick, *n, *t = NULL;
 	nick = next_arg(args, &args);
+	if (command && !my_stricmp(command, "TBAN"))
+		t = next_arg(args, &args);
 	while ((n = next_in_comma_list(nick, &nick)))
 	{
 		if (!n || !*n)
 			break;
+
 		if (args && *args)
-			send_ncommand(CMDS_BANUSER, "%s \"%s\"", n, args);
+		{
+			if (t)
+				send_ncommand(CMDS_BANUSER, "%s \"%s\" %s", n, args, t);
+			else
+				send_ncommand(CMDS_BANUSER, "%s \"%s\"", n, args);
+		}
 		else
-			send_ncommand(CMDS_BANUSER, "%s", n);
+		{
+			if (t)
+				send_ncommand(CMDS_BANUSER, "%s \"\" %s", n, t);
+			else
+				send_ncommand(CMDS_BANUSER, "%s", n);
+		}
 	}
 }
 
@@ -3076,20 +3295,27 @@ BUILT_IN_COMMAND(opsay_cmd)
 	if (!args || !*args)
 		return;
 	if (current_window && current_window->current_channel)
+	{
+		do_hook(SEND_OPS_LIST, "%s %s", current_window->current_channel, args);
 		send_ncommand(CMDS_OPWALL, "%s %s", current_window->current_channel, args);
+	}
 }
 
 BUILT_IN_COMMAND(mode_cmd)
 {
-char *mode[] = {"MODERATED", "PRIVATE", "TOPIC", "INVITE", NULL};
+char *mode[] = {"MODERATED", "PRIVATE", "TOPIC", "INVITE", "REGISTERED", NULL};
 char *new_mode = NULL;
 char *chan = NULL;
 unsigned int i;
-
 	if (args && *args)
 	{
 		int plus_mode = 0;		
 		char *m;
+		if (get_server_ircmode(from_server))
+		{
+			send_to_server("MODE %s", args);
+			return;
+		}
 		while (args && *args)
 		{
 			switch (*args)
@@ -3126,11 +3352,17 @@ unsigned int i;
 				send_ncommand(CMDS_CHANNELMODE, "%s %s", chan, new_mode);
 			else if (current_window && current_window->current_channel)
 				send_ncommand(CMDS_CHANNELMODE, "%s %s", current_window->current_channel, new_mode);
+			new_free(&new_mode);
 			return;
 		}
 	}
 	if (chan || (current_window && current_window->current_channel))
-		send_ncommand(CMDS_CHANNELMODE, "%s", chan ? chan : current_window->current_channel);
+	{
+		if (get_server_ircmode(from_server))
+			send_to_server("MODE %s", chan ? chan : current_window->current_channel);
+		else
+			send_ncommand(CMDS_CHANNELMODE, "%s", chan ? chan : current_window->current_channel);
+	}
 }
 
 BUILT_IN_COMMAND(invite_cmd)
@@ -3177,3 +3409,215 @@ char *nick, *level, *n;
 		}
 	}
 }
+
+void get_range(char *line, int *start, int *end)
+{
+char *q = line, *p = line;
+	while (*p && isdigit(*p))
+		p++;
+	if (*p)
+		*p++ = 0;
+	*start = my_atol(q);
+	*end = *p? my_atol(p): *start;
+	if (*end < *start)
+		*end = *start;
+}
+
+BUILT_IN_COMMAND(pastecmd)
+{
+	char	*lines;
+	char	*channel = NULL;
+	Window	*win;
+	int	winref = 0;
+	int	line = 1;
+	int	topic = 0;
+	int	start_line = 1,
+		end_line = 1,
+		count;
+	Display	*start_pos;
+
+	if ((lines = next_arg(args, &args)))
+		get_range(lines, &start_line, &end_line);
+	if (!args || !*args)
+		channel = get_current_channel_by_refnum(0);
+	else
+	{
+		char *t;
+		while (args && *args)
+		{
+			t = next_arg(args, &args);
+			if (*t == '-')
+			{
+				if (!my_strnicmp(t, "-win", strlen(t)))
+				{
+					if (*args && isdigit(*args))
+					{
+						t = next_arg(args, &args);
+						winref = my_atol(t);
+					}
+				}
+				else if (!my_strnicmp(t, "-topic", strlen(t)))
+				{
+					topic = 1;
+				}
+			}
+			else 
+				channel = t;
+		}
+	}
+	if (!channel && !(channel = get_current_channel_by_refnum(0)))
+		return;
+	win = get_window_by_refnum(winref);
+
+	line = end_line;
+	
+	if (line < 1 || line > win->display_buffer_size)
+	{
+		say("Try a realistic number within your scrollbuffer");
+		return;
+	}
+	start_pos = get_screen_hold(win);
+	for (start_pos = start_pos ? start_pos->prev : win->display_ip->prev; line > 1; start_pos = start_pos->prev)
+		line--;
+
+	if (!start_pos)
+		start_pos = win->display_ip->prev;
+
+	count = end_line - start_line + 1;
+	if (count == 1 && topic && start_pos)
+	{
+		send_ncommand(CMDS_TOPIC, "%s %s", channel, start_pos->line);
+		return;
+	}
+	line = 0;
+	while (count)
+	{
+		line++;
+		if (start_pos && start_pos->line)
+		{
+			if (do_hook(PASTE_LIST, "%s %s", channel, start_pos->line))
+			{
+				char buffer[BIG_BUFFER_SIZE+1];
+				snprintf(buffer, BIG_BUFFER_SIZE, "[%3d] %s", line, start_pos->line);
+				send_text(channel, buffer, NULL, 1, 0);
+			}
+			start_pos = start_pos->next;
+		}
+		count--;
+	}
+}
+
+BUILT_IN_COMMAND(stackcmd)
+{
+	char	*arg;
+	int	len, type;
+
+	if ((arg = next_arg(args, &args)) != NULL)
+	{
+		len = strlen(arg);
+		if (!my_strnicmp(arg, "PUSH", len))
+			type = STACK_PUSH;
+		else if (!my_strnicmp(arg, "POP", len))
+			type = STACK_POP;
+		else if (!my_strnicmp(arg, "LIST", len))
+			type = STACK_LIST;
+		else
+		{
+			say("%s is unknown stack verb", arg);
+			return;
+		}
+	}
+	else
+	{
+		say("Need operation for STACK");
+		return;
+	}
+	if ((arg = next_arg(args, &args)) != NULL)
+	{
+		len = strlen(arg);
+		if (!my_strnicmp(arg, "ON", len))
+			do_stack_on(type, args);
+		else if (!my_strnicmp(arg, "ALIAS", len))
+			do_stack_alias(type, args, STACK_DO_ALIAS);
+		else if (!my_strnicmp(arg, "ASSIGN", len))
+			do_stack_alias(type, args, STACK_DO_ASSIGN);
+		else if (!my_strnicmp(arg, "SET", len))
+			do_stack_set(type, args);
+		else
+		{
+			say("%s is not a valid STACK type", arg);
+			return;
+		}
+	}
+	else
+	{
+		say("Need stack type for STACK");
+		return;
+	}
+}
+
+BUILT_IN_COMMAND(pop_cmd)
+{
+	extern char *function_pop(char *, char *);
+        char *blah = function_pop(NULL, args);
+        new_free(&blah);
+}
+
+BUILT_IN_COMMAND(push_cmd)
+{
+	extern char *function_push(char *, char *);
+        char *blah = function_push(NULL, args);
+        new_free(&blah);
+}
+
+BUILT_IN_COMMAND(setenvcmd)
+{
+	char *env_var;
+
+	if ((env_var = next_arg(args, &args)) != NULL)
+		bsd_setenv(env_var, args, 1);
+	else
+		say("Usage: SETENV <var-name> <value>");
+}
+
+BUILT_IN_COMMAND(xtypecmd)
+{
+	char	*arg;
+
+	
+	if (args && (*args == '-' || *args == '/'))
+	{
+		char saved = *args;
+		args++;
+		if ((arg = next_arg(args, &args)) != NULL)
+		{
+			if (!my_strnicmp(arg, "L", 1))
+			{
+				for (; *args; args++)
+					input_add_character(*args, empty_string);
+			}
+			else
+				say ("Unknown flag -%s to XTYPE", arg);
+			return;
+		}
+		input_add_character(saved, empty_string);
+	}
+	else
+		typecmd(command, args, empty_string, helparg, 0);
+	return;
+}
+
+BUILT_IN_COMMAND(shiftcmd)
+{
+	extern char *function_shift(char *, char *);
+        char *blah = function_shift(NULL, args);
+        new_free(&blah);
+}
+
+BUILT_IN_COMMAND(unshiftcmd)
+{
+	extern char *function_unshift(char *, char *);
+        char *blah = function_unshift(NULL, args);
+        new_free(&blah);
+}
+
