@@ -5,7 +5,7 @@
  * public domain) in order to make epic compile on some of the more painful
  * systems.  None of this is part of EPIC-proper, so dont feel that youre 
  * going to hurt my feelings if you re-use this.
- * $Id: compat.c,v 1.1.1.1 2000/06/22 09:36:25 edwards Exp $
+ * $Id: compat.c,v 1.3 2000/07/23 20:14:41 cvs Exp $
  */
 
 #include "defs.h"
@@ -242,7 +242,7 @@ static int termcap;
 
 */
 
-char *tparm(const char *str, ...) {
+char *tparm1(const char *str, ...) {
 	static char OOPS[] = "OOPS";
 	static char buf[MAX_LINE];
 	register const char *sp;
@@ -1269,8 +1269,14 @@ int inet_aton(const char *cp, struct in_addr *addr)
   */
 
 /*
- * $Id: compat.c,v 1.1.1.1 2000/06/22 09:36:25 edwards Exp $
+ * $Id: compat.c,v 1.3 2000/07/23 20:14:41 cvs Exp $
  * $Log: compat.c,v $
+ * Revision 1.3  2000/07/23 20:14:41  cvs
+ * a
+ *
+ * Revision 1.2  2000/07/18 06:18:25  cvs
+ * a
+ *
  * Revision 1.1.1.1  2000/06/22 09:36:25  edwards
  * First Revision
  *
@@ -2018,7 +2024,7 @@ int	setsid	(void)
 #endif
 
 #ifndef HAVE_MEMMOVE
-/*  $Revision: 1.1.1.1 $
+/*  $Revision: 1.3 $
 **
 **  This file has been modified to get it to compile more easily
 **  on pre-4.4BSD systems.  Rich $alz, June 1991.
@@ -2345,12 +2351,12 @@ int uname(struct utsname *buf)
 #endif
 
 
-#ifdef WINNT
+#if defined(WINNT) && !defined(USING_NCURSES)
 int tputs(const unsigned char *str, int nlines, int (*outfun)(int))
 {
 register unsigned int count = 0;
   /* Safety check. */
-	if (str)
+	if (str && *str)
 	{
 		/* Now output the capability string. */
 		while (*str)
