@@ -4,7 +4,7 @@
  * Copyright 1990 Michael Sandrof
  * Copyright 1997 EPIC Software Labs
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
- * $Id: exec.c,v 1.3 2000/06/23 05:55:50 edwards Exp $
+ * $Id: exec.c,v 1.8 2000/08/28 16:36:26 cvs Exp $
  */
 
 #include "teknap.h"
@@ -113,6 +113,7 @@ BUILT_IN_COMMAND(execcmd)
 	char		*who = NULL,
 			*logical = NULL,
 			*flag;
+
 	unsigned 	refnum = 0;
 	int		sig,
 			len,
@@ -371,7 +372,7 @@ BUILT_IN_COMMAND(execcmd)
 			 */
 			for (sig = 1; sig < NSIG-1; sig++)
 			{
-				if (!my_strnicmp(sys_siglist[sig], flag, len))
+				if (sys_siglist[sig] && !my_strnicmp(sys_siglist[sig], flag, len))
 				{
 					kill_process(i, sig);
 					return;
@@ -421,7 +422,7 @@ BUILT_IN_COMMAND(execcmd)
 		/*
 		 * Check to see if the user is changing the default target
 		 */
-		malloc_strcpy(&(proc->who), who);
+		malloc_strcpy(&proc->who, who);
 
 		if (who)
 			say("Output from process %d (%s) now going to %s", i, proc->name, who);
@@ -668,8 +669,10 @@ BUILT_IN_COMMAND(execcmd)
 			proc->waitcmds = NULL;
 			proc->who = NULL;
 			proc->disowned = 0;
+
 			if (who)
 				proc->who = m_strdup(who);
+				
 			proc->dumb = 0;
 
 			if (stdoutc)
@@ -732,10 +735,12 @@ void 		do_processes (fd_set *rd)
 			continue;
 
 		if (proc->p_stdout != -1 && FD_ISSET(proc->p_stdout, rd))
-			handle_filedesc(proc, &proc->p_stdout, EXEC_PROMPT_LIST, EXEC_LIST);
+			handle_filedesc(proc, &proc->p_stdout, 
+				EXEC_PROMPT_LIST, EXEC_LIST);
 
 		if (proc->p_stderr != -1 && FD_ISSET(proc->p_stderr, rd))
-			handle_filedesc(proc, &proc->p_stderr, EXEC_PROMPT_LIST, EXEC_ERRORS_LIST);
+			handle_filedesc(proc, &proc->p_stderr, 
+				EXEC_PROMPT_LIST, EXEC_ERRORS_LIST);
 
 		if (limit && proc->counter >= limit)
 			ignore_process(proc->index);
@@ -764,6 +769,11 @@ static void 	handle_filedesc (Process *proc, int *fd, int hook_nonl, int hook_nl
 		}
 		case 0:		/* We didnt get a full line */
 		{
+			if (exec_buffer[0] == '\n')
+			{
+				exec_buffer[0] = 0;
+				goto this_sucks;
+			}
 			if (hook_nl == EXEC_LIST && proc->stdoutpc)
 				parse_line("EXEC", proc->stdoutpc, exec_buffer, 0, 0, 1);
 			else if (hook_nl == EXEC_ERRORS_LIST && proc->stderrpc)
@@ -781,6 +791,7 @@ static void 	handle_filedesc (Process *proc, int *fd, int hook_nonl, int hook_nl
 		}
 		default:	/* We got a full line */
 		{
+this_sucks:
 			message_to(proc->refnum);
 			proc->counter++;
 
@@ -802,23 +813,16 @@ static void 	handle_filedesc (Process *proc, int *fd, int hook_nonl, int hook_nl
 			{
 				if ((do_hook(hook_nl, "%s %s", 
 						proc->logical, exec_buffer)))
-/*					if (!proc->redirect)*/
+					if (!proc->who)
 						put_it("%s", exec_buffer);
 			}
 			else
 			{
 				if ((do_hook(hook_nl, "%d %s", 
 						proc->index, exec_buffer)))
-/*					if (!proc->redirect)*/
+					if (!proc->who)
 						put_it("%s", exec_buffer);
 			}
-#if 0
-			else
-			{
-				if (!proc->who)
-					put_it("%s", exec_buffer);
-			}
-#endif
 			message_to(0);
 		}
 	}
@@ -1043,9 +1047,10 @@ void 		add_process_wait (int proc_index, const char *cmd)
  */
 static void 	cleanup_dead_processes (void)
 {
-	int	i, flag;
+	int	i;
 	List	*cmd,
 		*next;
+	char	*exit_info;
 	Process *dead, *proc;
 
 	if (!process_list)
@@ -1067,40 +1072,44 @@ static void 	cleanup_dead_processes (void)
 		dead = process_list[i];
 		process_list[i] = NULL;
 
+		if (dead->logical)
+		{
+			size_t	len = strlen(dead->logical) + 25;
+
+			exit_info = alloca(len);
+			snprintf(exit_info, len, "%s %d %d", 
+					dead->logical, dead->termsig,
+					dead->retcode);
+		}
+		else
+		{
+			exit_info = alloca(40);
+			snprintf(exit_info, 32, "%d %d %d",
+				dead->index, dead->termsig, dead->retcode);
+		}
+
 		/*
 		 * First thing we do is run any /wait %proc -cmd commands
 		 */
+
 		next = dead->waitcmds;
 		dead->waitcmds = NULL;
 		while ((cmd = next))
 		{
 			next = cmd->next;
-			parse_line(NULL, cmd->name, empty_string, 0, 0, 1);
+			parse_line(NULL, cmd->name, exit_info, 0, 0, 1);
 			new_free(&cmd->name);
 			new_free((char **)&cmd);
 		}
-		if (dead->p_stdin != -1)
-			close(dead->p_stdin);
-		dead->p_stdout = new_close(dead->p_stdout);
-		dead->p_stderr = new_close(dead->p_stderr);
 
 		/*
 		 * Flag /on exec_exit
 		 */
-		if (dead->logical)
-			flag = do_hook(EXEC_EXIT_LIST, "%s %d %d",
-				dead->logical, dead->termsig,
-				dead->retcode);
-		else
-			flag = do_hook(EXEC_EXIT_LIST, "%d %d %d",
-				dead->index, dead->termsig, dead->retcode);
-
-
-		/*
-		 * Tell the user, if they care
-		 */
-		if (flag && 0/*get_int_var(NOTIFY_ON_TERMINATION_VAR)*/)
+		if (do_hook(EXEC_EXIT_LIST, "%s", exit_info))
 		{
+			/*
+			 * Tell the user, if they care
+			 */
 			if (dead->termsig > 0 && dead->termsig < NSIG)
 				say("Process %d (%s) terminated with signal %s (%d)", 
 				   dead->index, dead->name, sys_siglist[dead->termsig], dead->termsig);
@@ -1114,6 +1123,10 @@ static void 	cleanup_dead_processes (void)
 		/*
 		 * Clean up after the process
 		 */
+		if (dead->p_stdin != -1)
+			close(dead->p_stdin);
+		new_close(dead->p_stdout);
+		new_close(dead->p_stderr);
 		new_free(&dead->name);
 		new_free(&dead->logical);
 		new_free(&dead->who);
