@@ -5,7 +5,7 @@
  * Copyright 1990 Michael Sandrof
  * Copyright 1997 EPIC Software Labs
  * See the COPYRIGHT file for more info
- *  $Id: expr.c,v 1.1.1.1 2000/06/22 09:36:47 edwards Exp $
+ *  $Id: expr.c,v 1.2 2000/07/16 12:58:55 edwards Exp $
  */
 
 #include <math.h>
@@ -175,18 +175,12 @@ union
 	/* Make sure were really an implied case */			\
 	if (ptr[-1] == '=' && stage == NU_ASSN)				\
 	{								\
-		/* Turn off the display */				\
-		display = window_display;				\
-		window_display = 0;					\
-									\
 		/* Make sure theres an lvalue */			\
 		if (*varname)						\
-			add_var_alias(varname, tmp);			\
+			add_var_alias(varname, tmp, 0);			\
 		else							\
 			yell("Invalid assignment: No lvalue");		\
 									\
-		/* Turn the display back on */				\
-		window_display = display;				\
 	}								\
 	new_free(&varname);						\
 }
@@ -416,7 +410,7 @@ union
 			/* Taken more or less from call_user_function */
 			make_local_stack(NULL/*"anonymous"*/);
 			window_display = 0;
-			add_local_alias("FUNCTION_RETURN", empty_string);
+			add_local_alias("FUNCTION_RETURN", empty_string, 0);
 			window_display = display;
 
 			will_catch_return_exceptions++;
@@ -516,9 +510,9 @@ union
 					if (end && !*end)
 					{
 						if (j < 0)
-							result1 = extract2(args, SOS, -j);
+							result1 = extractw2(args, SOS, -j);
 						else
-							result1 = extract2(args, j, j);
+							result1 = extractw2(args, j, j);
 						*arg_flag = 1;
 					}
 					/*
@@ -526,7 +520,7 @@ union
 					 */
 					else if (*end == '-' && !end[1])
 					{
-						result1 = extract2(args, j, EOS);
+						result1 = extractw2(args, j, EOS);
   						*arg_flag = 1;
   					}
 					else
@@ -678,10 +672,7 @@ union
 				else    
 					r--;
 
-				display = window_display;
-				window_display = 0;
-				add_var_alias(varname,ltoa(r));
-				window_display = display;
+				add_var_alias(varname,ltoa(r), 0);
 
 				if (!prefix)
 					r--;
@@ -1035,16 +1026,13 @@ union
 				for (varname = result1; my_isspace(*varname);)
 					varname++;
 
-				display = window_display;
-				window_display = 0;
 				upper(varname);
 
 				if (*varname)
-					add_var_alias(varname, result2);
+					add_var_alias(varname, result2, 0);
 				else
 					yell("Invalid assignment: no lvalue");
 
-				window_display = display;
 				new_free(&result1);
 				return result2;
 			}
@@ -1737,7 +1725,7 @@ register unsigned char	c;
 			    if (!args)
 				tmp2 = m_strdup(empty_string);
 			    else
-				tmp2 = extract2(args, lower, upper);
+				tmp2 = extractw2(args, lower, upper);
 
 			    TruncateAndQuote(buffer, tmp2, length, quote_em, pad_char);
 			    new_free(&tmp2);
