@@ -1,12 +1,13 @@
 /*
  * various c functions for receiving files on napster.
  * Copyright Colten Edwards Feb 2000
- * $Id: files.c,v 1.6 2000/07/02 04:41:00 edwards Exp $
+ * $Id: files.c,v 1.11 2000/08/15 22:01:04 cvs Exp $
  */
  
 #include "teknap.h"
 #include "struct.h"
 #include "list.h"
+#include "hook.h"
 #include "server.h"
 #include "commands.h"
 #include "output.h"
@@ -23,6 +24,13 @@
 #ifdef HAVE_SYS_FILIO_H
 #include <sys/filio.h>
 #endif
+#if HAVE_LIBWRAP
+#include <tcpd.h>
+int allow_severity;              /* for connection logging */
+int deny_severity;               /* for connection logging */
+int hosts_ctl (char *, char *, char *, char *);
+#endif
+
 
 ResumeFile *resume_struct = NULL;
 int move_file_to_unfinished(GetFile *, char *);
@@ -164,12 +172,6 @@ GetFile *new;
 	new->write = new->socket = -1;
 
 	add_to_transfer_list(new);
-#if 0
-	new->next = transfer_struct;
-	if (transfer_struct)
-		transfer_struct->prev = new;
-	transfer_struct = new;
-#endif
 	if (!gui)
 	{
 		sf->getfile = new;
@@ -212,12 +214,6 @@ char indata[BIG_BUFFER_SIZE+1];
 	gf->flags = NAP_DOWNLOAD;
 
 	add_to_transfer_list(gf);
-#if 0
-	if (transfer_struct)
-		transfer_struct->prev = gf;
-	gf->next = transfer_struct;
-	transfer_struct = gf;
-#endif
 	return 1;
 }
 
@@ -264,11 +260,12 @@ char *nick, *filen, *comm;
 	{
 		int	count = 1;
 		FileStruct *sf = NULL;
-
+		FileStruct **f = NULL;
 		if (!args || !*args)
 		{
 			int count = 1, maxwidth;
-			if (!(sf = file_search))
+			f = get_search_head(from_server);
+			if (!f || !(sf = *f))
 			{
 				say("Need to specify a nick");
 				return;
@@ -281,10 +278,13 @@ char *nick, *filen, *comm;
 		while (args && *args)
 		{
 			int req, browse;
+			FileStruct **f = NULL;
 			sf = NULL;
 			req = browse = 0;
 			count = 1;
 			comm = next_arg(args, &args);
+			f = get_search_head(from_server);
+			
 			if (!my_strnicmp(comm, "-request", 3))
 			{
 				req = 1;
@@ -309,8 +309,8 @@ char *nick, *filen, *comm;
 			}
 			if (!req && !browse)
 			{
-				if (file_search)
-					sf = file_search;
+				if (f && *f)
+					sf = *f;
 				else
 				{
 					NickStruct *n;
@@ -327,8 +327,8 @@ char *nick, *filen, *comm;
 					}
 				}
 			} 
-			else if (req)
-				sf = file_search;
+			else if (req && f)
+				sf = *f;
 
 			if (sf && comm)
 			{
@@ -342,7 +342,7 @@ char *nick, *filen, *comm;
 				}
 			} 
 			else
-				say("No Such number [%s] in %s", comm ? comm : zero, sf ? (sf == file_search ? "search" : "browse"):"Ack");
+				say("No Such number [%s] in %s", comm ? comm : zero, sf ? (sf == *f ? "search" : "browse"):"Ack");
 		}
 	}
 }
@@ -423,8 +423,9 @@ GetFile *gf;
 		gf->next = finished_struct;
 		finished_struct = gf;
 	} 
-	else
-		close_socketread(snum);
+	close_socketread(snum);
+	if (gf)
+		gf->socket = -1;
 #ifdef SCOTT
 	if (in_browser)
 		refresh_browser();
@@ -464,7 +465,8 @@ int flag = NORMAL_FINISH;
 		{
 			rc += read(snum, &indata[gf->count], sizeof(indata)-1);
 			indata[rc] = 0;
-			say("Invalid Reply from %s is %s", gf->nick, indata);
+			if (do_hook(NAP_LIST, "Error get 0 %s %s", gf->nick, indata))
+				say("Invalid Reply from %s is %s", gf->nick, indata);
 			break_from_list((List **)&transfer_struct, (List *)gf);
 			nap_finished_file(snum, PREMATURE_FINISH);
 			return;
@@ -476,7 +478,8 @@ int flag = NORMAL_FINISH;
 			flags |= O_CREAT;
 		if (!gf->realfile || ((gf->write = open(gf->realfile, flags, 0644)) == -1))
 		{
-			say("Error opening output file %s: %s\n", base_name(gf->realfile), strerror(errno));
+			if (do_hook(NAP_LIST, "Error get 0 \"%s\" %s", gf->realfile, strerror(errno)))
+				say("Error opening output file %s: %s\n", base_name(gf->realfile), strerror(errno));
 			break_from_list((List **)&transfer_struct, (List *)gf);
 			nap_finished_file(snum, PREMATURE_FINISH);
 			return;
@@ -484,7 +487,8 @@ int flag = NORMAL_FINISH;
 		if (gf->resume)
 			lseek(gf->write, gf->resume, SEEK_SET);	
 		sprintf(indata, "(%lu/%s)", gf->resume, longcomma(gf->filesize));
-		say("%sing from %s %s %s", gf->resume ? "Resum":"Gett", gf->nick, indata, base_name(gf->filename));
+		if (do_hook(NAP_LIST, "request get 0 %s %s %s", gf->nick, indata, gf->filename))
+			say("%sing from %s %s %s", gf->resume ? "Resum":"Gett", gf->nick, indata, base_name(gf->filename));
 		send_ncommand(CMDS_UPDATE_GET1, NULL);
 		build_status(current_window, NULL, 0);
 		return;
@@ -501,8 +505,10 @@ int flag = NORMAL_FINISH;
 	}
 	switch (rc)
 	{
+get_error:
 		case -1:
-			say("ERROR reading file [%s]", strerror(errno));
+			if (do_hook(NAP_LIST, "Error get 0 %s %s", gf->nick, gf->filename))
+				say("ERROR reading file [%s]", strerror(errno));
 			flag = PREMATURE_FINISH;
 		case 0:
 		{
@@ -517,7 +523,8 @@ int flag = NORMAL_FINISH;
 			if ((gf->received + gf->resume) >= gf->filesize)
 			{
 				if (rc != -1)
-					put_it("* Finished %sing %s (%s) from %s %s", gf->resume ? "Resum":"Gett", base_name(gf->filename), longcomma(gf->filesize), gf->nick, speed1);
+					if (do_hook(NAP_LIST, "notify get 0 %s %lu %lu %4.2f %s", gf->nick, gf->filesize, gf->received, speed, gf->filename))
+						put_it("* Finished %sing %s (%s) from %s %s", gf->resume ? "Resum":"Gett", base_name(gf->filename), longcomma(gf->filesize), gf->nick, speed1);
 				if (speed > shared_stats.max_downloadspeed)
 					shared_stats.max_downloadspeed = speed;
 				shared_stats.files_received++;
@@ -526,7 +533,8 @@ int flag = NORMAL_FINISH;
 			else
 			{
 				flag = PREMATURE_FINISH;
-				bitchsay("Error %sing %s (%lu/%lu) from %s", gf->resume ? "Resum":"Gett", base_name(gf->filename), gf->received + gf->resume, gf->filesize, gf->nick);
+				if (do_hook(NAP_LIST, "error get 0 %s %lu %lu %s", gf->nick, gf->filesize, gf->received+gf->resume, gf->filename))
+					put_it("* Error %sing %s (%lu/%lu) from %s", gf->resume ? "Resum":"Gett", base_name(gf->filename), gf->received + gf->resume, gf->filesize, gf->nick);
 			}
 			send_ncommand(CMDS_UPDATE_GET, NULL);
 			nap_finished_file(snum, flag);
@@ -536,7 +544,9 @@ int flag = NORMAL_FINISH;
 		default:
 			break;
 	}
-	write(gf->write, indata, rc);
+	if (write(gf->write, indata, rc) != rc)
+		goto get_error;
+		
 	gf->received += rc;
 #ifdef SCOTT
 	if (in_browser) {
@@ -560,7 +570,8 @@ int flag = NORMAL_FINISH;
 		sprintf(speed1, "%4.2fK/s", speed);
 		if (speed > shared_stats.max_downloadspeed)
 			shared_stats.max_downloadspeed = speed;
-		put_it("* Finished %sing %s (%s) from %s %s", gf->resume ? "Resum":"Gett", base_name(gf->filename), longcomma(gf->filesize), gf->nick, speed1);
+		if (do_hook(NAP_LIST, "notify get 0 %s %lu %lu %4.2f %s", gf->nick, gf->filesize, gf->received, speed, gf->filename))
+			put_it("* Finished %sing %s (%s) from %s %s", gf->resume ? "Resum":"Gett", base_name(gf->filename), longcomma(gf->filesize), gf->nick, speed1);
 		shared_stats.files_received++;
 		shared_stats.filesize_received += gf->received;
 		send_ncommand(CMDS_UPDATE_GET, NULL);
@@ -601,7 +612,8 @@ int rc;
 	{
 		case -1:
 			close_socketread(snum);
-			say("ERROR in nap_firewall_get %s", strerror(errno));
+			if (do_hook(NAP_LIST, "Error get 1 %s", strerror(errno)))
+				say("ERROR in nap_firewall_get %s", strerror(errno));
 		case 0:
 			break;
 		default:
@@ -642,7 +654,8 @@ int rc;
 					flags |= O_CREAT;
 				if (!gf->realfile || ((gf->write = open(gf->realfile, flags, 0644)) == -1))
 				{
-					say("Error opening output file %s: %s\n", base_name(gf->realfile), strerror(errno));
+					if (do_hook(NAP_LIST, "Error get 1 \"%s\" %s", gf->realfile, strerror(errno)))
+						say("Error opening output file %s: %s\n", base_name(gf->realfile), strerror(errno));
 					gf = (GetFile *)break_from_list((List **)&transfer_struct, (List *)gf);
 					nap_finished_file(snum, PREMATURE_FINISH);
 					return;
@@ -652,7 +665,8 @@ int rc;
 				sprintf(indata, "%lu", gf->resume);
 				write(snum, indata, strlen(indata));
 				sprintf(indata, "%4.2g%s %4.2g%s", _GMKv(gf->resume), _GMKs(gf->resume), _GMKv(gf->filesize), _GMKs(gf->filesize));
-				put_it("* %sing from %s %s (%s)", gf->resume?"Resum":"Gett", gf->nick, base_name(gf->filename), longcomma(gf->filesize));
+				if (do_hook(NAP_LIST, "notify get 1 %s %lu %s", gf->nick, gf->filesize, gf->filename))
+					put_it("* %sing from %s %s (%s)", gf->resume?"Resum":"Gett", gf->nick, base_name(gf->filename), longcomma(gf->filesize));
 				send_ncommand(CMDS_UPDATE_GET1, NULL);
 				build_status(current_window, NULL, 0);
 				s->func_read = nap_getfile;
@@ -685,19 +699,22 @@ struct stat st;
 
 	if (!nick || !ip || !file || !checksum)
 	{
-		say("Error in cmd_getfile(), value == NULL");
+		if (do_hook(NAP_LIST, "error get -1 %s", "NULL"))
+			say("Error in cmd_getfile(), value == NULL");
 		return 0;
 	}
 	if (!strcmp(checksum, "firewallerror"))
 	{
-		say("You are both firewalled. Unable to complete transfer");
+		if (do_hook(NAP_LIST, "error get 1 %s", "firewall"))
+			say("You are both firewalled. Unable to complete transfer");
 		return 0;
 	}
 	if (!(gf = find_in_getfile(1, nick, NULL/*checksum*/, file, 0, NAP_DOWNLOAD)))
 	{
 		if (!(gf = find_in_queue(1, nick, NULL/*checksum*/, file, 0)))
 		{
-			say("request %s %s not in cmd_getfile()", nick, file);
+			if (do_hook(NAP_LIST, "error get -1 %s %s", nick, file))
+				say("request %s %s not in cmd_getfile()", nick, file);
 			return 0;
 		}
 	}
@@ -722,7 +739,8 @@ struct stat st;
 	{
 		/* this is a firewalled host. make a listen socket instead */
 		send_ncommand(CMDS_REQUESTFILEFIRE, "%s \"%s\"", nick, file);
-		say("Attempting to get from a firewalled host %s", nick);
+		if (do_hook(NAP_LIST, "request get 1 %s %lu %s", nick, gf->filesize, file))
+			say("Attempting to get from a firewalled host %s", nick);
 		gf->socket = -1;
 		gf->addtime = now;
 		add_to_transfer_list(gf);
@@ -862,6 +880,15 @@ int sra = sizeof(struct sockaddr_in);
 int sock = -1;
 	if ((sock = accept(snum, (struct sockaddr *) &remaddr, &sra)) > -1)
 	{
+#if HAVE_LIBWRAP
+		if (!hosts_ctl (PACKAGE, STRING_UNKNOWN, inet_ntoa (remaddr.sin_addr),
+			STRING_UNKNOWN))
+		{
+			say("tcp wrappers denied %s", inet_ntoa (rem.sin_addr));
+			close(sock);
+			return;
+		}
+#endif
 		set_keepalive(sock);
 		add_socketread(sock, snum, 0, inet_ntoa(remaddr.sin_addr), naplink_handleconnect, NULL);
 		write(sock, "1", 1); 
@@ -932,13 +959,6 @@ int done = 0;
 				ptr->flags &= ~NAP_QUEUED;
 				ptr->addtime = now;
 				add_to_transfer_list(ptr);
-
-#if 0
-				ptr->next = transfer_struct;
-				if (transfer_struct)
-					transfer_struct->prev = ptr;
-				transfer_struct = ptr;
-#endif
 				done++;
 			}
 		}
