@@ -1,4 +1,4 @@
-/* $Id: functions.c,v 1.7 2000/07/01 14:03:03 edwards Exp $ */
+/* $Id: functions.c,v 1.23 2000/09/04 23:28:22 cvs Exp $ */
  
 #include "teknap.h"
 #include "struct.h"
@@ -6,7 +6,9 @@
 #include "array.h"
 #include "files.h"
 #include "list.h"
+#include "input.h"
 #include "napster.h"
+#include "newio.h"
 #include "output.h"
 #include "ircaux.h"
 #include "ircterm.h"
@@ -16,6 +18,11 @@
 #ifdef HAVE_REGEX_H
 # include <regex.h>
 #endif
+#ifdef HAVE_UNAME
+# include <sys/utsname.h>
+#endif
+
+extern char *recv_nick, *sent_nick;
 
 BUILT_IN_FUNCTION(nap_channel);
 BUILT_IN_FUNCTION(cparse);
@@ -91,7 +98,45 @@ BUILT_IN_FUNCTION(function_channels);
 BUILT_IN_FUNCTION(function_fexist);
 BUILT_IN_FUNCTION(function_fsize);
 BUILT_IN_FUNCTION(teknap_version);
+BUILT_IN_FUNCTION(teknap_version1);
 BUILT_IN_FUNCTION(function_onchannel);
+BUILT_IN_FUNCTION(function_xmms);
+BUILT_IN_FUNCTION(function_numwords);
+BUILT_IN_FUNCTION(function_strlen);
+BUILT_IN_FUNCTION(function_afterw);
+BUILT_IN_FUNCTION(function_lastlog);
+BUILT_IN_FUNCTION(function_line);
+BUILT_IN_FUNCTION(function_uname);
+BUILT_IN_FUNCTION(function_pop);
+BUILT_IN_FUNCTION(function_push);
+BUILT_IN_FUNCTION(function_shift);
+BUILT_IN_FUNCTION(function_unshift);
+BUILT_IN_FUNCTION(function_jot);
+
+BUILT_IN_FUNCTION(function_isalpha);
+BUILT_IN_FUNCTION(function_isdigit);
+BUILT_IN_FUNCTION(function_isalnum);
+BUILT_IN_FUNCTION(function_isspace);
+BUILT_IN_FUNCTION(function_isxdigit);
+BUILT_IN_FUNCTION(function_findw);
+
+BUILT_IN_FUNCTION(function_rest);
+BUILT_IN_FUNCTION(function_restw);
+BUILT_IN_FUNCTION(function_remw);
+BUILT_IN_FUNCTION(function_insertw);
+BUILT_IN_FUNCTION(function_chngw);
+BUILT_IN_FUNCTION(function_common);
+BUILT_IN_FUNCTION(function_diff);
+BUILT_IN_FUNCTION(function_remws);
+BUILT_IN_FUNCTION(function_status);
+BUILT_IN_FUNCTION(alias_currdir);
+BUILT_IN_FUNCTION(alias_input);
+BUILT_IN_FUNCTION(function_crypt);
+BUILT_IN_FUNCTION(alias_idle);
+BUILT_IN_FUNCTION(alias_online);
+BUILT_IN_FUNCTION(alias_sent_nick);
+BUILT_IN_FUNCTION(alias_recv_nick);
+BUILT_IN_FUNCTION(function_chmod);
 
 typedef struct {
 	char *name;
@@ -99,107 +144,144 @@ typedef struct {
 } NapFunc;
 
 NapFunc built_in[] = {
-	{ "C",	nap_channel },
-	{ "F",	nap_fromserver },
-	{ "J",	teknap_version },
-	{ "L",	nap_server_level },
-	{ "N",	nap_nick },
-	{ "O",	nap_server_version },
-	{ "S",	nap_server },
-	{ "T",	function_time },
-	{ "$",  function_dollar },
-	{ NULL, NULL }
+	{ ".",			alias_sent_nick },
+	{ ",",			alias_recv_nick },
+	{ "C",			nap_channel },
+	{ "E",			alias_idle },
+	{ "F",			nap_fromserver },
+	{ "G",			alias_online },
+	{ "I",			alias_input },
+	{ "J",			teknap_version },
+	{ "L",			nap_server_level },
+	{ "N",			nap_nick },
+	{ "O",			nap_server_version },
+	{ "S",			nap_server },
+	{ "T",			function_time },
+	{ "W",			alias_currdir },
+	{ "$",  		function_dollar },
+	{ "v",			teknap_version1 },
+	{ NULL, 		NULL }
 };
 
 NapFunc nap_func[] = {
-	{ "AFTER",	function_after },
-	{ "ALIASCTL",	aliasctl },
-	{ "ASCII",	function_ascii },
-	{ "BEFORE",	function_before },
-	{ "BUILTIN",	function_builtin },
-	{ "CHANNEL",	nap_channel },
-	{ "CHANNELNICKS",function_channelnicks },
-	{ "CHANUSERS",	function_onchannel },
-	{ "CHR",	function_chr },
-	{ "CLOSE",	function_close },
-	{ "CONNECT",	function_connect },
-	{ "CPARSE",	cparse },
-	{ "DECODE",	function_decode },
-	{ "DELARRAY",	function_delarray },
-	{ "DELITEM",	function_delitem }, 
-	{ "EOF",	function_eof },
-	{ "ENCODE",	function_encode },
-	{ "FEXIST",	function_fexist },
-	{ "FILTER",	function_filter },
-	{ "FINDITEM",	function_finditem },
-	{ "FSIZE",	function_fsize },
-	{ "GETARRAYS",	function_getarrays },
-	{ "GETCAP",	function_getcap },
-	{ "GETITEM",	function_getitem },
-	{ "GETMATCHES",	function_getmatches },
-	{ "GETOPT",	function_getopt },
-	{ "GETRMATCHES",function_getrmatches },
-	{ "GETTMATCH",	function_gettmatch },
-	{ "GLOB",	function_glob },
-	{ "IFINDFIRST",	function_ifindfirst },
-	{ "IFINDITEM",	function_ifinditem },
-	{ "IGETITEM",	function_igetitem },
-	{ "IGETMATCHES",function_igetmatches },
-	{ "IGETRMATCHES",function_getrmatches },
-	{ "INDEXTOITEM",function_indextoitem },
-	{ "ISDISPLAYING",function_isdisplaying },
-	{ "ITEMTOINDEX",function_itemtoindex },
-	{ "LEFT",	function_left },
-	{ "LEFTW",	function_leftw },
-	{ "LISTARRAY",	function_listarray },
-	{ "NUMARRAYS",	function_numarrays },
-	{ "NUMITEMS",	function_numitems },
-	{ "NUMSORT",	function_numsort },
-	{ "MATCH",	function_match },
-	{ "MATCHITEM",	function_matchitem },
-	{ "MD5",	function_md5},
-	{ "MID",	function_mid },
-	{ "MIDW",	function_midw },
-	{ "MP3TIME",	function_mp3time },
-	{ "MSAR",	function_msar },
-	{ "ONCHANNELS",	function_channels },
-	{ "OPEN",	function_open },
-	{ "PATTERN",	function_pattern },
-	{ "RAND",	function_rand },
-	{ "READ",	function_read },
-	{ "REGCOMP",	function_regcomp },
-	{ "REGERROR",	function_regerror },
-	{ "REGEXEC",	function_regexec },
-	{ "REGFREE",	function_regfree },
-	{ "REVERSE",	function_reverse },
-	{ "REVW",	function_revw },
-	{ "RFILTER",	function_rfilter },
-	{ "RIGHT",	function_right },
-	{ "RIGHTW",	function_rightw },
-	{ "RMATCH",	function_rmatch },
-	{ "RMATCHITEM",	function_rmatchitem },
-	{ "RPATTERN",	function_rpattern },
-	{ "SAR",	function_sar },
-	{ "SETITEM",	function_setitem },
-	{ "SORT",	function_sort },
-	{ "SRAND",	function_srand },
-	{ "STRCHR",	function_strchar },
-	{ "STRCMP",	function_strcmp },
-	{ "STRIP",	function_strip },
-	{ "STRFTIME",	function_strftime },
-	{ "STRRCHR",	function_strchar },
-	{ "TIME",	function_time },
-	{ "TOLOWER",	function_tolower },
-	{ "TOUPPER",	function_toupper },
-	{ "TR",		function_translate },
-	{ "TWIDDLE",	function_twiddle },
-	{ "UNLINK",	function_unlink },
-	{ "UTIME",	function_time },
-	{ "WINNUM",	function_winnum },
-	{ "WINSIZE",	function_winsize },
-	{ "WORD",	function_word },
-	{ "WRITE",	function_write },
-	{ NULL, NULL }
+	{ "AFTER",		function_after },
+	{ "AFTERW",		function_afterw },
+	{ "ALIASCTL",		aliasctl },
+	{ "ASCII",		function_ascii },
+	{ "BEFORE",		function_before },
+	{ "BUILTIN",		function_builtin },
+	{ "CHANNEL",		nap_channel },
+	{ "CHANNELNICKS",	function_channelnicks },
+	{ "CHANUSERS",		function_onchannel },
+	{ "CHMOD",		function_chmod },
+	{ "CHNGW",		function_chngw },
+	{ "CHR",		function_chr },
+	{ "CLOSE",		function_close },
+	{ "COMMON",		function_common },
+	{ "CONNECT",		function_connect },
+	{ "CPARSE",		cparse },
+	{ "CRYPT",		function_crypt },
+	{ "DECODE",		function_decode },
+	{ "DELARRAY",		function_delarray },
+	{ "DELITEM",		function_delitem }, 
+	{ "DIFF",		function_diff },
+	{ "EOF",		function_eof },
+	{ "ENCODE",		function_encode },
+	{ "FEXIST",		function_fexist },
+	{ "FILTER",		function_filter },
+	{ "FINDITEM",		function_finditem },
+	{ "FINDW",		function_findw },
+	{ "FSIZE",		function_fsize },
+	{ "GETARRAYS",		function_getarrays },
+	{ "GETCAP",		function_getcap },
+	{ "GETITEM",		function_getitem },
+	{ "GETMATCHES",		function_getmatches },
+	{ "GETOPT",		function_getopt },
+	{ "GETRMATCHES",	function_getrmatches },
+	{ "GETTMATCH",		function_gettmatch },
+	{ "GLOB",		function_glob },
+	{ "IDLE",		alias_idle },
+	{ "IFINDFIRST",		function_ifindfirst },
+	{ "IFINDITEM",		function_ifinditem },
+	{ "IGETITEM",		function_igetitem },
+	{ "IGETMATCHES",	function_igetmatches },
+	{ "IGETRMATCHES",	function_getrmatches },
+	{ "INDEXTOITEM",	function_indextoitem },
+	{ "INSERTW",		function_insertw },
+	{ "ISALNUM",		function_isalnum },
+	{ "ISALPHA",		function_isalpha },
+	{ "ISDIGIT",		function_isdigit },
+	{ "ISDISPLAYING",	function_isdisplaying },
+	{ "ISSPACE",		function_isspace },
+	{ "ISXDIGIT",		function_isxdigit },
+	{ "ITEMTOINDEX",	function_itemtoindex },
+	{ "JOT",		function_jot },
+	{ "LASTLOG",		function_lastlog },
+	{ "LEFT",		function_left },
+	{ "LEFTW",		function_leftw },
+	{ "LINE",		function_line },
+	{ "LISTARRAY",		function_listarray },
+	{ "NUMARRAYS",		function_numarrays },
+	{ "NUMITEMS",		function_numitems },
+	{ "NUMSORT",		function_numsort },
+	{ "NUMWORDS",		function_numwords },
+	{ "MATCH",		function_match },
+	{ "MATCHITEM",		function_matchitem },
+	{ "MD5",		function_md5},
+	{ "MID",		function_mid },
+	{ "MIDW",		function_midw },
+	{ "MP3TIME",		function_mp3time },
+	{ "MSAR",		function_msar },
+	{ "ONCHANNELS",		function_channels },
+	{ "OPEN",		function_open },
+	{ "PATTERN",		function_pattern },
+	{ "POP",		function_pop },
+	{ "PUSH",		function_push },
+	{ "RAND",		function_rand },
+	{ "READ",		function_read },
+	{ "REGCOMP",		function_regcomp },
+	{ "REGERROR",		function_regerror },
+	{ "REGEXEC",		function_regexec },
+	{ "REGFREE",		function_regfree },
+	{ "REMW",		function_remw },
+	{ "REMWS",		function_remws },
+	{ "REST",		function_rest },
+	{ "RESTW",		function_restw },
+	{ "REVERSE",		function_reverse },
+	{ "REVW",		function_revw },
+	{ "RFILTER",		function_rfilter },
+	{ "RIGHT",		function_right },
+	{ "RIGHTW",		function_rightw },
+	{ "RMATCH",		function_rmatch },
+	{ "RMATCHITEM",		function_rmatchitem },
+	{ "RPATTERN",		function_rpattern },
+	{ "SAR",		function_sar },
+	{ "SETITEM",		function_setitem },
+	{ "SHIFT",		function_shift },
+	{ "SORT",		function_sort },
+	{ "SRAND",		function_srand },
+	{ "STATUS",		function_status },
+	{ "STRCHR",		function_strchar },
+	{ "STRCMP",		function_strcmp },
+	{ "STRIP",		function_strip },
+	{ "STRFTIME",		function_strftime },
+	{ "STRLEN",		function_strlen },
+	{ "STRRCHR",		function_strchar },
+	{ "TIME",		function_time },
+	{ "TOLOWER",		function_tolower },
+	{ "TOUPPER",		function_toupper },
+	{ "TR",			function_translate },
+	{ "TWIDDLE",		function_twiddle },
+	{ "UNAME",		function_uname },
+	{ "UNLINK",		function_unlink },
+	{ "UNSHIFT",		function_unshift },
+	{ "UTIME",		function_time },
+	{ "WINNUM",		function_winnum },
+	{ "WINSIZE",		function_winsize },
+	{ "WORD",		function_word },
+	{ "WRITE",		function_write },
+	{ "XMMS",		function_xmms },
+	{ NULL,			NULL }
 };
 
 #define	NUMBER_OF_FUNCTIONS (sizeof(nap_func) / sizeof(NapFunc)) - 1
@@ -311,6 +393,11 @@ BUILT_IN_FUNCTION(nap_nick)
 	return m_strdup(empty_string);
 }
 
+BUILT_IN_FUNCTION(alias_input)
+{
+	return m_strdup(get_input());
+}
+
 BUILT_IN_FUNCTION(nap_channel)
 {
 	if (from_server != -1)
@@ -338,21 +425,29 @@ BUILT_IN_FUNCTION(teknap_version)
 	return m_strdup(nap_version);
 }
 
+BUILT_IN_FUNCTION(teknap_version1)
+{
+	return m_strdup(internal_version);
+}
+
+BUILT_IN_FUNCTION(alias_currdir)
+{
+	char 	*tmp = (char *)new_malloc(MAXPATHLEN+1);
+	return getcwd(tmp, MAXPATHLEN);
+}
+
+
 BUILT_IN_FUNCTION(cparse)
 {
 	int 	i, j, max;
 	char 	*output;
-	int	noappend = 0;
+	int	noappend = 0, expand = 0;
 	char *	stuff;
-	char *	free_it = NULL;
 
 	if (*input == '"')
 	{
-		int	af;
-
 		stuff = new_next_arg(input, &input);
-		free_it = expand_alias(stuff, input, &af, NULL);
-		stuff = free_it;
+		expand = 1;
 	}
 	else
 		stuff = input;
@@ -488,8 +583,12 @@ BUILT_IN_FUNCTION(cparse)
 	}
 	output[j] = 0;
 
-	if (free_it)
-		new_free(&free_it);
+	if (expand)
+	{
+		int	af;
+		stuff = expand_alias(output, input, &af, NULL);
+		RETURN_MSTR(stuff);
+	}
 
 	return output ? m_strdup(output) : m_strdup(empty_string);
 }
@@ -578,7 +677,7 @@ BUILT_IN_FUNCTION(function_sar)
 	char    *data		= NULL;
 	char	*value		= NULL;
 	char	*booya		= NULL;
-	int	variable = 0,global = 0,searchlen,oldwindow = window_display;
+	int	variable = 0,global = 0,searchlen;
 	char *(*func) (const char *, const char *) = strstr;
 	char 	*svalue;
 
@@ -635,11 +734,7 @@ BUILT_IN_FUNCTION(function_sar)
 
 	malloc_strcat(&booya, value);
 	if (variable) 
-	{
-		window_display = 0;
-		add_var_alias(data, booya);
-		window_display = oldwindow;
-	}
+		add_var_alias(data, booya, 0);
 	new_free(&svalue);
 	return (booya);
 }
@@ -666,8 +761,7 @@ BUILT_IN_FUNCTION(function_msar)
 	char    *p              = NULL;
 	int     variable 	= 0,
 		global 		= 0,
-		searchlen,
-		oldwindow 	= window_display;
+		searchlen;
 	char 	*(*func) (const char *, const char *) = strstr;
 	char    *svalue;
 
@@ -755,11 +849,7 @@ BUILT_IN_FUNCTION(function_msar)
         } while (1);
 
         if (variable) 
-        {
-                window_display = 0;
-                add_var_alias(data, booya);
-                window_display = oldwindow;
-        }
+                add_var_alias(data, booya, 0);
         new_free(&svalue);
         return (booya);
 }
@@ -918,7 +1008,7 @@ BUILT_IN_FUNCTION(function_leftw)
 	if (value < 1)
 		RETURN_EMPTY;
 
-	return (extract(input, 0, value-1));	/* DONT USE RETURN_STR HERE! */
+	return (extractw(input, 0, value-1));	/* DONT USE RETURN_STR HERE! */
 }
 
 /* $rightw(num string of text)
@@ -933,7 +1023,7 @@ BUILT_IN_FUNCTION(function_rightw)
 	if (value < 1)
 		RETURN_EMPTY;
 		
-	return extract2(input, -value, EOS); 
+	return extractw2(input, -value, EOS); 
 }
 
 
@@ -952,7 +1042,7 @@ BUILT_IN_FUNCTION(function_midw)
 	if (num < 1)
 		RETURN_EMPTY;
 
-	return extract(input, start, (start + num - 1));
+	return extractw(input, start, (start + num - 1));
 }
 
 BUILT_IN_FUNCTION(function_chr)
@@ -1239,8 +1329,7 @@ static 	char	switches	[INPUT_BUFFER_SIZE+1] = "",
 		*optptr,
 		*tmp;
 	char	extra_args	[INPUT_BUFFER_SIZE+1] = "";
-	int	old_window_display = window_display,
-		arg_flag = 0;
+	int	arg_flag = 0;
 
 	if (strcmp(last_input, input)) 
 	{
@@ -1371,8 +1460,6 @@ static 	char	switches	[INPUT_BUFFER_SIZE+1] = "",
 		strlcat(args, extra_args, INPUT_BUFFER_SIZE);
 	}
 
-	window_display = 0;
-
 	if ((tmp = next_arg(sptr, &sptr)))
 	{
 		switch (*(tmp + 1))
@@ -1380,33 +1467,27 @@ static 	char	switches	[INPUT_BUFFER_SIZE+1] = "",
 			case '_':	
 				*(tmp + 1) = 0;
 				
-				add_var_alias(optopt_var, tmp);
-				add_var_alias(optarg_var, NULL);
+				add_var_alias(optopt_var, tmp, 0);
+				add_var_alias(optarg_var, NULL, 0);
 		
-				window_display = old_window_display;
 				RETURN_STR(tmp);
 			case ':':
 				*(tmp + 1) = 0;
 				
-				add_var_alias(optopt_var, tmp);
-				add_var_alias(optarg_var, next_arg(aptr, &aptr));
-				window_display = old_window_display;
+				add_var_alias(optopt_var, tmp, 0);
+				add_var_alias(optarg_var, next_arg(aptr, &aptr), 0);
 				RETURN_STR(tmp);
 			case '-':
 				*(tmp + 1) = 0;
 				
-				add_var_alias(optopt_var, tmp);
-				add_var_alias(optarg_var, NULL);
-				
-				window_display = old_window_display;
+				add_var_alias(optopt_var, tmp, 0);
+				add_var_alias(optarg_var, NULL, 0);
 				RETURN_STR("-");
 			case '!':
 				*(tmp + 1) = 0;
 			
-				add_var_alias(optopt_var, tmp);
-				add_var_alias(optarg_var, NULL);
-			
-				window_display = old_window_display;
+				add_var_alias(optopt_var, tmp, 0);
+				add_var_alias(optarg_var, NULL, 0);
 				RETURN_STR("!");
 			default:
 				/* This shouldn't happen */
@@ -1415,9 +1496,8 @@ static 	char	switches	[INPUT_BUFFER_SIZE+1] = "",
 	}
 	else
 	{
-		add_var_alias(optopt_var, NULL);
-		add_var_alias(optarg_var, aptr);
-		window_display = old_window_display;
+		add_var_alias(optopt_var, NULL, 0);
+		add_var_alias(optarg_var, aptr, 0);
 
 		*switches = 0;
 		*args = 0;
@@ -1777,50 +1857,49 @@ BUILT_IN_FUNCTION(function_open)
 {
 	char *filename;
 	char *mode;
-	
+	int bin_mode = 0;
+	short port = -1;
+		
 	GET_STR_ARG(filename, input);
 	GET_STR_ARG(mode, input);
-
-	if (tolower(*mode) == 'r')
-		RETURN_INT(open_file_for_read(filename));
+	if (input && *input)
+		GET_INT_ARG(port, input);
+	if (tolower(*mode) == 'i')
+		RETURN_INT(open_internet_socket(filename, port, SERVICE_CLIENT));
+	else if (tolower(*mode) == 'r')
+	{
+		if (tolower(*(mode+1)) == 'b')
+			bin_mode = O_BINARY;
+		RETURN_INT(open_file_for_read(filename, bin_mode));
+	}
 	else if (tolower(*mode) == 'w')
-		RETURN_INT(open_file_for_write(filename));
+	{
+		if (tolower(*(mode+1)) == 'b')
+			bin_mode = O_BINARY;
+		RETURN_INT(open_file_for_write(filename, bin_mode));
+	}
 	RETURN_EMPTY;	
 }
 
 BUILT_IN_FUNCTION(function_close)
 {
-int snum;
-struct sockaddr_in socka;
-int len = sizeof(struct sockaddr_in);
-
+	int snum;
 	GET_INT_ARG(snum, input);
-	if (snum > 0)
-	{
-		if (!getpeername(snum, (struct sockaddr *)&socka, &len))
-		{
-			close(snum);
-			RETURN_INT(0);
-		}
-		else
-			RETURN_INT(file_close(snum));
-	}
-	RETURN_INT(-1);
+	RETURN_INT(file_close(snum));
 }
 
 BUILT_IN_FUNCTION(function_write)
 {
-	char *fdc;
-	GET_STR_ARG(fdc, input);
-	RETURN_INT(file_write(atoi(fdc), input));
+	int fdc;
+	GET_INT_ARG(fdc, input);
+	RETURN_INT(file_write(fdc, input, strlen(input)));
 }
 
 BUILT_IN_FUNCTION(function_read)
 {
-	char *fdc = NULL;
-
-	GET_STR_ARG(fdc, input);
-	return file_read (atoi(fdc));
+	int fdc;
+	GET_INT_ARG(fdc, input);
+	return file_read (fdc);
 }
 
 BUILT_IN_FUNCTION(function_eof)
@@ -1997,16 +2076,26 @@ BUILT_IN_FUNCTION(function_winsize)
 
 /*
  * Usage: $rand(max)
+ * Usage: $rand(min max)
  * Returns: A random number from zero to max-1.
  * Example: $rand(10) might return any number from 0 to 9.
  */
 BUILT_IN_FUNCTION(function_rand)
 {
-	long	tempin;
+	long	tempin, tempin2 = 0;
 
 	GET_INT_ARG(tempin, input);
+	if (input && *input)
+		GET_INT_ARG(tempin2, input);
 	if (tempin == 0)
 		tempin = (unsigned long) -1;	/* This is cheating. :P */
+	if (tempin2 != 0)
+	{
+		long ret;
+		while ((ret = random_number(0) % tempin2) < tempin)
+			;
+		RETURN_INT(ret);
+	}
 	RETURN_INT(random_number(0) % tempin);
 }
 
@@ -2042,6 +2131,8 @@ char *ret = NULL;
 	while (traverse_all_windows(&tmp))
 	{
 		ChannelStruct *ch;
+		if (tmp->server != from_server)
+			continue;
 		ch = tmp->nchannels;
 		while(ch)
 		{
@@ -2161,3 +2252,705 @@ BUILT_IN_FUNCTION(function_onchannel)
 	RETURN_EMPTY;
 }
 
+BUILT_IN_FUNCTION(function_numwords)
+{
+	RETURN_INT(word_count(input));
+}
+
+BUILT_IN_FUNCTION(function_strlen)
+{
+	if (input && *input)
+		RETURN_INT(strlen(input));
+	RETURN_INT(0);
+}
+
+/* Returns the string after the word being matched */
+BUILT_IN_FUNCTION(function_afterw)
+{
+	int     where;
+	char	*lame = NULL;
+	char	*placeholder;
+
+	lame = LOCAL_COPY(input);
+	where = my_atol((placeholder = function_rmatch(NULL, input)));
+	new_free(&placeholder);
+
+	if (where < 1)
+		RETURN_EMPTY;
+	placeholder = extractw(lame, where + 1, EOS);
+	return placeholder;
+}
+
+/*
+ * $uname()
+ * Returns system information.  Expandoes %a, %s, %r, %v, %m, and %n
+ * correspond to uname(1)'s -asrvmn switches.  %% expands to a literal
+ * '%'.  If no arguments are given, the function returns %s and %r (the
+ * same information given in the client's ctcp version reply).
+ *
+ * Contributed by Matt Carothers (CrackBaby) on Dec 20, 1997
+ */
+BUILT_IN_FUNCTION(function_uname)
+{
+#ifndef HAVE_UNAME
+	RETURN_STR("unknown");
+#else
+	struct utsname un;
+	char 	tmp[BIG_BUFFER_SIZE+1];
+	char	*ptr = tmp;
+	size_t	size = BIG_BUFFER_SIZE;
+	int 	i;
+	int	len;
+
+	if (uname(&un) == -1)
+		RETURN_STR("unknown");
+
+	if (!*input)
+		input = "%s %r";
+
+	*tmp = 0;
+	for (i = 0, len = strlen(input); i < len; i++)
+	{
+		if (ptr - tmp >= size)
+			break;
+
+		if (input[i] == '%') 
+		{
+		    switch (input[++i]) 
+		    {
+			case 'm':	strlcpy(ptr, un.machine, size);
+					break;
+			case 'n':	strlcpy(ptr, un.nodename, size);
+					break;
+			case 'r':	strlcpy(ptr, un.release, size);
+					break;
+			case 's':	strlcpy(ptr, un.sysname, size);
+					break;
+			case 'v':	strlcpy(ptr, un.version, size);
+					break;
+			case 'a':	
+					snprintf(ptr, size, "%s %s %s %s %s",
+						un.sysname, un.nodename,
+						un.release, un.version,
+						un.machine);
+					break;
+			case '%':	strlcpy(ptr, "%", size);
+		    }
+		    ptr += strlen(ptr);
+		}
+		else
+		    *ptr++ = input[i];
+	}
+	*ptr = 0;
+
+	RETURN_STR(tmp);
+#endif
+}
+
+BUILT_IN_FUNCTION(function_shift)
+{
+	char    *value = NULL;
+	char    *var    = NULL;
+	char	*booya 	= NULL;
+	char    *placeholder;
+
+	GET_STR_ARG(var, input);
+
+	if (input && *input)
+		RETURN_STR(var);
+
+	value = get_variable(var);
+
+	if (!value && !*value)
+		RETURN_EMPTY;
+
+	placeholder = value;
+	booya = m_strdup(new_next_arg(value, &value));
+	if (var)
+		add_var_alias(var, value, 0);
+	new_free(&placeholder);
+	if (!booya)
+		RETURN_EMPTY;
+	return booya;
+}
+
+BUILT_IN_FUNCTION(function_unshift)
+{
+	char    *value = NULL;
+	char    *var    = NULL;
+	char	*booya  = NULL;
+
+	GET_STR_ARG(var, input);
+	value = get_variable(var);
+	if (!input || !*input)
+		return value;
+
+	booya = m_strdup(input);
+	m_s3cat_s(&booya, space, value);
+
+	add_var_alias(var, booya, 0);
+	new_free(&value);
+	return booya;
+}
+
+BUILT_IN_FUNCTION(function_push)
+{
+	char    *value = NULL;
+	char    *var    = NULL;
+
+	GET_STR_ARG(var, input);
+	upper(var);
+	value = get_variable(var);
+	m_s3cat(&value, space, input);
+	add_var_alias(var, value, 0);
+	return value;
+}
+
+BUILT_IN_FUNCTION(function_pop)
+{
+	char *value	= NULL;
+	char *var	= NULL;
+	char *pointer	= NULL;
+	char *blech     = NULL;
+
+	GET_STR_ARG(var, input);
+
+	if (input && *input)
+	{
+		pointer = strrchr(input, ' ');
+		RETURN_STR(pointer ? pointer : input);
+	}
+
+	upper(var);
+	value = get_variable(var);
+	if (!value || !*value)
+	{
+		new_free(&value);
+		RETURN_EMPTY;
+	}
+
+	if (!(pointer = strrchr(value, ' ')))
+	{
+		add_var_alias(var, empty_string, 0); /* dont forget this! */
+		return value;	/* one input -- return it */
+	}
+
+	*pointer++ = '\0';
+	add_var_alias(var, value, 0);
+
+	/* 
+	 * because pointer points to value, we *must* make a copy of it
+	 * *before* we free value! (And we cant forget to free value, either)
+	 */
+	blech = m_strdup(pointer);
+	new_free(&value);
+	return blech;
+}
+
+/*
+ * To save time, try to precalc the size of the output buffer.
+ * jfn 03/18/98
+ */
+BUILT_IN_FUNCTION(function_jot)
+{
+	int     start 	= 0;
+	int     stop 	= 0;
+	int     interval = 1;
+	int     counter;
+	char	*booya 	= NULL;
+	int	range;
+	size_t	size;
+
+        GET_INT_ARG(start,input)
+        GET_INT_ARG(stop, input)
+        if (input && *input)
+                GET_INT_ARG(interval, input)
+        else
+                interval = 1;
+
+	if (interval == 0)
+		RETURN_EMPTY;
+        if (interval < 0) 
+                interval = -interval;
+
+	range = abs(stop - start) + 1;
+	size = range * 10;
+	booya = new_malloc(size);	/* Blah. This BETTER be enough */
+
+        if (start < stop)
+	{
+		strlcpy(booya, ltoa(start), size);
+		for (counter = start + interval; 
+		     counter <= stop; 
+		     counter += interval)
+		{
+			strlcat(booya, space, size);
+			strlcat(booya, ltoa(counter), size);
+		}
+	}
+        else
+	{
+		strlcpy(booya, ltoa(start), size);
+		for (counter = start - interval; 
+		     counter >= stop; 
+		     counter -= interval)
+		{
+			strlcat(booya, space, size);
+			strlcat(booya, ltoa(counter), size);
+		}
+	}
+
+	return booya;
+}
+
+BUILT_IN_FUNCTION(function_isalpha)
+{
+	if (isalpha(*input))
+		RETURN_INT(1);
+	else
+		RETURN_INT(0);
+}
+
+BUILT_IN_FUNCTION(function_isdigit)
+{
+	if (isdigit(*input))
+		RETURN_INT(1);
+	else
+		RETURN_INT(0);
+}
+
+BUILT_IN_FUNCTION(function_isalnum)
+{
+	if (isalpha(*input) || isdigit(*input))
+		RETURN_INT(1);
+	else
+		RETURN_INT(0);
+}
+
+BUILT_IN_FUNCTION(function_isspace)
+{
+	if (isspace(*input))
+		RETURN_INT(1);
+	else
+		RETURN_INT(0);
+}
+
+BUILT_IN_FUNCTION(function_isxdigit)
+{
+	#define ishex(x) \
+		((x >= 'A' && x <= 'F') || (x >= 'a' && x <= 'f'))
+
+	if (isdigit(*input) || ishex(*input))
+		RETURN_INT(1);
+	else
+		RETURN_INT(0);	
+}
+
+BUILT_IN_FUNCTION(function_findw)
+{
+	char	*word, *this_word;
+	int	word_cnt = 0;
+
+	GET_STR_ARG(word, input);
+
+	while (input && *input)
+	{
+		GET_STR_ARG(this_word, input);
+		if (!my_stricmp(this_word, word))
+			RETURN_INT(word_cnt);
+
+		word_cnt++;
+	}
+
+	RETURN_INT(-1);
+}
+
+/*
+ * $rest(index string)
+ * Returns 'string' starting at the 'index'th character
+ * Just like $restw() does for words.
+ */
+BUILT_IN_FUNCTION(function_rest)
+{
+	int	start = 1;
+
+	if (my_atol(input))
+		GET_INT_ARG(start, input);
+
+	if (start <= 0)
+		RETURN_STR(input);
+
+	if (start >= strlen(input))
+		RETURN_EMPTY;
+
+	RETURN_STR(input + start);
+}
+
+/* $restw(num string of text)
+ * returns "string of text" that occurs starting with and including
+ * word number "num"
+ * NOTE: the first word is numbered 0.
+ * EX: $restw(3 now is the time for) returns "time for"
+ */
+BUILT_IN_FUNCTION(function_restw)
+{
+	int     where;
+	
+	GET_INT_ARG(where, input);
+	if (where < 0)
+		RETURN_EMPTY;
+	return extract(input, where, EOS);
+}
+
+/* $remw(word string of text)
+ * returns "string of text" with the word "word" removed
+ * EX: $remw(the now is the time for) returns "now is time for"
+ */
+BUILT_IN_FUNCTION(function_remw)
+{
+	char 	*word_to_remove;
+	int	len;
+	char	*str;
+
+	GET_STR_ARG(word_to_remove, input);
+	len = strlen(word_to_remove);
+
+	str = stristr(input, word_to_remove);
+	for (; str && *str; str = stristr(str + 1, word_to_remove))
+	{
+		if (str == input || isspace(str[-1]))
+		{
+			if (!str[len] || isspace(str[len]))
+			{
+				if (!str[len])
+				{
+					if (str != input)
+						str--;
+					*str = 0;
+				}
+				else if (str > input)
+				{
+					char *safe = (char *)alloca(strlen(str));
+					strcpy(safe, str + len);
+					strcpy(str - 1, safe);
+				}
+				else 
+				{
+					char *safe = (char *)alloca(strlen(str));
+					strcpy(safe, str + len + 1);
+					strcpy(str, safe);
+				}
+				break;
+			}
+		}
+	}
+
+	RETURN_STR(input);
+}
+
+/* $insertw(num word string of text)
+ * returns "string of text" such that "word" is the "num"th word
+ * in the string.
+ * NOTE: the first word is numbered 0.
+ * EX: $insertw(3 foo now is the time for) returns "now is the foo time for"
+ */
+BUILT_IN_FUNCTION(function_insertw)
+{
+	int     where;
+	char    *what;
+	char    *booya= NULL;
+	char 	*str1, *str2;
+
+	GET_INT_ARG(where, input);
+	
+	/* If the word goes at the front of the string, then it
+	   already is: return it. ;-) */
+	if (where < 1)
+		booya = m_strdup(input);
+	else
+	{
+		GET_STR_ARG(what, input);
+		str1 = extract(input, 0, (where - 1));
+		str2 = extract(input, where, EOS);
+		booya = m_strdup(str1);
+		if (str1 && *str1)
+			booya = m_2dup(str1, space);
+		malloc_strcat(&booya, what);
+		m_s3cat_s(&booya, space, str2);
+		new_free(&str1);
+		new_free(&str2);
+	}
+
+	return booya;				/* DONT USE RETURN_STR HERE! */
+}
+
+/* $chngw(num word string of text)
+ * returns "string of text" such that the "num"th word is removed 
+ * and replaced by "word"
+ * NOTE: the first word is numbered 0
+ * EX: $chngw(3 foo now is the time for) returns "now is the foo for"
+ */
+BUILT_IN_FUNCTION(function_chngw)
+{
+	int     which;
+	char    *what;
+	char    *booya= NULL;
+	char	*str1, *str2;
+	
+	GET_INT_ARG(which, input);
+	GET_STR_ARG(what, input);
+
+	if (which < 0)
+		RETURN_STR(input);
+
+	/* hmmm. if which is 0, extract does the wrong thing. */
+	str1 = extract(input, 0, which - 1);
+	str2 = extract(input, which + 1, EOS);
+	booya = m_strdup(str1);
+	if (str1 && *str1)
+		booya = m_2dup(str1, space);
+	malloc_strcat(&booya, what);
+	m_s3cat_s(&booya, space, str2);
+	new_free(&str1);
+	new_free(&str2);
+
+	return (booya);
+}
+
+
+/* $common (string of text / string of text)
+ * Given two sets of words seperated by a forward-slash '/', returns
+ * all words that are found in both sets.
+ * EX: $common(one two three / buckle my two shoe one) returns "one two"
+ * NOTE: returned in order found in first string.
+ */
+BUILT_IN_FUNCTION(function_common)
+{
+	char    *left = NULL, **leftw = NULL,
+		*right = NULL, **rightw = NULL,	*booya = NULL;
+	int	leftc, lefti;
+	int	rightc, righti;
+	
+	left = input;
+	if ((right = strchr(input,'/')) == NULL)
+		RETURN_EMPTY;
+
+ 	*right++ = 0;
+	leftc = splitw(left, &leftw);
+	rightc = splitw(right, &rightw);
+
+	for (lefti = 0; lefti < leftc; lefti++)
+	{
+		for (righti = 0; righti < rightc; righti++)
+		{
+			if (rightw[righti] && !my_stricmp(leftw[lefti], rightw[righti]))
+			{
+				m_s3cat(&booya, space, leftw[lefti]);
+				rightw[righti] = NULL;
+			}
+		}
+	}
+	new_free((char **)&leftw);
+	new_free((char **)&rightw);
+	if (!booya)
+		RETURN_EMPTY;
+
+	return (booya);				/* DONT USE RETURN_STR HERE! */
+}
+
+/* $diff(string of text / string of text)
+ * given two sets of words, seperated by a forward-slash '/', returns
+ * all words that are not found in both sets
+ * EX: $diff(one two three / buckle my two shoe)
+ * returns "one two three buckle my shoe"
+ */
+BUILT_IN_FUNCTION(function_diff)
+{
+	char 	*left = NULL, **leftw = NULL,
+	     	*right = NULL, **rightw = NULL, *booya = NULL;
+	int 	lefti, leftc,
+	    	righti, rightc;
+	int 	found;
+
+	left = input;
+	if ((right = strchr(input,'/')) == NULL)
+		RETURN_EMPTY;
+
+	*right++ = 0;
+	leftc = splitw(left, &leftw);
+	rightc = splitw(right, &rightw);
+
+	for (lefti = 0; lefti < leftc; lefti++)
+	{
+		found = 0;
+		for (righti = 0; righti < rightc; righti++)
+		{
+			if (rightw[righti] && !my_stricmp(leftw[lefti], rightw[righti]))
+			{
+				found = 1;
+				rightw[righti] = NULL;
+			}
+		}
+		if (!found)
+			m_s3cat(&booya, space, leftw[lefti]);
+	}
+
+	for (righti = 0; righti < rightc; righti++)
+	{
+		if (rightw[righti])
+			m_s3cat(&booya, space, rightw[righti]);
+	}
+
+	new_free((char **)&leftw);
+	new_free((char **)&rightw);
+
+	if (!booya)
+		RETURN_EMPTY;
+
+	return (booya);
+}
+
+/*
+ * $remws(word word word / word word word)
+ * Returns the right hand side unchanged, except that any word on the right
+ * hand side that is also found on the left hand side will be removed.
+ * Space is *not* retained.  So there.
+ */
+BUILT_IN_FUNCTION(function_remws)
+{
+	char    *left = NULL,
+		**lhs = NULL,
+		*right = NULL,
+		**rhs = NULL, 
+		*booya = NULL;
+	int	leftc, 	
+		lefti,
+		rightc,
+		righti;
+	int	found = 0;
+
+	left = input;
+	if (!(right = strchr(input,'/')))
+	RETURN_EMPTY;
+
+	*right++ = 0;
+	leftc = splitw(left, &lhs);
+	rightc = splitw(right, &rhs);
+
+	for (righti = 0; righti < rightc; righti++)
+	{
+		found = 0;
+		for (lefti = 0; lefti < leftc; lefti++)
+		{
+			if (rhs[righti] && lhs[lefti]  &&
+			    !my_stricmp(lhs[lefti], rhs[righti]))
+			{
+				found = 1;
+				break;
+			}
+		}
+		if (!found)
+			m_s3cat(&booya, space, rhs[righti]);
+		rhs[righti] = NULL;
+	}
+
+	new_free((char **)&lhs);
+	new_free((char **)&rhs);
+
+	if (!booya)
+		RETURN_EMPTY;
+
+	return (booya);				/* DONT USE RETURN_STR HERE! */
+}
+
+
+BUILT_IN_FUNCTION(function_status)
+{
+	int window_refnum;
+	int status_line;
+
+	GET_INT_ARG(window_refnum, input);
+	GET_INT_ARG(status_line, input);
+	RETURN_STR(get_status_by_refnum(window_refnum, status_line));
+}
+
+/*
+ * $crypt(password seed)
+ * What it does: Returns a 13-char encrypted string when given a seed and
+ *    password. Returns zero (0) if one or both args missing. Additional
+ *    args ignored.
+ * Caveats: Password truncated to 8 chars. Spaces allowed, but password
+ *    must be inside "" quotes.
+ * Credits: Thanks to Strongbow for showing me how crypt() works.
+ * This cheap hack by: CrowMan
+ */
+BUILT_IN_FUNCTION(function_crypt)
+{
+#if defined(WINNT)
+	RETURN_STR(empty_string);
+#else
+        char pass[9] = "\0";
+        char seed[3] = "\0";
+        char *blah, *bleh, *crypt (const char *, const char *);
+
+	GET_STR_ARG(blah, input)
+	GET_STR_ARG(bleh, input)
+	strmcpy(pass, blah, 8);
+	strmcpy(seed, bleh, 2);
+
+	RETURN_STR(crypt(pass, seed));
+#endif
+}
+
+BUILT_IN_FUNCTION(alias_idle)
+{
+	return m_sprintf("%ld", now - idle_time);
+}
+
+BUILT_IN_FUNCTION(alias_online)
+{
+	return m_sprintf("%ld", start_time);
+}
+
+BUILT_IN_FUNCTION(alias_sent_nick)
+{
+	return m_strdup(sent_nick ? sent_nick : empty_string);
+}
+
+BUILT_IN_FUNCTION(alias_recv_nick)
+{
+	return m_strdup(recv_nick ? recv_nick : empty_string);
+}
+
+BUILT_IN_FUNCTION(function_chmod)
+{
+	char 	*filearg, 
+		*after;
+	int 	fd = -1;
+	char 	*perm_s;
+	mode_t 	perm;
+	char 	*expanded;
+	int	retval;
+
+	GET_STR_ARG(filearg, input);
+	fd = (int) strtoul(filearg, &after, 0);
+
+	GET_STR_ARG(perm_s, input);
+	perm = (mode_t) strtol(perm_s, &perm_s, 8);
+
+	if (after != input && *after == 0)
+	{
+		if (file_valid(fd))
+			RETURN_INT(fchmod(fd, perm));
+		else
+			RETURN_EMPTY;
+	}
+	else
+	{
+		expanded = expand_twiddle(filearg);
+		retval = chmod(expanded, perm);
+		new_free(&expanded);
+		RETURN_INT(retval);
+	}
+}
+
