@@ -37,7 +37,7 @@
  */
 
 /* from: static char sccsid[] = "@(#)glob.c	8.3 (Berkeley) 10/13/93"; */
-/* $Id: glob.c,v 1.1.1.1 2000/06/22 09:37:24 edwards Exp $ */
+/* $Id: glob.c,v 1.2 2000/08/14 12:27:27 cvs Exp $ */
 
 /*
  * glob(3) -- a superset of the one defined in POSIX 1003.2.
@@ -530,14 +530,14 @@ static int glob2		(	Char *pathbuf,
 				return(0);
 		
 			if (((pglob->gl_flags & GLOB_MARK) &&
-			    pathend[-1] != SEP) && (S_ISDIR(sb.st_mode)
+			    (pathend[-1] != SEP) && S_ISDIR(sb.st_mode))
 #ifdef S_ISLNK
 			    || ((pglob->gl_flags & GLOB_LINK) &&
 			    S_ISLNK(sb.st_mode) &&
 			    (g_stat(pathbuf, &sb, pglob) == 0) &&
 			    S_ISDIR(sb.st_mode))
 #endif
-			    )) 
+			    ) 
 			{
 				*pathend++ = SEP;
 				*pathend = EOS;
