@@ -8,7 +8,7 @@
  * Copyright(c) 1992 
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
- * $Id: help.c,v 1.5 2000/07/01 14:03:03 edwards Exp $
+ * $Id: help.c,v 1.7 2000/07/16 13:24:35 edwards Exp $
  */
 
 /*
@@ -375,6 +375,9 @@ static	void	help_me (char *topics, char *args)
 
 	create_help_window();
 
+	if (!help_window)
+		return;
+
 	/*
 	 * This is just a bogus while loop which is intended to allow
 	 * the user to do '/help alias expressions' without having to
@@ -459,7 +462,7 @@ static	void	help_me (char *topics, char *args)
 			int len2 = strlen(str2);
 
 
-			     if (len1 == len2 && !my_stricmp(str1, str2))
+			if (len1 == len2 && !my_stricmp(str1, str2))
 				entries = 1;
 			else if (len1 - 3 == len2 && !my_strnicmp(str1, str2, len2) && !end_strcmp(str1, ".gz", 3))
 				entries = 1;
@@ -561,6 +564,8 @@ switch (entries)
 
 		for (i = 0; i < entries; i++)
 		{
+			if (!strcmp(g.gl_pathv[i], "CVS"))
+				continue;
 			if (!end_strcmp(g.gl_pathv[i], ".gz", 3))
 				chop(g.gl_pathv[i], 3);
 			else if (!end_strcmp(g.gl_pathv[i], ".bz2", 4))
@@ -678,9 +683,15 @@ static	void create_help_window (void)
 
 	if (get_int_var(HELP_WINDOW_VAR))
 	{
+		if (!(help_window = new_window(current_window->screen)))
+		{
+			say("Could not create a new window, maybe you have "
+				"all your windows fixed?");
+			return;
+		}
 		use_help_window = 1;
-		help_window = new_window(current_window->screen);
 		help_window->hold_mode = OFF;
+/*		help_window->window_level = LOG_HELP;*/
 		update_all_windows();
 	}
 	else
