@@ -5,7 +5,7 @@
  * Copyright 1994 Matthew Green
  * Copyright 1997 EPIC Software Labs
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
- * $Id: hook.c,v 1.6 2000/06/28 09:11:40 edwards Exp $
+ * $Id: hook.c,v 1.12 2000/08/27 23:52:23 cvs Exp $
  */
 
 #include "teknap.h"
@@ -15,6 +15,7 @@
 #include "ircaux.h"
 #include "window.h"
 #include "output.h"
+#include "stack.h"
 
 /*
  * The various ON levels: SILENT means the DISPLAY will be OFF and it will
@@ -45,6 +46,7 @@ struct	numericlist_stru *next;
 
 NumericList *numeric_list 	= NULL;
 
+#define MAXHOOK 11000
 
 extern char *next_expr(char **, char);
 extern int will_catch_return_exceptions;
@@ -66,13 +68,12 @@ hook_functions[] =
 	{ "BROWSE_BEGIN",	(Hook *) 0,	1,	0,	0 },
 	{ "BROWSE_END",		(Hook *) 0,	1,	0,	0 },
 	{ "CHANNEL_JOIN",	(Hook *) 0,	1,	0,	0 },
-	{ "CHANNEL_NICK",	(Hook *) 0,	3,	0,	0 },
 	{ "CONNECT",		(Hook *) 0,	2,	0,	0 },
 	{ "DISCONNECT",		(Hook *) 0,	1,	0,	0 },
 	{ "ERROR",		(Hook *) 0,	0,	0,	0 },
 	{ "EXEC",		(Hook *) 0,	2,	0,	0 },
 	{ "EXEC_ERRORS",	(Hook *) 0,	2,	0,	0 },
-	{ "EXEC_EXIT",		(Hook *) 0,	3,	0,	0 },
+	{ "EXEC_EXIT",		(Hook *) 0,	1,	0,	0 },
 	{ "EXEC_PROMPT",	(Hook *) 0,	2,	0,	0 },
 	{ "EXIT",		(Hook *) 0,	1,	0,	0 },
 	{ "FLOOD",		(Hook *) 0,	2,	0,	0 },
@@ -85,24 +86,28 @@ hook_functions[] =
 	{ "KILL",		(Hook *) 0,	5,	0,	0 },
 	{ "LEAVE",		(Hook *) 0,	2,	0,	0 },
 	{ "LIST",		(Hook *) 0,	3,	0,	0 },
+	{ "MP3",		(Hook *) 0,	1,	0,	0 },
 	{ "MSG",		(Hook *) 0,	2,	0,	0 },
 	{ "NAMES",		(Hook *) 0,	1,	0,	0 },
+	{ "NAP",		(Hook *) 0,	1,	0,	0 },
 	{ "NAPERROR",		(Hook *) 0,	1,	0,	0 },
 	{ "NAPFINISH",		(Hook *) 0,	1,	0, 	0 },
 	{ "NAPGET",		(Hook *) 0,	1,	0,	0 },
 	{ "NAPREQUEST",		(Hook *) 0,	1,	0,	0 },
 	{ "NAPSEND",		(Hook *) 0,	1,	0,	0 },
+	{ "PASTE",		(Hook *) 0,	2,	0,	0 },
 	{ "PONG",		(Hook *) 0,	1,	0,	0 },
 	{ "PUBLIC",		(Hook *) 0,	3,	0,	0 },
 	{ "PUBLIC_OTHER",	(Hook *) 0,	3,	0,	0 },
 	{ "RAW_NAP",		(Hook *) 0,	1,	0,	0 },
-	{ "REDIRECT",		(Hook *) 0,	2,	0,	HF_NORECURSE },
 	{ "SEARCH_BEGIN",	(Hook *) 0,	1,	0,	0 },
 	{ "SEARCH_END",		(Hook *) 0,	1,	0,	0 },
+	{ "SEND_ACTION",	(Hook *) 0,	2,	0,	0 },
 	{ "SEND_MSG",		(Hook *) 0,	2,	0,	HF_NORECURSE },
 	{ "SEND_OPS",		(Hook *) 0,	1,	0,	0 },
 	{ "SEND_PUBLIC",	(Hook *) 0,	2,	0,	HF_NORECURSE },
 	{ "SERVERMSG",		(Hook *) 0,	1,	0,	0 },
+	{ "SIGUSR1",		(Hook *) 0,	1,	0,	HF_NORECURSE },
 	{ "STATUS_UPDATE",	(Hook *) 0,	3,	0,	0 },
 	{ "SWITCH_CHANNELS",	(Hook *) 0,	3,	0,	0 },
 	{ "SWITCH_WINDOWS",	(Hook *) 0,	4,	0,	0 },
@@ -211,9 +216,9 @@ static int 	find_hook (char *name, int *first)
 		{
 			which = atol(name);
 
-			if ((which < 0) || (which > 11000))
+			if ((which < 0) || (which > MAXHOOK))
 			{
-				say("Numerics must be between 001 and 11000");
+				say("Numerics must be between 001 and %d", MAXHOOK);
 				return INVALID_HOOKNUM;
 			}
 			which = -which;
@@ -457,7 +462,7 @@ void    flush_on_hooks (void)
         int old_display = window_display;
         
         window_display = 0;
-        for (x = 1; x < 11000; x++)
+        for (x = 1; x < MAXHOOK; x++)
 		remove_numeric_hook(x, NULL, x, 1);
         for (x = 0; x < NUMBER_OF_LISTS; x++)
 		remove_hook(x, NULL, 0, 1); 
@@ -1227,6 +1232,146 @@ void	save_hooks (FILE *fp, int do_all)
 }
 
 
+/* * * * * * * * * * STACKING A HOOK * * * * * * * * */
+typedef struct  onstacklist
+{
+	int     which;
+	Hook    *list;
+	struct onstacklist *next;
+}       OnStack;
+
+static	OnStack	*	on_stack = NULL;
+
+void	do_stack_on (int type, char *args)
+{
+	int	which;
+	Hook	*list;
+	NumericList *nhook = NULL, *nptr;
+
+	if (!on_stack && (type == STACK_POP || type == STACK_LIST))
+	{
+		say("ON stack is empty!");
+		return;
+	}
+	if (!args || !*args)
+	{
+		say("Missing event type for STACK ON");
+		return;
+	}
+
+	if ((which = find_hook(args, NULL)) == INVALID_HOOKNUM)
+		return;		/* Error message already outputted */
+
+	if (which < 0)
+	{
+		if ((nhook = find_numeric_list(-which)))
+			list = nhook->list;
+		else
+			list = NULL;
+	}
+	else
+		list = hook_functions[which].list;
+
+
+	if (type == STACK_PUSH)
+	{
+		OnStack	*new_os;
+		new_os = (OnStack *) new_malloc(sizeof(OnStack));
+		new_os->which = which;
+		new_os->list = list;
+		new_os->next = on_stack;
+		on_stack = new_os;
+
+		if (which < 0)
+		{
+			if (nhook && numeric_list)
+				remove_numeric_list(-which);
+		}
+		else
+			hook_functions[which].list = NULL;
+
+		return;
+	}
+
+	else if (type == STACK_POP)
+	{
+		OnStack	*p, *tmp = NULL;
+
+		for (p = on_stack; p; tmp = p, p = p->next)
+		{
+			if (p->which == which)
+			{
+				if (p == on_stack)
+					on_stack = p->next;
+				else
+					tmp->next = p->next;
+				break;
+			}
+		}
+		if (!p)
+		{
+			say("No %s on the stack", args);
+			return;
+		}
+
+		if ((which < 0 && nhook) || 
+		    (which >= 0 && hook_functions[which].list))
+			remove_hook(which, NULL, 0, 1);	/* free hooks */
+
+		if (which < 0)
+		{
+			/* look -- do we have any hooks already for this numeric? */
+			if (!(nptr = find_numeric_list(-which)))
+			{
+				if (p->list)	/* not just a placeholder? */
+				{
+					/* No. make a new list and put the stack on it */
+					nptr = (NumericList *) new_malloc(sizeof(NumericList));
+					nptr->list = p->list;
+					nptr->next = NULL;
+					nptr->numeric = -which;
+					sprintf(nptr->name, "%3.3u", -which);
+					add_numeric_list(nptr);
+				}
+			}
+			else
+			{
+				remove_hook(which, NULL, 0, 1);
+				nptr->list = p->list;
+			}
+		}
+		else
+			hook_functions[which].list = p->list;
+
+		new_free((char **)&p);
+		return;
+	}
+
+	else if (type == STACK_LIST)
+	{
+		int	slevel = 0;
+		OnStack	*osptr;
+
+		for (osptr = on_stack; osptr; osptr = osptr->next)
+		{
+			if (osptr->which == which)
+			{
+				Hook	*hptr;
+
+				slevel++;
+				say("Level %d stack", slevel);
+				for (hptr = osptr->list; hptr; hptr = hptr->next)
+					show_hook(hptr, args);
+			}
+		}
+
+		if (!slevel)
+			say("The STACK ON %s list is empty", args);
+		return;
+	}
+	say("Unknown STACK ON type ??");
+}
+
 
 /* List manips especially for on's. */
 static void 	add_to_list (Hook **list, Hook *item)
