@@ -8,7 +8,7 @@
  * Copyright(c) 1990 
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
- * $Id: input.c,v 1.4 2000/07/01 14:03:04 edwards Exp $
+ * $Id: input.c,v 1.8 2000/08/27 06:15:45 cvs Exp $
  */
 
 #include "teknap.h"
@@ -553,7 +553,11 @@ int	pos;
 	else
 	{
 		term_delete(1);
-		pos = INPUT_ONSCREEN + last_input_screen->co - 1;
+		if (INPUT_ONSCREEN == 0)		/* UGH! */
+			pos = last_input_screen->co - INPUT_PROMPT_LEN - 1;
+		else
+			pos = INPUT_ONSCREEN + last_input_screen->co - 1;
+
 		if (pos < strlen(INPUT_BUFFER))
 		{
 			term_move_cursor(last_input_screen->co - 1, INPUT_LINE);
@@ -1374,12 +1378,17 @@ BUILT_IN_COMMAND(typecmd)
 	{
 		if (*args == '^')
 		{
-			args++;
-			if (*args == '?')
-				key = '\177';
-			else if (*args)
+			switch (*(++args))
 			{
-				c = *args;
+			    case '?':
+			    {
+				key = '\177';
+				args++;
+				break;
+			    }
+			    default:
+			    {
+				c = *(args++);
 				if (islower(c))
 					c = toupper(c);
 				if (c < 64)
@@ -1388,19 +1397,18 @@ BUILT_IN_COMMAND(typecmd)
 					return;
 				}
 				key = c - 64;
+				break;
+			    }
 			}
-			else
-				break;
 		}
-		else 
+		else if (*args == '\\')
 		{
-			if (*args == '\\')
-				args++;
-			if (*args)
-				key = *args;
-			else
-				break;
+			key = *++args;
+			args++;
 		}
+		else
+			key = *(args++);
+
 		edit_char(key);
 	}
 }
@@ -1415,6 +1423,7 @@ struct _name_type {
 
  struct _name_type 
  	name_type[] = {	
+			{"MP3",		2},
 			{"EXEC",	4}, 
 			{"LS",		2}, 
 			{"LOAD",	4}, 
@@ -1425,7 +1434,7 @@ struct _name_type {
 			{"SHARE",	2},
 			{ "",		0}};
 
-enum tab_types { EXEC, LS, LOAD, SERVER, IMSG, CHDIR, CDCC, SHARE};
+enum tab_types { MP3_FILE, EXEC, LS, LOAD, SERVER, IMSG, CHDIR, CDCC, SHARE};
 
 char *get_home_dir(char *possible, int *count)
 {
@@ -1823,6 +1832,7 @@ char *possible = NULL, *old_pos = NULL;
 char *cmdchar;
 char *suggested = NULL;
 int got_space = 0;
+int got_comm = 0;
 char *get = NULL;
 
 	/* 
@@ -1839,6 +1849,9 @@ char *get = NULL;
 	wcount = word_count(inp);
 	if (!(cmdchar = get_string_var(CMDCHARS_VAR)))
 		cmdchar = "/";
+	if (*inp && (*inp == *cmdchar))
+		got_comm = 1;
+
 	switch(wcount)
 	{
 		case 0:
@@ -1892,6 +1905,7 @@ char *get = NULL;
 						case EXEC:
 							type = EXEC_COMPLETION;
 							break;
+						case MP3_FILE:
 						case CHDIR:
 						case LS:
 						case SHARE:
@@ -1985,7 +1999,7 @@ do_more_tab:
 					n = new_next_arg(use, &use);
 					while (n && *n)
 					{
-						put_it("%s", n);
+						put_it("%s", base_name(n));
 						completes[c++] = n;
 						n = new_next_arg(use, &use);
 					}
