@@ -6,7 +6,7 @@
  * Copyright(c) 1990, 1991 
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
- * $Id: ircaux.c,v 1.5 2000/07/01 14:03:04 edwards Exp $
+ * $Id: ircaux.c,v 1.8 2000/08/03 21:09:15 cvs Exp $
  */
 #include "teknap.h"
 #include "struct.h"
@@ -322,6 +322,9 @@ char	*rstristr (char *source, char *search)
 	return NULL;
 }
 
+#define risspace(c) (c == ' ')
+
+#if 0
 /* 
  * word_count:  Efficient way to find out how many words are in
  * a given string.  Relies on isspace() not being broken.
@@ -353,6 +356,58 @@ register char *foo = str;
         }
         return (cocs + 1) / 2;
 }
+#else
+int	word_count (const char *ptr)
+{
+	int	count = 0;
+	if (!ptr || !*ptr)
+		return 0;
+
+	/* Skip any leading whitespace */
+	while (*ptr && risspace(*ptr))
+		ptr++;
+
+ 	while (ptr && *ptr)
+	{
+		/* Always pre-count words */
+		count++;
+
+		/* 
+		 * If this is an extended word, then skip everything
+		 * up to the first un-backslashed double quote.
+		 */
+		if (*ptr == '"')
+		{
+			for (ptr++; *ptr; ptr++)
+			{
+				if (*ptr == '\\' && ptr[1])
+					ptr++;
+				else if (*ptr == '"')
+				{
+					ptr++;
+					break;
+				}
+			}
+		}
+
+		/* 
+		 * This is a regular word, skip all of the non-whitespace
+		 * characters.
+		 */
+		else
+		{
+			while (*ptr && !risspace(*ptr))
+				ptr++;
+		}
+
+		/* Skip any leading whitespace before the next word */
+		while (*ptr && risspace(*ptr))
+			ptr++;
+	}
+
+	return count;
+}
+#endif
 
 #if 0
 extern  int     word_scount (char *str)
@@ -483,7 +538,7 @@ char *last_arg (char **src)
 
 }
 
-#define risspace(c) (c == ' ')
+
 char	*new_next_arg (char *str, char **new_ptr)
 {
 	char	*ptr,
@@ -726,7 +781,7 @@ char *chop (char *stuff, int nchar)
  * the original string with a null.  end must be one place beyond where
  * you want to copy, ie, its the first character you dont want to copy.
  */
-char *strext(char *start, char *end)
+char *strext(const char *start, const char *end)
 {
 	char *ptr, *retval;
 
@@ -2403,6 +2458,27 @@ int i = 0, j = 0, len;
 	return buff;
 }
 
+char *slongcomma(char *val)
+{
+static char buff[80];
+char *s = buff;
+int i = 0, j = 0, len;
+	len = strlen(val);
+	for (i = len % 3; i > 0; i--)
+		*s++ = val[j++];	 
+	if (len > 3 && len % 3)
+		*s++ = ',';
+	len -= (len % 3);	
+	while (len --)
+	{
+		*s++ = val[j++];
+		if (!(len % 3) && len)
+			*s++ = ',';
+	}
+	*s = 0;
+	return buff;
+}
+
 /* RANDOM NUMBERS */
 /*
  * Random number generator #1 -- psuedo-random sequence
@@ -2813,6 +2889,167 @@ char *remove_brackets (const char *name, const char *args, int *arg_flag)
 }
 
 
+/* is_number: returns 1 if the given string is a real number, 0 otherwise */
+int	is_real_number (const char *str)
+{
+	int	dot = 0;
+
+	if (!str || !*str)
+		return 0;
+
+	while (*str == ' ')
+		str++;
+
+	if (*str == '-')
+		str++;
+
+	if (!*str)
+		return 0;
+
+	for (; *str; str++)
+	{
+		if (isdigit((*str)))
+			continue;
+
+		if (*str == '.' && dot == 0)
+		{
+			dot = 1;
+			continue;
+		}
+
+		return 0;
+	}
+
+	return 1;
+}
+
+/*
+ * A "forward quote" is the first double quote we find that has a space
+ * or the end of string after it; or the end of the string.
+ */
+const char *	find_forward_quote (const char *input, const char *start)
+{
+	/*
+	 * An "extended word" is defined as:
+	 *	<SPACE> <QUOTE> <ANY>* <QUOTE> <SPACE>
+	 * <SPACE> := <WORD START> | <WORD END> | <" "> | <"\t"> | <"\n">
+	 * <QUOTE> :- <'"'>
+	 * <ANY>   := ANY ASCII CHAR 0 .. 256
+	 */
+	/* Make sure we are actually looking at a double-quote */
+	if (*input != '"')
+		return NULL;
+
+	/* 
+	 * Make sure that the character before is the start of the
+	 * string or that it is a space.
+	 */
+	if (input > start && !risspace(input[-1]))
+		return NULL;
+
+	/*
+	 * Walk the string looking for a double quote.  Yes, we do 
+	 * really have to check for \'s, still, because the following
+	 * still is one word:
+	 *			"one\" two"
+	 * Once we find a double quote, then it must be followed by 
+	 * either the end of string (chr 0) or a space.  If we find 
+	 * that, return the position of the double-quote.  
+	 */
+	for (input++; *input; input++)
+	{
+		if (input[0] == '\\' && input[1])
+			input++;
+		else if (input[0] == '"')
+		{
+			if (input[1] == 0)
+				return input;
+			else if (risspace(input[1]))
+				return input;
+		}
+	}
+
+	/*
+	 * If we get all the way to the end of the string w/o finding 
+	 * a matching double-quote, return the position of the (chr 0), 
+	 * which is a special meaning to the caller. 
+	 */
+	return input;		/* Bumped the end of string. doh! */
+}
+
+/*
+ * A "backward quote" is the first double quote we find, going backward,
+ * that has a space or the start of string after it; or the start of 
+ * the string.
+ */
+const char *	find_backward_quote (const char *input, const char *start)
+{
+	const char *	saved_input = input;
+
+	/*
+	 * An "extended word" is defined as:
+	 *	<SPACE> <QUOTE> <ANY>* <QUOTE> <SPACE>
+	 * <SPACE> := <WORD START> | <WORD END> | <" "> | <"\t"> | <"\n">
+	 * <QUOTE> :- <'"'>
+	 * <ANY>   := ANY ASCII CHAR 0 .. 256
+	 */
+	/* Make sure we are actually looking at a double-quote */
+	if (input[0] != '"')
+		return NULL;
+
+	/* 
+	 * Make sure that the character after is either the end of the
+	 * string, or that it is a space.
+	 */
+	if (input[1] && !risspace(input[1]))
+		return NULL;
+
+	/*
+	 * Walk the string looking for a double quote.  Yes, we do 
+	 * really have to check for \'s, still, because the following
+	 * still is one word:
+	 *			"one\" two"
+	 * Once we find a double quote, then it must be followed by 
+	 * either the end of string (chr 0) or a space.  If we find 
+	 * that, return the position of the double-quote.  
+	 */
+	for (input--; input > start; input--)
+	{
+		if (input[0] == '\\' && input[1])
+			input++;
+		else if (input[0] == '"')
+		{
+			if (input[1] == 0)
+				return input;
+			else if (risspace(input[1]))
+				return input;
+		}
+	}
+
+	/*
+	 * If we get all the way to the start of the string w/o finding 
+	 * a matching double-quote, then THIS IS NOT AN EXTENDED WORD!
+	 * We need to re-do this word entirely by starting over and looking
+	 * for a normal word.
+	 */
+	input = saved_input;
+	while (input > start && !risspace(input[0]))
+		input--;
+
+	if (risspace(input[0]))
+		input++;		/* Just in case we've gone too far */
+
+	return input;		/* Wherever we are is fine. */
+}
+
+char *off_on(int val)
+{
+static char *onoff[] = { "off", "on" };
+	if (!val)
+		return onoff[val];
+	return onoff[val];
+}
+
 #if 0
 /* #define MEM_DEBUG 1 */
 
