@@ -8,7 +8,7 @@
  * Substantial re-implementation by Jeremy Nelson
  * Copyright 1998 EPIC Software Labs
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
- * $Id: keys.c,v 1.4 2000/07/01 14:03:04 edwards Exp $ 
+ * $Id: keys.c,v 1.6 2000/08/27 06:15:45 cvs Exp $ 
  */
 
 #include "teknap.h"
@@ -170,7 +170,7 @@ static KeyMapNames 	key_names[] =
 	{ "BOLD",			insert_bold 			},
 	{ "CLEAR_SCREEN",		clear_screen 			},
 	{ "COMMAND_COMPLETION",		command_completion		},
-	{ "CPU_SAVER",			cpu_saver_on 			},
+	{ "CPUSAVER",			cpu_saver_on 			},
 	{ "DELETE_CHARACTER",		input_delete_character 		},
 	{ "DELETE_NEXT_WORD",		input_delete_next_word 		},
 	{ "DELETE_PREVIOUS_WORD",	input_delete_previous_word 	},
@@ -191,7 +191,7 @@ static KeyMapNames 	key_names[] =
 	{ "NEXT_WINDOW",		next_window 			},
 	{ "PARSE_COMMAND",		parse_text 			},
 	{ "PREVIOUS_WINDOW",		previous_window 		},
-	{ "QUIT_IRC",			irc_quit 			},
+	{ "QUIT_NAP",			nap_quit 			},
 	{ "QUOTE_CHARACTER",		quote_char 			},
 	{ "REFRESH_INPUTLINE",		refresh_inputline 		},
 	{ "REFRESH_SCREEN",		(KeyBinding) refresh_screen 	},
@@ -203,7 +203,7 @@ static KeyMapNames 	key_names[] =
 	{ "SCROLL_START",		scrollback_start 		},
 	{ "SEND_LINE",			send_line 			},
 	{ "SHOVE_TO_HISTORY",		shove_to_history 		},
-	{ "STOP_IRC",			term_pause 			},
+	{ "STOP_NAP",			term_pause 			},
 	{ "SWAP_LAST_WINDOW",		swap_last_window 		},
 	{ "SWAP_NEXT_WINDOW",		swap_next_window 		},
 	{ "SWAP_PREVIOUS_WINDOW",	swap_previous_window 		},
@@ -613,7 +613,7 @@ static void	show_all_bindings (int meta)
 		return;
 	}
 
-	if (!keys[meta])
+	if (meta > MAX_META || !keys[meta])
 		return;
 
 	k = charset_size();
@@ -1259,7 +1259,7 @@ void 	init_keys (void)
 	snew_key(0, 24, "SWITCH_CHANNELS");		/* ^X */
 
 	snew_key(0, 25, "YANK_FROM_CUTBUFFER");		/* ^Y */
-	snew_key(0, 26, "STOP_IRC");			/* ^Z */
+	snew_key(0, 26, "STOP_NAP");			/* ^Z */
 	snew_key(0, 27, "META1_CHARACTER");		/* ^[ (escape) */
 	snew_key(0, 29, "AUTOREPLY");			/* ^] */
 	snew_key(0, 31, "UNDERLINE");			/* ^_ */
@@ -1311,7 +1311,7 @@ void 	init_keys (void)
 	snew_key(1, '0', "WINDOW_SWAP_10");
 
 	/* meta 2 characters */
-	snew_key(2,  26, "STOP_IRC");
+	snew_key(2,  26, "STOP_NAP");
 
 	snew_key(2, 110, "SWAP_NEXT_WINDOW");
 	snew_key(2, 112, "PREVIOUS_WINDOW");
