@@ -2,7 +2,7 @@
  * TekNap: a new napster client.
  * Copyright Colten Edwards Feb 14/2000
  *
- * $Id: main.c,v 1.12 2000/07/02 04:20:37 edwards Exp $
+ * $Id: main.c,v 1.29 2000/09/09 08:15:41 cvs Exp $
  */
 
 #include "teknap.h"
@@ -38,6 +38,9 @@
 #include "cdns.h"
 #include "ptest.h"
 #include "tgtk.h"
+#ifdef WANT_MP3PLAYER
+#include "mp3control.h"
+#endif
 
 #include <pwd.h>
 
@@ -46,11 +49,11 @@
 #endif
 
 #ifndef VERSION
-	const char nap_version[] = "TekNap-1.0";
+	const char nap_version[] = "TekNap-1.2";
 #else
 	const char nap_version[] = VERSION;
 #endif
-static char main_version[] = "$Id: main.c,v 1.12 2000/07/02 04:20:37 edwards Exp $";
+static char main_version[] = "$Id: main.c,v 1.29 2000/09/09 08:15:41 cvs Exp $";
 
 /*
  * major version is 75
@@ -67,7 +70,7 @@ static char main_version[] = "$Id: main.c,v 1.12 2000/07/02 04:20:37 edwards Exp
  * Its format is actually YYYYMMDD, for the _release_ date of the
  * client..
  */
-const char internal_version[] = "20000701";
+const char internal_version[] = "20000909";
 
 int	nap_port = 8875,			/* port of ircd */
 	use_input = 1,				/* if 0, stdin is never
@@ -113,6 +116,7 @@ char
 
 	int cpu_saver = 0;
 	int timeout_select = 60;
+	int scan_done = 0;		/* is our scan done */
 	
 time_t	idle_time = 0,
 	now = 0,
@@ -128,6 +132,9 @@ fd_set	readables, writables;
 static	void	quit_response (char *, char *);
 static	char	*parse_args (char **, int, char **);
 
+#ifdef WINNT
+char *getpass(char *);
+#endif
 static	int create = 0;
 	
 static	volatile int	cntl_c_hit = 0;
@@ -148,34 +155,51 @@ static		char	switch_help[] =
    -r filename\tuse filename for the serverlist\n\
    -v\t\ttells you about the client's version\n";
 
+#if defined(WINNT) || defined(EMX)
+static	char	switch_help_w[] = 
+"   -s\t\tsetup TekNap for use, by prompting\n";
+#endif
+
 char	*time_format = NULL;	/* XXX Bogus XXX */
 char	*strftime_24hour = "%R";
 char	*strftime_12hour = "%I:%M%p";
 char	time_str[61];
 
+volatile int update_refresh = 0;
+
+
 Stats	shared_stats = { 0 };
 
 void check_serverlag(int server)
 {
 	int i;
 	struct timeval tv;
+	int ofs = from_server;
 	get_time(&tv);
 	if (server == -1)
 	{
 		for (i = 0; i < server_list_size(); i++)
 		{
-			if (is_connected(i) && !get_server_sping(i) && (get_server_admin(i) > USER_USER))
+			if (is_connected(i) && !get_server_sping(i))
 			{
-				send_ncommand(CMDS_SPING, "%s LAG %lu %lu", get_server_name(i), tv.tv_sec, tv.tv_usec);
-				set_server_sping(i, 1);
+				from_server = i;
+				if (!get_server_ircmode(i) && (get_server_admin(i) > USER_USER))
+				{
+					send_ncommand(CMDS_SPING, "%s LAG %lu %lu", get_server_name(i), tv.tv_sec, tv.tv_usec);
+					set_server_sping(i, 1);
+				}
 			}
 		}
+		from_server = ofs;
 		return;
 	}
 	if (is_connected(server) && !get_server_sping(server) && (get_server_admin(server) > USER_USER))
 	{
-		send_ncommand(CMDS_SPING, "%s LAG %lu %lu", get_server_name(server), tv.tv_sec, tv.tv_usec);
-		set_server_sping(server, 1);
+		if (!get_server_ircmode(server))
+		{
+			send_ncommand(CMDS_SPING, "%s LAG %lu %lu", get_server_name(server), tv.tv_sec, tv.tv_usec);
+			set_server_sping(server, 1);
+		}
 	}
 }
 
@@ -251,7 +275,14 @@ void reset_clock(Window *win, char *unused, int unused1)
 /* sig_refresh_screen: the signal-callable version of refresh_screen */
 SIGNAL_HANDLER(sig_refresh_screen)
 {
-	refresh_screen(0, NULL);
+	/* We used to call refresh_screen() here directly, but we have found
+	 * that some libc implementations cannot deal with us refreshing
+	 * the window while in a signal handler, while using threads.
+	 * So now we toggle update_refresh variable, to tell our main io
+	 * loop to do the refresh. This resolves the SIGWINCH - thread
+	 * problem. -SHK
+	 */
+	update_refresh = 1;
 }
 
 SIGNAL_HANDLER(irc_exit_old)
@@ -280,24 +311,58 @@ static int sigpipe_hit = 0;
 	sigpipe_hit++;
 	
 }
+
+SIGNAL_HANDLER(sigusr1)
+{
+	do_hook(SIGUSR1_LIST, "%s", "Sig user1 signal");
+}
+
+/* Call this, when you are about to quit the client. It will kill all known threads */
+static void kill_all_threads(void)
+{
+#ifdef WANT_THREAD
+#ifdef WANT_NSLOOKUP
+	kill_dns();
+#endif
+#ifdef WANT_PTEST
+	kill_ptest();
+#endif
+#ifdef GTK
+	kill_tgtk();
+#endif
+#ifdef WANT_MP3PLAYER
+	kill_mp3();
+#endif
+#if !defined(WIN32_GUI)
+	clean_share_mutexes();
+#endif
+#endif
+}
+
           
 /* nap_exit: cleans up and leaves */
 void nap_exit (int really_quit, char *reason, char *format, ...)
 {
-	if (dead == 1)
+	if (dead == 1) {
+		kill_all_threads();
 		exit(1);
-	else if (dead == 2)
+	}
+	else if (dead == 2) {
+		kill_all_threads();
 		_exit(1);
-	else if (dead == 3)
+	}
+	else if (dead == 3) {
+		kill_all_threads();
 		kill(getpid(), SIGKILL);
+	}
 	dead++;
-	close_all_servers();
 
 	set_lastlog_size(NULL, NULL, 0);
 	set_history_size(NULL, NULL, 0);
 
 	if (really_quit)
 	{
+		kill_all_threads();
 		say("Signon time  :    %s", my_ctime(start_time));
 		say("Signoff time :    %s", my_ctime(now));
 		say("Total uptime :    %s", convert_time(now - start_time));
@@ -306,6 +371,8 @@ void nap_exit (int really_quit, char *reason, char *format, ...)
 	if (reason)
 		say("%s", reason);
 
+	close_all_servers();
+
 	if (term_initialized)
 	{
 		cursor_to_input();
@@ -319,20 +386,6 @@ void nap_exit (int really_quit, char *reason, char *format, ...)
 	delete_all_windows();
 	destroy_call_stack();		
 	write_unfinished_list();
-#ifdef THREAD
-#ifdef WANT_NSLOOKUP
-	kill_dns();
-#endif
-#ifdef WANT_PTEST
-	kill_ptest();
-#endif
-#ifdef GTK
-	kill_tgtk();
-#endif
-#if !defined(WIN32_GUI)
-	clean_share_mutexes();
-#endif
-#endif
 	
 	debug_cleanup();
 	fprintf(stdout, "\r");
@@ -340,6 +393,7 @@ void nap_exit (int really_quit, char *reason, char *format, ...)
 	if (really_quit)
 		exit(0);
 
+	kill_all_threads();
 	my_signal(SIGABRT, SIG_DFL, 0);
 	kill(getpid(), SIGABRT);
 	kill(getpid(), SIGQUIT);
@@ -347,7 +401,7 @@ void nap_exit (int really_quit, char *reason, char *format, ...)
 }
 
 /*
- * quit_response: Used by irc_io when called from irc_quit to see if we got
+ * quit_response: Used by irc_io when called from nap_quit to see if we got
  * the right response to our question.  If the response was affirmative, the
  * user gets booted from irc.  Otherwise, life goes on. 
  */
@@ -360,8 +414,8 @@ static	void quit_response(char *dummy, char *ptr)
 			nap_exit(1, "TekNap Rest in peace", NULL);
 }
 
-/* irc_quit: prompts the user if they wish to exit, then does the right thing */
-void irc_quit(char key, char * ptr)
+/* nap_quit: prompts the user if they wish to exit, then does the right thing */
+void nap_quit(char key, char * ptr)
 {
 	static	int in_it = 0;
 
@@ -531,10 +585,18 @@ static	char	*parse_args (char *argv[], int argc, char **envp)
 			case 'v':
 				fprintf(stderr, "%s %s\n", nap_version, internal_version); 
 				exit(1);
+#if defined(WINNT) || defined(EMX)
+			case 's':
+				setup_autoexec();
+				exit(1);
+#endif
 			default:
 				fprintf(stderr, "Unknown flag: %s\n",argv[ac]);
 			case 'h':
 				fprintf(stderr, "%s", switch_help);
+#if defined(WINNT) || defined(EMX)
+				fprintf(stderr, "%s", switch_help_w);
+#endif
 				exit(1);
 		   } /* End of switch */
 		}
@@ -668,14 +730,19 @@ static	struct	timeval my_now,
 	set_server_bits(&rd, &wd);
 	set_process_bits(&rd);
 	set_socket_read(&rd, &wd);
-
-#ifdef THREAD
+	icmp_sockets(&rd, &wd);
+	
+#if defined(WANT_THREAD)
 #ifdef WANT_NSLOOKUP
 	set_dns_output_fd(&rd);
 #endif
 #ifdef WANT_PTEST
 	set_ptest_output_fd(&rd);
 #endif
+#ifdef WANT_MP3PLAYER
+	set_mp3_output_fd(&rd);
+#endif
+	
 # if defined(GTK)
 	if (tgtk_okay())
 		tgtk_set_output_fd(&rd);
@@ -726,13 +793,17 @@ static	struct	timeval my_now,
 			now = time(NULL);
 			make_window_current(NULL);
 			do_screens(&rd);
-#ifdef THREAD
+			check_icmpresult(&rd, &wd);
+#if defined(WANT_THREAD)
 #ifdef WANT_NSLOOKUP
 			dns_check(&rd);
 #endif
 #ifdef WANT_PTEST
 			ptest_check(&rd);
 #endif
+#ifdef WANT_MP3PLAYER
+			mp3_check(&rd);
+#endif
 # if defined(GTK)
 			if (tgtk_okay())
 				tgtk_check(&rd);
@@ -750,6 +821,11 @@ static	struct	timeval my_now,
 	ExecuteTimers();
 
         get_child_exit(-1);
+	if (update_refresh)
+	{
+		update_refresh = 0;
+		refresh_screen(0, NULL);
+	}
 	if (!hold_over)
 		cursor_to_input();
 
@@ -761,13 +837,14 @@ static	struct	timeval my_now,
 			update_all_status(current_window, NULL, 0);
 			cursor_to_input();
 		}
-		clean_queue(30 * 60); /* timeout if send time is greater than 5 minutes */
+		clean_queue(get_int_var(TRANSFER_TIMEOUT_VAR)); /* timeout if send time is greater than 5 minutes */
 	}
 
 	/* (set in term.c) -- we should redraw the screen here */
 	if (need_redraw)
 		refresh_screen(0, NULL);
-	
+	if (scan_done)
+		scan_is_done();	
 	alloca(0);
 	return;
 }
@@ -800,7 +877,8 @@ int main(int argc, char *argv[], char *envp[])
         my_signal(SIGCHLD, child_reap, 0);
 	my_signal(SIGCONT, term_cont, 0);
 	my_signal(SIGWINCH, sig_refresh_screen, 0);
-        
+	my_signal(SIGUSR1, sigusr1, 0);
+	        
 	if (!init_screen()) 
 	{
 		create_new_screen();
@@ -810,11 +888,15 @@ int main(int argc, char *argv[], char *envp[])
 	init_keys();
 	init_keys2();
 	init_variables();
-#ifdef THREAD
+#ifdef WANT_THREAD
 	init_share_mutexes();
 #ifdef WANT_NSLOOKUP
 	start_dns();
 #endif
+#ifdef WANT_MP3PLAYER
+	start_mp3();
+#endif
+
 #ifdef GTK
 	start_tgtk();
 #endif
