@@ -1,4 +1,4 @@
-/* $Id: napster.c,v 1.11 2000/07/01 14:03:04 edwards Exp $ */
+/* $Id: napster.c,v 1.30 2000/09/06 22:41:21 cvs Exp $ */
  
 #include "teknap.h"
 #include "keys.h"
@@ -21,7 +21,9 @@
 #include "tgtk.h"
 #include <string.h>
 
+#if 0
 FileStruct *file_search = NULL;
+#endif
 
 #ifdef THREAD1
 pthread_mutex_t file_search_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -30,6 +32,7 @@ pthread_mutex_t file_search_mutex = PTHREAD_MUTEX_INITIALIZER;
 NickStruct *nap_hotlist = NULL;
 
 char *last_invited = NULL;
+extern char *FromUserHost;
 
 PT_STATS ptest_stats;
 
@@ -46,68 +49,10 @@ UrlList *url_list = NULL,
 static int url_count = 0;
 
 
-NAP_COMM(cmd_stats);
-NAP_COMM(cmd_motd);
-NAP_COMM(cmd_email);
-NAP_COMM(cmd_bastard);
-NAP_COMM(cmd_login);
-NAP_COMM(cmd_unknown);
-NAP_COMM(cmd_joined);
-NAP_COMM(cmd_parted);
-NAP_COMM(cmd_names);
-NAP_COMM(cmd_endnames);
-NAP_COMM(cmd_topic);
-NAP_COMM(cmd_public);
-NAP_COMM(cmd_msg);
-NAP_COMM(cmd_search);
-NAP_COMM(cmd_browse);
-NAP_COMM(cmd_endsearch);
-NAP_COMM(cmd_endbrowse);
-NAP_COMM(cmd_whois);
-NAP_COMM(cmd_whowas);
-NAP_COMM(cmd_channellist);
-NAP_COMM(cmd_endchannel);
-NAP_COMM(cmd_newchannellist);
-NAP_COMM(cmd_newendchannel);
-NAP_COMM(cmd_hotlist);
-NAP_COMM(cmd_hotlistsuccess);
-NAP_COMM(cmd_hotlisterror);
-NAP_COMM(cmd_offline);
-NAP_COMM(cmd_alreadyregistered);
-NAP_COMM(cmd_dataport);
-NAP_COMM(cmd_registerinfo);
-NAP_COMM(cmd_banlist);
-NAP_COMM(cmd_ping);
-NAP_COMM(cmd_sping);
-NAP_COMM(cmd_pingresponse);
-NAP_COMM(cmd_recname);
-NAP_COMM(cmd_endname);
-NAP_COMM(cmd_getfileinfo);
-NAP_COMM(cmd_getfile);
-NAP_COMM(cmd_fatalerror);
-NAP_COMM(cmd_me);
-NAP_COMM(cmd_banlistend);
-NAP_COMM(cmd_dataport);
-NAP_COMM(cmd_filerequest);
-NAP_COMM(cmd_accepterror);
-NAP_COMM(cmd_setdataport);
-NAP_COMM(cmd_firewall_request);
-NAP_COMM(cmd_toomanyfiles);
-NAP_COMM(cmd_getlinespeed);
-NAP_COMM(cmd_nosuchnick);
-NAP_COMM(cmd_announce);
-NAP_COMM(cmd_links);
-NAP_COMM(cmd_usage);
-NAP_COMM(cmd_servermsg);
-NAP_COMM(cmd_resumerequest);
-NAP_COMM(cmd_resumerequestend);
-NAP_COMM(cmd_wallop);
-NAP_COMM(cmd_partchannel);
-NAP_COMM(cmd_showusersend);
-NAP_COMM(cmd_showusers);
 
 int grab_http(char *, char *, char *);
 void switch_channels_window(Window *);
+extern char *recv_nick;
 
 char *_speed_color[] = 
 	{ "[1;30m", "[0;32m", "[0;33m", "[0;35m", "[0;31m", "[0;00m", "[1;34m", "[1;31m", "[1;35m", "[1;33m", "[1;32m", "" };
@@ -474,15 +419,15 @@ time_t online;
 	d_port = next_arg(args, &args);
 	email = next_arg(args, &args);
 		
-	put_it("--------------- --  -");
+	put_it("5232523252325032503250-      -");
 	if (l_ip)
-		put_it("| User    : %s(%s) %s l:%s d:%s", nick, email, l_ip, l_port, d_port);
+		put_it("5257 User     :-1 %s50(-1%s50)-1 %s l57:-1%s d57:-1%s",nick, email, l_ip, l_port, d_port);
 	else
-		put_it("| User       : %s", nick);
-	put_it(" Class      : %s", class);
-	put_it(": Last online: %s", my_ctime(online));
+		put_it("5257 User     :-1 %s",nick);
+	put_it("52|57 C-1lass    57:-1 %s", class);
+	put_it("32 57L-1ast online 57:-1 %s", my_ctime(online));
 	if (t_down || t_up)
-		put_it(": Total Uploads : %d Downloading : %d", t_up, t_down);
+		put_it("50:-1 Total  U 57:-1 %d D57 :-1 %d", t_up, t_down);
 	who_remove_queue(from_server, nick);
 	return 0;
 }
@@ -543,7 +488,13 @@ FileStruct *new;
 #endif
 		scott2_add_search_file(from_server, new->nick, new->filename, new->filesize, new->bitrate, n_speed(new->speed));
 #endif
-	add_to_list_double((List **)&file_search, (List *)new);
+	if (get_icmp_socket(from_server) > -1)
+	{
+		new->icmp = 1;
+		send_ping(get_icmp_socket(from_server), new->ip);
+		get_time(&new->start);
+	}
+	add_to_list_double((List **)/*&file_search*/get_search_head(from_server), (List *)new);
 #ifdef THREAD1
 	pthread_mutex_unlock(&file_search_mutex);
 #endif
@@ -622,13 +573,15 @@ void print_file(const FileStruct *f, int count, int maxwidth)
 {
 static char dir[BIG_BUFFER_SIZE];
 char *fs;
+int search_flags = get_server_search(from_server);
+
 	if (!f || !f->filename)
 		return;
 	if (count == 1)
 	{
 		char fmtbuf[128];
-		snprintf(fmtbuf,127,"%%3s . %%-%ds . %%3s . %%5s . %%6s . %%10s . %%-16s . %%s",maxwidth);
-		put_it(fmtbuf,"#","Song","Bit","Freq","Time","Size","Nick","Speed");
+		snprintf(fmtbuf,127,"%%3s . %%-%ds . %%3s . %%5s . %%6s . %%10s . %%4s . %%-16s . %%s",maxwidth);
+		put_it(fmtbuf,"#","Song","Bit","Freq","Time","Size","Nick","Ping", "Speed");
 		*dir = 0;
 	}
 	if ((fs = get_string_var(SEARCH_FORMAT_VAR)))
@@ -647,12 +600,15 @@ char *fs;
 	{
 		char sizebuf[32],fmtbuf[128];
 		char filename[PATH_MAX+1];
+		char ping[32] = "N/A";
 		strncpy(filename,f->filename,PATH_MAX);
 		remove_trailing_spaces(filename);
-		snprintf(fmtbuf,127,"%%3d | %%-%ds | %%3u | %%5u | %%6s | %%10s | %%-16s | %%s",maxwidth);
+		snprintf(fmtbuf,127,"%%3d | %%-%ds | %%3u | %%5u | %%6s | %%10s | %%4s | %%-16s |%%s",maxwidth);
 		snprintf(sizebuf,31,"%4.2f%2s",(float)_GMKv(f->filesize),_GMKs(f->filesize));
-		put_it(fmtbuf, count, base_name(filename), f->bitrate, f->freq,
-			mp3_time(f->seconds), sizebuf,f->nick, n_speed(f->speed));
+		if (f->result.tv_sec != 0)
+			snprintf(ping, 31, "%1.3f", time_diff(f->start, f->result)/2);
+		put_it(fmtbuf, count, (search_flags == SEARCH_FULLPATH) ? filename : base_name(filename), f->bitrate, f->freq,
+			mp3_time(f->seconds), sizebuf,f->nick, ping, n_speed(f->speed));
 	}
 }
 
@@ -680,11 +636,13 @@ NickStruct *n;
 
 NAP_COMM(cmd_endsearch)
 {
-FileStruct *f;
+FileStruct **f, *s;
 int count = 0;
-	for (f = file_search; f; f = f->next, count++)
+	if (!(f = get_search_head(from_server)))
+		return 0;
+	for (s = *f ; s; s = s->next, count++)
 		;
-	if (file_search)
+	if (f && *f)
 	{
 		if (do_hook(SEARCH_END_LIST, "%d", count))
 #ifdef SCOTT
@@ -695,7 +653,7 @@ int count = 0;
 	}
 	else
 		say("Search finished. Server found no matches.");
-	set_server_search(from_server, 0);
+	set_server_search(from_server, SEARCH_FINISH);
 	return 0;
 }
 
@@ -806,9 +764,12 @@ NAP_COMM(cmd_msg)
 		return 0;
 	set_display_target(from, LOG_MSG);
 	if (do_hook(MSG_LIST, "%s %s", from, args))
-		put_it(FORMAT_MSG, from, "*@*", args);
-	if (!my_stricmp(get_server_nickname(from_server), from))
+		put_it(FORMAT_MSG, from, (FromUserHost && *FromUserHost) ? FromUserHost : "*@*", args);
+	if (my_stricmp(get_server_nickname(from_server), from))
+	{
 		grab_http(from, get_server_nickname(from_server), args);
+		malloc_strcpy(&recv_nick, from);
+	}
 	addtabkey(from, "msg", args);
 	return 0;
 }
@@ -866,13 +827,15 @@ NickStruct *n;
 				continue;
 			if ((new = (ChannelStruct *)find_in_list((List **)&tmp->nchannels, chan, 0)))
 				break;
+			else
+				new = NULL;
 		}
 		if (!new)
 			return 0;
 		if (!(nick = next_arg(args, &args)))
 			return 0;
 		set_display_target(chan, LOG_CRAP);
-		if ((n = (NickStruct *)remove_from_list((List **)&new->nicks, nick)))
+		if ((n = (NickStruct *)remove_from_list_double((List **)&new->nicks, nick)))
 		{
 			int shared = 0;
 			int speed = 0;
@@ -914,6 +877,8 @@ ChannelStruct *new = NULL;
 				continue;
 	 		if ((new = (ChannelStruct *)find_in_list((List **)&tmp->nchannels, chan, 0)))
 	 			break;
+			else
+				new = NULL;
 	 	}
 	 	if (!new)
 	 		return 0;
@@ -941,6 +906,8 @@ Window *tmp = NULL;
 			continue;
 		if ((ch = (ChannelStruct *)find_in_list((List **)&tmp->nchannels, chan, 0)))
 			break;
+		else
+			ch = NULL;
 	}
 	if (ch)
 	{
@@ -949,7 +916,7 @@ Window *tmp = NULL;
 		{
 			n = (NickStruct *)new_malloc(sizeof(NickStruct));
 			n->nick = m_strdup(nick);
-			add_to_list((List **)&ch->nicks, (List *)n);
+			add_to_list_double((List **)&ch->nicks, (List *)n);
 		}
 		n->shared = my_atol(next_arg(args, &args));
 		n->speed = my_atol(args);
@@ -957,32 +924,42 @@ Window *tmp = NULL;
 		if (!ch->injoin && do_hook(JOIN_LIST, "%s %s %lu %d", chan, nick, n->shared, n->speed) && !check_cignore(chan) && !check_nignore(nick))
 		{
 			char join_str[200], *p;
-			strcpy(join_str, "%s has joined %s [1;30m[       %d/%s[1;30m]");
+			if (FromUserHost && *FromUserHost)
+				strcpy(join_str, "%s(%s) has joined %s [1;30m[       %d/%s[1;30m]");
+			else
+				strcpy(join_str, "%s%s has joined %s [1;30m[       %d/%s[1;30m]");
 			p = strstr(join_str, "       ");
 			memcpy(p, speed_color(n->speed), 7);
-			say(join_str, nick, chan, n->shared, n_speed(n->speed));
+			say(join_str, nick, FromUserHost && *FromUserHost ? FromUserHost : empty_string, chan, n->shared, n_speed(n->speed));
 		}
 	}
 	reset_display_target();
 	return 0;
 }
 
-void name_print(const char *channel, const NickStruct *nicks, int hotlist, int sort)
+void name_print(const char *channel, const NickStruct *nicks, int hotlist, int sort, char *pattern)
 {
 char buffer[BIG_BUFFER_SIZE+1];
 char tmp[BIG_BUFFER_SIZE];
 char *p;
 int cols = get_int_var(NAMES_COLUMNS_VAR);
 int count = 0;
-NickStruct *n1;
+NickStruct *n1, *n2 = NULL;
 int buf_len = 0;
 char *name = NULL;
 
+	if (sort == NORMAL_SORT)
+		n1 = (NickStruct *)nicks;
+	else
+		n1 = n2;
+		
 	if (!cols)
 		cols = 1;
 	*buffer = 0;
 	for (n1 = (NickStruct *)nicks; n1; n1 = n1->next)
 	{
+		if (pattern && !wild_match(pattern, n1->nick))
+			continue;
 		m_s3cat(&name, " ", n1->nick);
 		m_s3cat(&name, " ", ltoa(n1->speed));
 		m_s3cat(&name, " ", ltoa(n1->shared));
@@ -999,6 +976,8 @@ char *name = NULL;
 	new_free(&name);
 	for (n1 = (NickStruct *)nicks; n1; n1 = n1->next)
 	{
+		if (pattern && !wild_match(pattern, n1->nick))
+			continue;
 		buf_len = strlen(buffer);
 		if (hotlist)
 			sprintf(buffer+buf_len, "%s%16s%s%3s ", 
@@ -1039,6 +1018,8 @@ Window *tmp = NULL;
 		{
 			if ((ch = (ChannelStruct *)find_in_list((List **)&tmp->nchannels, chan, 0)))
 				break;
+			else
+				ch = NULL;
 		}
 	}
 	if (!tmp)	
@@ -1047,7 +1028,7 @@ Window *tmp = NULL;
 	{
 		ch->injoin = 0;
 		if (get_int_var(SHOW_CHANNEL_NAMES_VAR))
-			name_print(chan, ch->nicks, 0, NORMAL_SORT);
+			name_print(chan, ch->nicks, 0, NORMAL_SORT, NULL);
 	} else 
 		say("Something screwy in cmd_endnames()");
 	malloc_strcpy(&tmp->current_channel, chan);
@@ -1181,7 +1162,14 @@ NAP_COMM(cmd_alreadyregistered)
 
 NAP_COMM(cmd_offline)
 {
-	say("User %s offline", args);
+NickStruct *new;
+	if ((new = (NickStruct *)find_in_list((List **)&nap_hotlist, args, 0)))
+	{
+		if (do_hook(HOTLIST_LIST, "%s OFF", new->nick))
+			say("HotList User %s has signed off",new->nick);
+	}
+	else
+		say("User %s offline", args);
 	return 0;
 }
 
@@ -1287,7 +1275,7 @@ char *nick;
 			start.tv_usec = my_atol(next_arg(args, &args));
 			snprintf(t, 70, "%1.3f", time_diff(start, tv) / 2);
 			if (do_hook(PONG_LIST, "%s %s", nick, t))
-				say("recieved pong from %s [%s seconds]", nick, t);
+				say("received pong from %s [%s seconds]", nick, t);
 		}
 	}
 	return 0;
@@ -1564,8 +1552,8 @@ NAP_COMM(cmd_usage)
 {
 	char *num_clients, *num_servers, *num_channels, *total_users;
 	unsigned long registered;
-	char buff[80];
-	unsigned long num_files, num_bytes;
+	char buff[80], buff1[80];
+	unsigned long num_files;
 	long mem_usage;
 	
 	time_t suptime, start_t;
@@ -1574,9 +1562,11 @@ NAP_COMM(cmd_usage)
 	num_clients = next_arg(args, &args);
 	num_servers = next_arg(args, &args);
 	total_users = next_arg(args, &args);
+
 	num_files = my_atoul(next_arg(args, &args));
-	num_bytes = my_atoul(next_arg(args, &args));
+	strcpy(buff1, next_arg(args, &args));
 	num_channels = next_arg(args, &args);
+
 	start_t = my_atoul(next_arg(args, &args));
 	suptime = my_atoul(next_arg(args, &args));
 	mem_usage = my_atol(next_arg(args, &args));
@@ -1590,17 +1580,42 @@ NAP_COMM(cmd_usage)
 	days = (suptime - hours) / 24;
 
 	strcpy(buff, ulongcomma(num_files));
+	
 	say("Server stats: Started %s", my_ctime(start_t));
 	put_it("\tClients %7s\t Servers %12s\t Channels %18s", num_clients, num_servers, num_channels);
-	put_it("\tShares  %7s\t Files   %12s\t Bytes    %18s", total_users, buff, ulongcomma(num_bytes));
+	put_it("\tShares  %7s\t Files   %12s\t Bytes    %18s", total_users, buff, slongcomma(buff1));
 	if (mem_usage != -1)
 		put_it("\tMemory Usage %lu", mem_usage);
-	put_it("\tRegistered users %lu", registered);
+	if (args && *args)
+	{
+		char *in_speed, *out_speed, *searches;
+		char bytes_in[80], bytes_out[80];
+		double g_bytes_in, g_bytes_out;
+		double total;
+
+		in_speed = next_arg(args, &args);
+		out_speed = next_arg(args, &args);
+		searches = next_arg(args, &args);
+		strcpy(bytes_in, next_arg(args, &args));
+		strcpy(bytes_out, next_arg(args, &args));
+		put_it("\tRegistered users %lu\tSearches/second %s", registered, searches);
+		total = atof(in_speed) + atof(out_speed);
+		put_it("\tBW IN/OUT %8sKB/s %8sKB/s == %4.1fKB/s", in_speed, out_speed, total);
+		strcpy(buff, slongcomma(bytes_in));
+		g_bytes_in = atof(bytes_in);
+		g_bytes_out = atof(bytes_out);
+		total = g_bytes_in + g_bytes_out;
+		put_it("\tBytes IN/OUT %s(%4.1f%s) %s(%4.1f%s) == %4.1f%s", 
+			buff, _GMKv(g_bytes_in), _GMKs(g_bytes_in), 
+			slongcomma(bytes_out), _GMKv(g_bytes_out), _GMKs(g_bytes_out),
+			_GMKv(total), _GMKs(total));
+	} else
+		put_it("\tRegistered users %lu", registered);
 	put_it("\tUptime  %7d Days  %02d Hours  %02d Minutes  %02d Seconds", days, hours, minutes, seconds);
 	return 0;
 }
 
-#ifdef WANT_PTEST
+#if defined(WANT_PTEST) && defined(WANT_THREAD)
 
 /* SHEIK */
 
@@ -1770,7 +1785,7 @@ static void parse_servmsg(char *line)
 NAP_COMM(cmd_servermsg)
 {
 	char *copy = NULL;
-#ifdef WANT_PTEST
+#if defined(WANT_PTEST) && defined(WANT_THREAD)
 	if (get_int_var(PTEST_VAR) == ON) {
 		malloc_strcpy(&copy, args);
 		parse_servmsg(copy);
@@ -1785,6 +1800,8 @@ NAP_COMM(cmd_servermsg)
 		n = next_arg(nick, &nick);
 		who_remove_queue(from_server, n);
 		/* damn them. need to remove the whois queue for this nick */
+		say("%s", args);
+		return 0;
 	}
 	else if (args && wild_match("*set your user level to*", args))
 	{
@@ -1793,6 +1810,8 @@ NAP_COMM(cmd_servermsg)
 		for (i = USER_LEECH; *class[i]; i++)
 			if (strstr(args, class[i]))
 				set_server_admin(from_server, i);
+		say("%s", args);
+		return 0;
 	}
 	else if (check_wait_command(args))
 		return 0;
@@ -1806,7 +1825,6 @@ NAP_COMM(cmd_servermsg)
 		set_server_cloak(from_server, 0);
 		build_status(current_window, NULL, 0);
 	}
-
 	if (wild_match("% invited you to channel *", args))
 	{
 		char *chan, *nick;
@@ -1889,15 +1907,28 @@ NAP_COMM(cmd_showusers)
 {
 	if (args && *args)
 	{
-		char *nick, *m;
+		char *nick, *host, *m;
 		unsigned int *count;
 		nick = next_arg(args, &args);
+		host = next_arg(args, &args);
 		m = get_server_showuser(from_server, &count);
-		if (!m || wild_match(m, nick) || wild_match(m, args))
+		if (!m || wild_match(m, nick) || wild_match(m, host))
 		{
 			if (*count == 0)
-				say("%-32s IP", "Nickname");
-			put_it("    %-32s %s", nick, args);
+			{
+#if 0
+				if (host && *host)
+					say("%-22s %-18s Hostname", "Nickname", "IP");
+				else
+#endif
+					say("%-22s IP", "Nickname");
+			}
+#if 0
+			if (host && *host)
+				put_it("    %-22s %-18s %s", nick, args, host);
+			else
+#endif
+				put_it("    %-22s %s", nick, host);
 			(*count)++;
 		}
 	}
