@@ -1,4 +1,4 @@
-/* $Id: scott.c,v 1.5 2000/06/27 22:20:02 nuke Exp $ */
+/* $Id: scott.c,v 1.9 2000/08/07 01:50:20 cvs Exp $ */
  
 /*
  * File:	scott.c
@@ -66,7 +66,6 @@ static int block_mode = 0;
 static int help_toggle = 0;
 
 /* External global linked lists, we need to be able to access */
-extern FileStruct *file_search;
 extern FileStruct *file_browse;
 extern char *n_speed(int);
 
@@ -151,8 +150,10 @@ BUILT_IN_COMMAND(scott)
 		say("Please specify a nick or -search!");
 		return;
 	}
-	if (!my_strnicmp(nick, "-search", 2)) {
-		true_top = top = file_search;
+	if (!my_strnicmp(nick, "-search", 7)) {
+		FileStruct **f;
+		f = get_search_head(from_server);
+		true_top = top = *f;
 		curr_nick = NULL;
 	}
 	else if ((n = (NickStruct *)find_in_list((List **)&server_list[from_server].users, nick, 0)))
@@ -835,7 +836,7 @@ do_download_of_one_file(WINDOW *win)
 	FileStruct *tmp;
 
 	getyx(win, c_y, c_x);
-	if (!(tmp = correlate_line_to_file(c_y)))
+	if (!(tmp = correlate_line_to_file(c_y)) || tmp->getfile)
 		return;
 	do_actual_get(tmp);
 }
@@ -845,7 +846,7 @@ do_download_of_toggles(void)
 {
 	FileStruct *tmp;
 	for (tmp = true_top; tmp; tmp = tmp->next) {
-		if (tmp->flags & BROWSE_TOGGLE) {
+		if (tmp->flags & BROWSE_TOGGLE && !tmp->getfile) {
 			tmp->flags &= ~BROWSE_TOGGLE;
 			do_actual_get(tmp);
 		}
@@ -921,6 +922,9 @@ cleanup(void)
 	in_browser = 0;
 	refresh_screen(0, NULL);
 	cursor_to_input();
+#ifdef WINNT
+	reset_console_mode();
+#endif
 }
 
 int
