@@ -11,7 +11,7 @@
  * Copyright(c) 2000
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT
- * $Id: scott2.c,v 1.3 2000/06/23 05:55:54 edwards Exp $
+ * $Id: scott2.c,v 1.4 2000/09/09 06:21:22 cvs Exp $
  */
 
 #include "teknap.h"
@@ -26,7 +26,7 @@
 
 #define WANT_SCOTT2
 
-#if defined(THREAD) && defined(WANT_SCOTT2) && defined(GTK)
+#if defined(WANT_THREAD) && defined(WANT_SCOTT2) && defined(GTK)
 
 #define BROWSE_TYPE 0
 #define SEARCH_TYPE 1
@@ -986,4 +986,4 @@ get_file_node_from_tree(GtkCTreeNode *parent, char *file, int remove)
 }
 
 
-#endif /* THREAD  && WANT_GTK */
+#endif /* WANT_THREAD  && WANT_GTK */
