@@ -9,7 +9,7 @@
  * Copyright 1998 J. Kean Johnston, used with permission
  * Modifications Copyright Colten Edwards 1997-1998
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT
- * $Id: screen.c,v 1.5 2000/07/01 14:03:05 edwards Exp $
+ * $Id: screen.c,v 1.10 2000/09/09 08:15:41 cvs Exp $
  */
 
 
@@ -532,7 +532,7 @@ const 	u_char	*ptr = NULL;
 			if (ptr[1] == 0)
 				break;
 			if (!word_break || (flags & PREPARE_NOWRAP))
-				word_break = max_cols /*pos - 1*/;
+				word_break = pos /*pos - 1*/;
 			else if (col > max_cols)
 				word_break = pos - 1;
 								
@@ -574,21 +574,60 @@ const 	u_char	*ptr = NULL;
 					output[output_size++] = 0;
 			}
 
+			if (!*cont)
+			{
+				int	lhs_count = 0;
+				int	continued_count = 0;
+
+				if (do_indent && (indent < (max_cols / 3)) &&
+						(strlen(cont_ptr) < indent))
+				{
+					cont = alloca(indent+10); /* sb pana */
+					sprintf(cont, "%-*s", indent, cont_ptr);
+				}
+
+				/*
+				 * Otherwise, we just use /set continued_line, 
+				 * whatever it is.
+				 */
+				else if (!*cont && *cont_ptr)
+					cont = cont_ptr;
+
+
+				/*
+				 * XXXX "line wrap bug" fix.  If we are here,
+				 * then we are between the first and second
+				 * lines, and we might have a word that does
+				 * not fit on the first line that also does
+				 * not fit on the second line!  We check for
+				 * that right here, and if it won't fit on
+				 * the next line, we revert "word_break" to
+				 * the current position.
+				 *
+				 * THIS IS UNFORTUNATELY VERY EXPENSIVE! :(
+				 */
+				c = buffer[word_break];
+				buffer[word_break] = 0;
+				lhs_count = output_with_count(buffer, 0, 0);
+				buffer[word_break] = c;
+				continued_count = output_with_count(cont, 0, 0);
+
+				/* 
+				 * Chop the line right here if it will
+				 * overflow the next line.
+				 */
+				if (lhs_count <= continued_count)
+					word_break = pos;
+
+				/*
+				 * XXXX End of nasty nasty hack.
+				 */
+			}
 			c = buffer[word_break];
 			buffer[word_break] = 0;
 			malloc_strcpy((char **)&(output[line++]), buffer);
-
 			buffer[word_break] = c;
 
-			if (!*cont && do_indent && (indent < (max_cols / 3)) &&
-					(strlen(cont_ptr) < indent))
-			{
-				cont = alloca(indent+10);
-				sprintf(cont, "%-*s", indent, cont_ptr);
-			}
-			else if (!*cont && *cont_ptr)
-				cont = cont_ptr;
-
 			while (buffer[word_break] == ' ' && word_break < pos)
 				word_break++;
 			buffer[pos] = 0;
@@ -597,7 +636,8 @@ const 	u_char	*ptr = NULL;
 			
 			strlcpy (buffer, cont, BIG_BUFFER_SIZE/2);
 			strlcat (buffer, pos_copy, BIG_BUFFER_SIZE/2);
-			col = pos = strlen(buffer);
+			pos = strlen(buffer);
+			col = output_with_count(buffer, 0, 0);
 
 			word_break = 0;
 			newline = 0;
@@ -822,7 +862,6 @@ static	int	altc = OFF;
 
 	if (window->visible && foreground && window->display_size)
 	{
-#if 1
 		display_standout(high);
 		display_bold(bold);
 		display_blink(blink);
@@ -837,13 +876,11 @@ static	int	altc = OFF;
 		undl = display_underline(OFF);
 		blink = display_blink(OFF);
 		altc = display_altcharset(OFF);
-#else
-		tputs_x(replace_color(-2,-2));
-		output_line(str);
-#endif
+
 		term_newline();
 		term_cr();
 	}
+
 	window->cursor++;
 	return 0;
 }
@@ -1539,6 +1576,8 @@ void add_wait_prompt(char *prompt, void (*func)(char *, char *), char *data, int
 	WaitPrompt **AddLoc,
 		   *New;
 
+	if (!current_window->screen)
+		return;
 	New = (WaitPrompt *) new_malloc(sizeof(WaitPrompt));
 	New->prompt = m_strdup(prompt);
 	New->data = m_strdup(data);
@@ -2020,7 +2059,9 @@ unsigned char *strip_ansi (const unsigned char *str)
 			case ('*') : case ('+') : case ('$') :
 			case ('@') :
 			{
-				next_char();	/* Skip the argument */
+				chr = next_char();	/* Skip the argument */
+				if (chr == 0)
+					put_back();
 				break;
 			}
 
@@ -2500,29 +2541,6 @@ static 	char 	retbuf[512];
 	if (back == -1)
 		back = last_back;
 
-#if 0
-	if (back == 58)
-		strcat(retbuf, current_term->TI_sgrstrs[TERM_SGR_BLINK_ON - 1]);
-#if 1
-	if (fore > -1)
-	{
-/*		if (fore_conv[fore] < 8)
-			strcat(retbuf, current_term->TI_sgrstrs[TERM_SGR_BOLD_OFF-1]);*/
-		strcat(retbuf, current_term->TI_forecolors[fore_conv[fore]]);
-	}
-	if (back > -1 && back != 58)
-		strcat(retbuf, current_term->TI_backcolors[back_conv[back]]);
-#else
-	if (fore > -1)
-		strcat(retbuf, current_term->TI_forecolors[fore_conv[fore]]);
-	if (back > -1 && back != 58)
-		strcat(retbuf, current_term->TI_backcolors[back_conv[back]]);
-#endif
-	last_fore = fore;
-	last_back = back;
-
-	return retbuf;
-#else
 	if (fore > -1 && fore_conv[fore] < 8)
 		strcat(retbuf, current_term->TI_sgrstrs[TERM_SGR_BOLD_OFF - 1]);
 	if (back > -1 && back < 58)
@@ -2540,5 +2558,4 @@ static 	char 	retbuf[512];
 	last_back = back;
 
 	return retbuf;
-#endif
 }
