@@ -1,4 +1,4 @@
-/* $Id: server.c,v 1.4 2000/06/28 09:11:41 edwards Exp $ */
+/* $Id: server.c,v 1.15 2000/09/09 06:21:23 cvs Exp $ */
  
 #include "teknap.h"
 #include "struct.h"
@@ -14,12 +14,13 @@
 #include "napster.h"
 #include "scott2.h"
 #include "tgtk.h"
+#include "irchandler.h"
 
 extern int from_server;
 extern	int nick_count;
 extern	int channel_count;
 
-#ifdef THREAD
+#ifdef WANT_THREAD
 #ifdef WINNT
 pthread_mutex_t send_ncommand_mutex = {0};
 #else
@@ -88,6 +89,8 @@ int server = my_atol((char *)arg);
 int rc = -1;
 	while (traverse_all_windows(&tmp))
 	{
+		if (server >= number_of_servers)
+			continue;
 		if ((tmp->server == -1) && (tmp->last_server != -1) && (tmp->last_server == server))
 		{
 			last = tmp;
@@ -99,7 +102,7 @@ int rc = -1;
 			break;
 		}
 	}	
-	if (rc == -1)
+	if (rc == -1 && server < number_of_servers)
 	{
 		add_timer(0, empty_string, 15 * 1000, 1, auto_reconnect, arg, NULL, last ? last->refnum : current_window->refnum, "auto-reconnect");
 		return 0;
@@ -133,6 +136,7 @@ void do_server(fd_set *rd, fd_set *wr)
 char buffer[BIG_BUFFER_SIZE+1];
 int i;
 N_DATA data;
+
 	for (i = 0; i < number_of_servers; i++)
 	{
 		if (server_list[i].read < 0)
@@ -153,7 +157,11 @@ N_DATA data;
 			int junk;
 			from_server = i;
 			*bufptr = 0;
-			junk = dgets(bufptr, server_list[i].read, 1, BIG_BUFFER_SIZE);
+			/* irc mode check here */
+			if (server_list[i].irc_mode)
+				junk = new_dgets(bufptr, server_list[i].read, 1, BIG_BUFFER_SIZE);
+			else
+				junk = dgets(bufptr, server_list[i].read, 1, BIG_BUFFER_SIZE);
 			switch (junk)
 			{
 				case -2:
@@ -161,20 +169,27 @@ N_DATA data;
 				case -1:
 				{
 					close_server(i);
-					close_data(i);
-					say("do_server() Connection closed from %s: %s", server_list[i].name, (dgets_errno <= 0) ? "Remote end closed connection" : strerror(dgets_errno));
+					if (do_hook(DISCONNECT_LIST, "Connection closed from %s: %s", server_list[i].name, (dgets_errno <= 0) ? "Remote end closed connection" : strerror(dgets_errno)))
+						say("do_server() Connection closed from %s: %s", server_list[i].name, (dgets_errno <= 0) ? "Remote end closed connection" : strerror(dgets_errno));
 					start_autoconnect(i);
 					break;
 				}					
 				case 0:
+					if (server_list[i].irc_mode)
+						break;
 					dget_data(server_list[i].read, &data, sizeof(N_DATA));
 					if (data.len)
 						break;
 				default:
 				{
-					dget_data(server_list[i].read, &data, sizeof(N_DATA));
-					parse_server(&data, buffer);
-					dget_clear_data(server_list[i].read);
+					if (server_list[i].irc_mode)
+						parse_irc_server(buffer);
+					else
+					{
+						dget_data(server_list[i].read, &data, sizeof(N_DATA));
+						parse_server(&data, buffer);
+						dget_clear_data(server_list[i].read);
+					}
 					break;
 				}
 			}
@@ -544,6 +559,11 @@ int rc;
 int writelen;
 
 	if (from_server == -1)
+	{
+		do_hook(DISCONNECT_LIST, "Try connecting to a server first");
+		return -1;
+	}
+	if (get_server_ircmode(from_server))
 		return -1;
 	if (fmt)
 	{
@@ -562,7 +582,7 @@ int writelen;
 	n_data.len = BSWAP16(n_data.len);
 	n_data.command = BSWAP16(n_data.command);
 
-#ifdef THREAD
+#ifdef WANT_THREAD
 	pthread_mutex_lock(&send_ncommand_mutex);
 #endif
 	server_list[from_server].sent = 1;
@@ -570,12 +590,12 @@ int writelen;
 	if (fmt)
 	{
 		rc = NAP_send(buffer, writelen);
-#ifdef THREAD
+#ifdef WANT_THREAD
 		pthread_mutex_unlock(&send_ncommand_mutex);
 #endif
 		return rc;
 	}
-#ifdef THREAD
+#ifdef WANT_THREAD
 	pthread_mutex_unlock(&send_ncommand_mutex);
 #endif
 	return (rc != -1) ? 0 : -1;
@@ -677,6 +697,7 @@ NickStruct *new, *last;
 N_STATS *stats;
 Window *tmp = NULL;
 ChannelStruct *ch;
+int last_server = 1;
 
 	if (server < 0 || server >= number_of_servers)
 		return;
@@ -695,8 +716,10 @@ ChannelStruct *ch;
 	nick_count = 0;
 	channel_count = 0;
 	clear_nicks(server);
-	
-	clear_filelist(&file_search);
+	if (server_list[server].icmp_sock > -1)
+		server_list[server].icmp_sock = new_close(server_list[server].icmp_sock);	
+
+	clear_filelist(&server_list[server].search_results);
 	clear_shared();
 	new_free(&last_invited);
 #if defined(GTK)
@@ -726,14 +749,21 @@ ChannelStruct *ch;
 		{
 			tmp->server = -1;
 			tmp->last_server = server;
+			for (ch = tmp->nchannels; ch; ch = ch->next)
+				ch->injoin = 1;
+			tmp->oldchannels = tmp->nchannels;
+			tmp->nchannels = NULL;
+			new_free(&tmp->current_channel);
 		}
-		for (ch = tmp->nchannels; ch; ch = ch->next)
-			ch->injoin = 1;
-		tmp->oldchannels = tmp->nchannels;
-		tmp->nchannels = NULL;
+		else if (tmp->server != -1)
+			last_server = 0;
 	}
-	set_current_channel_by_refnum(0, NULL);
-	set_server_search(server, 0);
+	if (last_server)
+	{
+		clear_filelist(&server_list[server].search_results);
+		clear_shared();
+	}
+	set_server_search(server, SEARCH_FINISH);
 	update_window_status_all();
 }
 
@@ -766,14 +796,35 @@ int i;
 
 void server_disconnect(Window *window, char *args)
 {
+Window *tmp = NULL;
+int serv = -1;
+
 	if (window->server == -1)
 		return;
+	while (traverse_all_windows(&tmp))
+	{
+		if (tmp->server != -1)
+			serv = tmp->server;
+	}
+	if (serv == -1)
+		close_data(window->server);
 	close_server(window->server);
-	close_data(window->server);
+
 	build_status(window, NULL, 0);
-	say("Server and Dataport closed %s", args ? args : "by user request");
-	window->server = -1;
-	from_server = -1;
+	if (do_hook(DISCONNECT_LIST, "Server and dataport closed %s", args ? args : "by user request"))
+		say("Server and Dataport closed %s", args ? args : "by user request");
+
+	tmp = NULL; serv = window->server;
+	while (traverse_all_windows(&tmp))
+	{
+		if (tmp->server == serv)
+		{
+			window->last_server = window->server;
+			window->server = -1;
+		}
+	}
+
+	from_server = serv;
 }
 
 int make_listen(int port, int *portnum)
@@ -913,6 +964,7 @@ int serv;
 		server_list[serv].read = -1;
 		server_list[serv].write = -1;
 		server_list[serv].port = port;
+		server_list[serv].icmp_sock = -1;
 		server_list[serv].motd = get_int_var(SUPPRESS_SERVER_MOTD_VAR);
 		if (password && *password)
 			malloc_strcpy(&(server_list[serv].password), password);
@@ -1006,6 +1058,8 @@ int 	find_server_refnum (char *server, char **rest)
 {
 	int 	refnum;
 	int	port = get_int_var(DEFAULT_METASERVER_VAR);
+	int	my_meta = 0;
+
 	char 	*cport = NULL, 
 		*password = NULL,
 		*nick = NULL,
@@ -1042,11 +1096,16 @@ int 	find_server_refnum (char *server, char **rest)
 	if (cport && *cport)
 		port = my_atol(cport);
 
+	if (meta && isdigit(*meta))
+		my_meta = my_atol(meta);
+	else if (meta && my_stricmp(meta, "on"))
+		my_meta = 1;
+
 	/*
 	 * Add to the server list (this will update the port
 	 * and password fields).
 	 */
-	return add_to_server_list(server, port, password, nick, my_atol(meta), 1);
+	return add_to_server_list(server, port, password, nick, my_meta, 1);
 }
 
 
@@ -1078,6 +1137,7 @@ int	build_server_list (char *servers)
 		*meta = NULL;
 
 	int	port_num;
+	int	my_meta = 0;
 	int	i = 0;
 	
 	if (!servers || !*servers)
@@ -1103,8 +1163,12 @@ int	build_server_list (char *servers)
                         }
 			else
 				port_num = get_int_var(DEFAULT_METASERVER_VAR);
+			if (meta && isdigit(*meta))
+				my_meta = my_atol(meta);
+			else if (meta && my_stricmp(meta, "on"))
+				my_meta = 1;
 
-			add_to_server_list(host, port_num, pass, nick, my_atol(meta), 1);
+			add_to_server_list(host, port_num, pass, nick, my_meta, 1);
 			i++;
 		}
 		servers = rest;
@@ -1118,11 +1182,11 @@ int i;
 	put_it("%4s %35s %15s %10s %s", "Ref", "Server:Port", "Nick", "Password", "Meta");
 	for (i = 0; i < number_of_servers; i++)
 	{
-		put_it("%4d %30s:%d %15s %10s %d %s", i, server_list[i].name,
+		put_it("%4d %30s:%d %15s %10s %3s %s", i, server_list[i].name,
 			server_list[i].port, 
 			server_list[i].d_nickname ? server_list[i].d_nickname : "(none)", 
 			server_list[i].password ? server_list[i].password : "(none)", 
-			server_list[i].meta, is_connected(i) ? "<--":"");
+			off_on(server_list[i].meta), is_connected(i) ? "<--":"");
 	}
 }
 
@@ -1153,6 +1217,26 @@ void send_all_servers(int cmd, char *format, ...)
 	from_server = ofs;
 }
 
+void send_to_server(const char *format, ...)
+{
+	va_list args;
+	int len;
+	char buff[BIG_BUFFER_SIZE+1];
+	if (from_server < 0)
+		return;
+	if (!server_list[from_server].irc_mode || 
+			server_list[from_server].write == -1 || !format)
+		return;
+	va_start(args, format);
+	vsnprintf(buff, BIG_BUFFER_SIZE, format, args);
+	va_end(args);
+	len = strlen(buff);
+	if (len > (IRCD_BUFFER_SIZE - 2) || len == -1)
+		buff[IRCD_BUFFER_SIZE - 2] = (char) 0;
+	strmcat(buff, "\r\n", IRCD_BUFFER_SIZE);
+	write(server_list[from_server].write, buff, strlen(buff));
+}
+
 static void removetabkey(int server)
 {
 Msgs *new, *last;
@@ -1474,3 +1558,99 @@ void set_server_showuser(int server, char *match, int count)
 	else
 		malloc_strcpy(&server_list[server].showuser, match);
 }
+
+FileStruct **get_search_head(int server)
+{
+	if (server < 0 || server > number_of_servers)
+		return NULL;
+	return &server_list[server].search_results;
+}
+
+int get_icmp_socket(int server)
+{
+	if (server < 0 || server > number_of_servers)
+		return -1;
+	return server_list[server].icmp_sock;
+}
+
+void icmp_sockets(fd_set *rd, fd_set *wr)
+{
+int i;
+	for (i = 0; i < number_of_servers; i++)
+	{
+		if (server_list[i].icmp_sock > -1)
+			FD_SET(server_list[i].icmp_sock, rd);
+	}
+}
+
+void set_icmp_socket(int server, int val)
+{
+	if (server < 0 || server > number_of_servers)
+		return;
+	server_list[server].icmp_sock = val;
+}
+
+void add_ping(int serv, struct sockaddr_in *from)
+{
+FileStruct *n;
+	for (n = server_list[serv].search_results; n; n = n->next)
+	{
+		if (n->ip != from->sin_addr.s_addr)
+			continue;
+		n->icmp = 1;
+		get_time(&n->result);
+	}	
+}
+
+void check_icmpresult(fd_set *rd, fd_set *wr)
+{
+int i;
+char buf[80];
+struct sockaddr_in from;
+int fromlen = sizeof(from);
+
+	for (i = 0; i < number_of_servers; i++)
+	{
+		if (server_list[i].icmp_sock > -1)
+		{
+			if (FD_ISSET(server_list[i].icmp_sock, rd))
+			{
+				recvfrom(server_list[i].icmp_sock, buf, sizeof(buf)-1, 0, (struct sockaddr *)&from, &fromlen);
+				add_ping(i, &from);
+			}		
+		}
+	}
+}
+
+int connect_to_irc_server(int serv)
+{
+int des = -1;
+unsigned short port = 6667;
+	if (serv < 0 || serv > number_of_servers)
+		return -1;
+	des = connect_by_number(server_list[serv].name, &port, SERVICE_CLIENT, PROTOCOL_TCP, 0);
+	if (des > -1)
+	{
+		server_list[serv].read = server_list[serv].write = des;
+		server_list[serv].irc_mode = 1;
+		server_list[serv].flags |= SERVER_CONNECTED;
+		new_open(des);
+	}
+	return des;
+}
+
+void login_to_ircserver(int serv)
+{
+int ofs = from_server;
+	from_server = serv;
+	send_to_server("NICK %s", get_server_nickname(serv));
+	send_to_server("USER %s NULL NULL :%s", get_server_nickname(serv), get_server_nickname(serv));
+	from_server = ofs;
+}
+
+int get_server_ircmode(int server)
+{
+	if (server < 0 || server > number_of_servers)
+		return 0;
+	return server_list[server].irc_mode;
+}
