@@ -1,4 +1,4 @@
-/* $Id: share.c,v 1.5 2000/06/29 05:30:36 edwards Exp $ */
+/* $Id: share.c,v 1.17 2000/09/09 08:15:41 cvs Exp $ */
 
 #include "teknap.h"
 #include "struct.h"
@@ -29,10 +29,10 @@
 #define DEFAULT_FILEMASK "*.mp3"
 
 #ifdef WINNT
-#define THREAD
+#define WANT_THREAD
 #endif
 
-#if defined(THREAD) && !defined(WINNT)
+#if defined(WANT_THREAD) && !defined(WINNT)
 
 static pthread_t fserv_thread = {0};
 
@@ -42,8 +42,10 @@ static pthread_mutex_t fserv_struct_mutex = PTHREAD_MUTEX_INITIALIZER;
 pthread_mutex_t shared_count_mutex = PTHREAD_MUTEX_INITIALIZER;
 static void share_thread_signal_setup(void);
 
+
 #endif
 
+extern int scan_done;
 
 #define DEFAULT_MD5_SIZE 292 * 1024
 #define DET_BUFFER_SIZE 1024
@@ -78,8 +80,31 @@ static void *do_update(void *);
 unsigned long shared_count = 0;
 
 static int in_sharing = 0;
+
 int in_load = 0;
 
+static time_t fserv_start = 0;
+
+/* Xing header flags */
+#define VBR_FRAMES_FLAG     0x0001
+#define VBR_BYTES_FLAG      0x0002
+#define VBR_TOC_FLAG        0x0004
+#define VBR_SCALE_FLAG  0x0008
+
+typedef struct
+{
+	unsigned long h_id;	/* from MPEG header, 0=MPEG2, 1=MPEG1 */
+	unsigned long samprate;	/* determined from MPEG header */
+	unsigned long flags;	/* from Xing header data */
+	unsigned long frames;	/* total bit stream frames from Xing header data */
+	unsigned long bytes;	/* total bit stream bytes from Xing header data */
+	long	      vbr_scale;/* encoded vbr scale from Xing header data */
+	unsigned char *toc;	/* pointer to unsigned char toc_buffer[100] */
+	/* may be NULL if toc not desired */
+}
+XHEADDATA;
+
+
 
 char *find_mime_type(char *fn)
 {
@@ -587,9 +612,6 @@ static int mpg123_head_check(unsigned long head)
 		return 0;
 	if (((head >> 19) & 1) == 1 && ((head >> 17) & 3) == 3 && ((head >> 16) & 1) == 1)
 		return 0;
-	if ((head & 0xffff0000) == 0xfffe0000)
-		return 0;
-	
 	return 1;
 }
 
@@ -714,15 +736,72 @@ double compute_tpf(AUDIO_HEADER *fr)
 	return tpf;
 }
 
+int check_vbrheader(AUDIO_HEADER *fr, unsigned char *buf, XHEADDATA *xing)
+{
+	int ssize;
+
+	if (fr->true_layer != 3)
+		return 0;
+
+	if (fr->ID)
+		ssize = (fr->stereo == 1) ? 9 : 17;
+	else
+		ssize = (fr->stereo == 1) ? 17 : 32;
+
+	xing->samprate = fr->freq;
+	xing->h_id = fr->ID;
+
+	buf += ssize;
+	if (( buf[0] != 'X' ) || ( buf[1] != 'i' ) ||
+	    ( buf[2] != 'n' ) || ( buf[3] != 'g' ) ) 
+		return 0;
+
+	buf += 4;
+	xing->flags = convert_to_header(buf);
+
+#if 1
+	if (xing->h_id == 0)
+		xing->samprate >>= 1;
+#endif
+	
+	if (xing->flags & VBR_FRAMES_FLAG)
+	{
+		xing->frames = convert_to_header(buf);
+		buf += 4;
+	}
+
+	if (xing->flags & VBR_BYTES_FLAG) 
+	{
+		xing->bytes  = convert_to_header(buf); 
+		buf += 4;
+	}
+
+	if (xing->flags & VBR_TOC_FLAG) 
+	{
+		xing->toc = new_malloc(101);
+		memcpy(xing->toc,buf,100);
+		buf += 100;
+	}
+	xing->vbr_scale = -1;
+	if (xing->flags & VBR_SCALE_FLAG) 
+	{
+		xing->vbr_scale = convert_to_header(buf);
+		buf += 4;
+	}
+	return 1;
+}
 
 long get_bitrate(int fdes, time_t *mp3_time, unsigned int *freq_rate, unsigned long *filesize, int *stereo, long *id3, int *mime_type)
 {
 
 
-	AUDIO_HEADER header = {0};
+	AUDIO_HEADER header = {0}, hr1 = {0};
+	XHEADDATA xing_header = {0};
 	unsigned long btr = 0;
 	struct stat	st;
 	unsigned long	head;
+	double tpf;
+	unsigned long avg_bitrate = 0;
 
 unsigned char	buf[DET_BUFFER_SIZE+1];
 unsigned char	tmp[5];
@@ -780,13 +859,52 @@ unsigned char	tmp[5];
 		}
 	}
 	header.filesize = st.st_size;
-	parse_header(&header, head);
+	if (!(parse_header(&header, head)))
+		return 0;
+	read(fdes, buf, header.framesize);
+	if (!check_vbrheader(&header, buf, &xing_header))
+	{	
+		btr = header.bitrate;
+		*mp3_time = (time_t) (header.totalframes * compute_tpf(&header));
+	}
+	else if (xing_header.toc)
+	{
+		int i = 0, count = 0;
 
-	btr = header.bitrate;
+		tpf = compute_tpf(&header);
+		*mp3_time = (time_t) (tpf * xing_header.frames * 1000);
 
-	*mp3_time = (time_t) (header.totalframes * compute_tpf(&header));
+#if 1
+		btr = header.bitrate;
+		avg_bitrate = header.bitrate;
+		lseek(fdes, 0, SEEK_SET);
+		parse_header(&hr1, head);
+		while (1)
+		{
+			i++;
+			count += hr1.framesize;
+			if (count >= header.filesize)
+				break;
+			count += hr1.error_protection ? 6 : 4;
+			lseek(fdes, count, SEEK_SET);
+			if ((read(fdes, tmp, 4)) < 4)
+				break;
+			head = convert_to_header(tmp);
+			if (!mpg123_head_check(head))
+				continue;
+			memset(&hr1, 0, sizeof(XHEADDATA));
+			parse_header(&hr1, head);
+			avg_bitrate += hr1.bitrate;
+		}
+		if (i)
+			btr = header.bitrate = avg_bitrate / i;
+#else
+		header.bitrate = (xing_header.bytes * 8) / (tpf * xing_header.frames * 1000);
+#endif
+		header.totalframes = xing_header.frames;
+		new_free(&xing_header.toc);
+	}
 	*freq_rate = header.freq;
-
 	if (id3)
 	{
 		char buff[130];
@@ -958,7 +1076,6 @@ static void *scan_mp3_dir(void *arg)
 	int	type;
 	Share_Struct *sf, *last;
 	int count = 0;
-	time_t start_time;
 	in_load++;
 
 #ifdef WINNT
@@ -968,12 +1085,12 @@ static void *scan_mp3_dir(void *arg)
 #endif		
 	if (get_int_var(SHARE_LINKS_VAR))
 		globflag |= GLOB_LINK;
-#if defined(THREAD) && !defined(WINNT)
+#if defined(WANT_THREAD) && !defined(WINNT)
 	/* Lock the queue */
 	share_thread_signal_setup();
 	pthread_mutex_lock(&fserv_struct_mutex);
 #endif
-	start_time = now;
+	fserv_start = now;
 	
 	for (sf = share_files; sf; sf = last)
 	{
@@ -994,7 +1111,7 @@ static void *scan_mp3_dir(void *arg)
 			char	*fn;
 			long	id3 = 0;
 			fn = globpat.gl_pathv[i];
-#if defined(THREAD) && !defined(WINNT)
+#if defined(WANT_THREAD) && !defined(WINNT)
 			if (pthread_mutex_trylock(&quit_mutex) == 0) 
 			{
 				pthread_mutex_unlock(&quit_mutex);
@@ -1003,7 +1120,7 @@ static void *scan_mp3_dir(void *arg)
 #endif
 			if (fn[strlen(fn)-1] == '/')
 			{
-#if defined(THREAD) && defined(WINNT)
+#if defined(WANT_THREAD) && defined(WINNT)
 				strncpy(return_dir, fn, BIG_BUFFER_SIZE);
 				build_status(current_window, NULL, 0);
 #endif
@@ -1081,14 +1198,14 @@ static void *scan_mp3_dir(void *arg)
 				shared_stats.total_files++;
 				shared_stats.total_filesize += new->filesize;
 				count++;
-#if defined(THREAD) && !defined(WINNT)
+#if defined(WANT_THREAD) && !defined(WINNT)
 				pthread_mutex_lock(&shared_count_mutex);
 				shared_count++;
 				pthread_mutex_unlock(&shared_count_mutex);
 #else
 				shared_count++;
 #endif
-#if !defined(THREAD) && !defined(WINNT)
+#if !defined(WANT_THREAD) && !defined(WINNT)
 				if (!(count % 25))
 				{
 					io("scan_mp3_dir");
@@ -1108,7 +1225,7 @@ static void *scan_mp3_dir(void *arg)
 				shared_stats.total_files++;
 				shared_stats.total_filesize += new->filesize;
 				count++;
-#if defined(THREAD) && !defined(WINNT)
+#if defined(WANT_THREAD) && !defined(WINNT)
 				pthread_mutex_lock(&shared_count_mutex);
 				shared_count++;
 				pthread_mutex_unlock(&shared_count_mutex);
@@ -1150,13 +1267,18 @@ static void *scan_mp3_dir(void *arg)
 		new_free(&sf);
 		share_files = last;
 	}
+	if (shared_count > 0)
+		scan_done = shared_count;
+	else
+		scan_done = -1;
+
 #if !defined(WINNT)
 	if (shared_count)
-		say("Found %d files in %s", shared_count, print_time(time(NULL) - start_time));
+		say("Found %d files in %s", shared_count, print_time(time(NULL) - fserv_start));
 	else
 		say("No files found");
 #endif
-#if defined(THREAD) && !defined(WINNT)
+#if defined(WANT_THREAD) && !defined(WINNT)
 	pthread_mutex_lock(&shared_count_mutex);
 	shared_count = 0;
 	pthread_mutex_unlock(&shared_count_mutex);
@@ -1164,16 +1286,12 @@ static void *scan_mp3_dir(void *arg)
 #else
 	shared_count = 0;
 #endif
-
 	*return_dir = 0;
-#if defined(THREAD) && defined(WINNT)
-	build_status(current_window, NULL, 0);
-#endif
 	in_load--;
-#ifdef WINNT
+#if defined(WANT_THREAD)
+#if defined(WINNT)
 	return 0;
-#else
-	return NULL;
+#endif
 #endif
 }
 
@@ -1206,7 +1324,7 @@ FileStruct *new;
 		char *fn, *md5, *fs, *br, *fr, *t, *args;
 		while (!feof(fp))
 		{
-#if defined(THREAD) && !defined(WINNT)
+#if defined(WANT_THREAD) && !defined(WINNT)
 			if (pthread_mutex_trylock(&quit_mutex) == 0) 
 			{
 				pthread_mutex_unlock(&quit_mutex);
@@ -1308,35 +1426,6 @@ int ots = timeout_select;
 	{
 		int rc = 0;
 		char *name;
-#if 0
-		if (new->shared)
-			continue;
-		if (!new->checksum || !new->filesize || !new->filename)
-			continue;
-		if (args && *args && !matchnumber(args, count))
-			continue;
-		count++;
-		name = LOCAL_COPY(new->filename);
-		name = convertnap_dos(name);
-		
-		if (new->freq && new->bitrate)
-		{
-			rc = send_ncommand(CMDS_ADDFILE, "\"%s\" %s %lu %u %u %lu", 
-				name, new->checksum, new->filesize, new->bitrate, new->freq, new->seconds);
-		}
-		else
-		{
-			char *s;
-			if (!(s = find_mime_type(new->filename)))
-				continue;
-			rc = send_ncommand(CMDS_ADDMIMEFILE, "\"%s\" %lu %s %s", 
-				name, new->filesize, new->checksum, s);
-		}
-		shared_stats.shared_files++;
-		shared_stats.shared_filesize += new->filesize;
-		new->shared = 1;
-		new = new->next;
-#else
 		if (new->shared || (args && *args && !matchnumber(args, count)))
 		{
 			new = new->next;
@@ -1353,6 +1442,7 @@ int ots = timeout_select;
 		name = base_name(name);
 				
 		snprintf(buffer, 1500, "\"%s\" \"%s\" %s %lu %u %u %lu", dirbuffer, name, new->checksum, new->filesize, new->bitrate, new->freq, new->seconds);
+		new->shared = 1; /* maybe? */
 		do {
 			shared_stats.shared_files++;
 			shared_stats.shared_filesize += new->filesize;
@@ -1392,7 +1482,6 @@ int ots = timeout_select;
 		} while (new);
 		if (*buffer)
 			rc = send_ncommand(CMDS_SHAREPATH, "%s", buffer);
-#endif
 		if (rc == -1)
 		{
 			in_sharing = 0;
@@ -1415,11 +1504,16 @@ int ots = timeout_select;
 
 unsigned long unshare_nap(void)
 {
+#if 0
 FileStruct *new;
-unsigned long count = 0;
 char *filename;
+#endif
+unsigned long count = 0;
 	if (shared_stats.shared_files)
 	{
+		send_ncommand(CMDS_REMOVEALLFILES, NULL);
+		count = shared_stats.shared_files;
+#if 0
 		for (new = fserv_files; new; new = new->next)
 		{
 			if (new->shared)
@@ -1429,6 +1523,7 @@ char *filename;
 				count++;
 			}
 		}
+#endif
 		clear_shared();
 	}
 	shared_stats.shared_files = 0;
@@ -1454,7 +1549,7 @@ void *load_file(void *args)
 #endif
 {
 Share_Struct *new, *last;
-#if defined(THREAD) && !defined(WINNT)
+#if defined(WANT_THREAD) && !defined(WINNT)
 	/* Lock the queue */
 	share_thread_signal_setup();
 	pthread_mutex_lock(&fserv_struct_mutex);
@@ -1470,15 +1565,17 @@ Share_Struct *new, *last;
 		new_free(&new);
 		load_share_files = last;
 	}
-#if defined(THREAD) && !defined(WINNT)
+	in_load--;
+#if defined(WANT_THREAD) && !defined(WINNT)
 	/* Lock the queue */
 	pthread_mutex_unlock(&fserv_struct_mutex);
 #endif
-	in_load--;
 #ifdef WINNT
 	return 0;
 #else
-	return NULL;
+#if defined(WANT_THREAD) && !defined(WINNT)
+	pthread_exit(NULL);
+#endif
 #endif
 }
 
@@ -1495,12 +1592,12 @@ void *do_update(void *arg)
 	unsigned long count = 0;
 	struct stat st;
 	in_load++;
-#if defined(THREAD) && !defined(WINNT)
+#if defined(WANT_THREAD) && !defined(WINNT)
 	pthread_mutex_lock(&fserv_struct_mutex);
 #endif
 	for (tmp = fserv_files; tmp; tmp = last)
 	{
-#if defined(THREAD) && !defined(WINNT)
+#if defined(WANT_THREAD) && !defined(WINNT)
 		if (pthread_mutex_trylock(&quit_mutex) == 0) 
 		{
 			pthread_mutex_unlock(&quit_mutex);
@@ -1526,11 +1623,11 @@ void *do_update(void *arg)
 		}
 		count++;
 	}
-#if defined(THREAD) && !defined(WINNT)
-	pthread_mutex_unlock(&fserv_struct_mutex);
-#endif
 	in_load--;
-	return NULL;
+#if defined(WANT_THREAD) && !defined(WINNT)
+	pthread_mutex_unlock(&fserv_struct_mutex);
+	pthread_exit(NULL);
+#endif
 }
 
 unsigned long remove_nap(char *pattern)
@@ -1556,7 +1653,7 @@ int count = 0;
 /* stolen blatantly from Sheiks cdns non-blocking DNS routines */
 void init_share_mutexes(void)
 {
-#ifdef THREAD
+#ifdef WANT_THREAD
 #ifndef WINNT
 	pthread_mutex_init(&fserv_struct_mutex, NULL);
 	pthread_mutex_init(&shared_count_mutex, NULL);
@@ -1648,7 +1745,7 @@ int share_total = -1;
 			if (share_files)
 				new->next = load_share_files;
 			load_share_files = new;
-#if !defined(THREAD)
+#if !defined(WANT_THREAD)
 #if !defined(WINNT)
 			load_file(load_share_files);
 			goto done_share;
@@ -1717,7 +1814,7 @@ int share_total = -1;
 		}
 	}
 do_sharing:
-#ifdef THREAD
+#ifdef WANT_THREAD
 	if (load_share_files)
 	{
 #ifndef WINNT
@@ -1730,7 +1827,7 @@ do_sharing:
 #endif
 	if (share_files)
 	{
-#if !defined(THREAD)
+#if !defined(WANT_THREAD)
 		scan_mp3_dir(share_files);
 #elif !defined(WINNT)
 		pthread_create(&fserv_thread, NULL, scan_mp3_dir, share_files);
@@ -1743,7 +1840,7 @@ done_share:
 	build_status(current_window, NULL, 0);
 }
 
-#ifdef THREAD
+#ifdef WANT_THREAD
 #ifndef WINNT
 static void share_thread_signal_setup(void)
 {
@@ -1753,6 +1850,10 @@ static void share_thread_signal_setup(void)
 
 	/* Fill set with every signal */
 	sigfillset(&set);
+	/* Remove the few signals that POSIX says we should */
+	sigdelset(&set, SIGFPE);
+	sigdelset(&set, SIGILL);
+	sigdelset(&set, SIGSEGV);
 	/* Apply the mask on this thread */
 	pthread_sigmask(SIG_BLOCK, &set, NULL);
 }	
@@ -1766,5 +1867,24 @@ void clean_share_mutexes(void)
 	pthread_mutex_destroy(&quit_mutex);
 	pthread_mutex_destroy(&send_ncommand_mutex);
 }
+
+void scan_is_done()
+{
+	void *ptr;
+	pthread_join(fserv_thread, &ptr);
+	scan_done = 0;
+}
+#else
+
+void scan_is_done()
+{
+	if (scan_done > 0)
+		say("Found %d files in %s. Use /share -share in order to share these files", scan_done, print_time(time(NULL) - fserv_start));
+	else
+		say("No files found");
+	build_status(current_window, NULL, 0);
+	scan_done = 0;
+}
 #endif
+
 #endif
