@@ -6,7 +6,7 @@
  * Copyright(c) 1990 
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
- * $Id: status.c,v 1.4 2000/07/01 14:03:05 edwards Exp $
+ * $Id: status.c,v 1.5 2000/09/09 06:21:23 cvs Exp $
  */
 
 
@@ -1317,13 +1317,10 @@ double perc = 0.0;
 	return buff;
 }
 
-#ifdef WINNT
-#define THREAD
-#endif
 
 static char *napster_load_share(Window *win)
 {
-#ifdef THREAD
+#ifdef WANT_THREAD
 static char my_buffer[MY_BUFFER];
 extern int in_load;
 	if (in_load)
