@@ -9,7 +9,7 @@
  * Copyright 1998 J. Kean Johnston, used with permission
  * Modifications Copyright Colten Edwards 1997-1998.
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT
- * $Id: term.c,v 1.7 2000/06/27 22:20:02 nuke Exp $
+ * $Id: term.c,v 1.14 2000/09/01 06:57:06 cvs Exp $
  */
 
 #include "teknap.h"
@@ -38,9 +38,6 @@ DWORD gdwPlatform;
 #include "window.h"
 #include "screen.h"
 #include "output.h"
-#ifdef TRANSLATE
-#include "translat.h"
-#endif
 
 
 #if defined(_ALL_SOURCE) || defined(__EMX__)
@@ -66,6 +63,9 @@ char username[40];
 void put_it(const char *str, ...) { return; }
 #endif
 
+#ifdef WINNT
+#define tparm tparm1
+#endif
                 
 #if !defined(WTERM_C)
 
@@ -691,6 +691,23 @@ void	term_putchar (unsigned char c)
 		putchar_x(c);
 }
 
+
+#ifdef WINNT
+static DWORD saved_dwmode;
+
+void reset_console_mode(void)
+{
+
+HANDLE hstdin=GetStdHandle(STD_INPUT_HANDLE);
+
+	SetConsoleMode(hstdin, saved_dwmode );
+}
+#else
+void reset_console_mode(void)
+{
+}
+#endif
+
 /*
  * term_reset: sets terminal attributed back to what they were before the
  * program started 
@@ -711,6 +728,7 @@ void term_reset (void)
 		tputs_x(current_term->TI_smam);
 #endif
 	term_flush();
+	reset_console_mode();
 }
 
 void term_reset2 (void)
@@ -743,6 +761,8 @@ extern int foreground;
 }
 
 #if !defined(STERM_C)
+
+
 /*
  * term_pause: sets terminal back to pre-program days, then SIGSTOPs itself. 
  */
@@ -767,7 +787,8 @@ PROCESS_INFORMATION pi = { 0 };
 	if (!(GetConsoleMode(hstdin,&dwmode)))
 		return;
 
-	if (!SetConsoleMode(hstdin, dwmode | ((ENABLE_LINE_INPUT|ENABLE_ECHO_INPUT|ENABLE_PROCESSED_INPUT) & ~ENABLE_WINDOW_INPUT)))
+	if (!SetConsoleMode(hstdin, dwmode | ((ENABLE_LINE_INPUT|ENABLE_ECHO_INPUT|
+		ENABLE_PROCESSED_INPUT) & ~ENABLE_WINDOW_INPUT)))
 		return;
 
 	if(!CreateProcess(NULL, shell, NULL, NULL, TRUE, 0, NULL, NULL, &si, &pi) )
@@ -809,21 +830,20 @@ int term_init (char *term)
 	int	desired;
 
 
+#ifdef WINNT
+#endif
+
 #if !defined(__EMX__) && !defined(WINNT) && !defined(GUI1)
 	memset(current_term, 0, sizeof(struct term_struct));
 
 	*termcap2 = 0;
-	if (!term && !(term = getenv("TERM")))
+	if (!term && !(term = bsd_getenv("TERM")))
 	{
 		fprintf(stderr, "\n");
 		fprintf(stderr, "You do not have a TERM environment variable.\n");
 		fprintf(stderr, "So we'll be running in dumb mode...\n");
 		return -1;
 	}
-#ifdef WANT_DETACH
-	else if (!already_detached)
-		fprintf(stdout, "Using terminal type [%s]\n", term);
-#endif
 #ifdef HAVE_TERMINFO
 	setupterm(NULL, 1, &i);
 	if (i != 1)
@@ -941,6 +961,9 @@ int term_init (char *term)
 	ghstdout = GetStdHandle(STD_OUTPUT_HANDLE);
 #endif
 
+	if (!(bsd_getenv("TERM")))
+		bsd_setenv("TERM", "vt100", 0);
+
 	if(!GetConsoleScreenBufferInfo(ghstdout, &gscrbuf) ) 
 	{
 		fprintf(stderr,"Error from getconsolebufinfo %ld\n",GetLastError());
@@ -949,8 +972,9 @@ int term_init (char *term)
 
 	GetConsoleMode(hinput,&dwmode);
 	SetConsoleMode(hinput,dwmode & (~(ENABLE_LINE_INPUT |ENABLE_ECHO_INPUT
-				| ENABLE_PROCESSED_INPUT)| ENABLE_WINDOW_INPUT/* | ENABLE_WRAP_AT_EOL_OUTPUT*/)
+				| ENABLE_PROCESSED_INPUT)| ENABLE_WINDOW_INPUT)
 				  );
+	GetConsoleMode(hinput, &saved_dwmode);
 
 	osver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
 	GetVersionEx(&osver);
@@ -1015,6 +1039,7 @@ int term_init (char *term)
 	current_term->TI_nel = strdup("\n");
 	current_term->TI_bel = strdup("\007");
 	current_term->TI_cub = strdup("\010");
+	my_PC = 0;
 #ifdef __EMXPM__
 	current_term->TI_kend = strdup("\eO");
 	current_term->TI_khome = strdup("\eG");
@@ -1177,10 +1202,10 @@ int term_init (char *term)
 		cbuf[0] = '\0';
 		if (i >= 8)
 			strcpy(cbuf, current_term->TI_sgrstrs[TERM_SGR_BOLD_ON-1]);
-		if (current_term->TI_setf) 
-			strcat(cbuf, (char *)tparm(current_term->TI_setf, i & 0x07, 0));
-		else if (current_term->TI_setaf)
+		if (current_term->TI_setaf) 
 			strcat(cbuf, (char *)tparm(current_term->TI_setaf, i & 0x07, 0));
+		else if (current_term->TI_setf)
+			strcat(cbuf, (char *)tparm(current_term->TI_setf, i & 0x07, 0));
 		else if (i >= 8)
 			sprintf(cbuf, "\033[1;%dm", (i & 0x07) + 30);
 		else
