@@ -7,7 +7,7 @@
  * Copyright(c) 1990 
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
- * $Id: vars.c,v 1.7 2000/06/27 10:38:30 edwards Exp $
+ * $Id: vars.c,v 1.12 2000/08/25 17:42:55 cvs Exp $
  */
 
 
@@ -27,6 +27,7 @@
 #include "output.h"
 #include "screen.h"
 #include "server.h"
+#include "stack.h"
 #include "list.h"
 #include "window.h"
 #include "alist.h"
@@ -87,7 +88,7 @@ static	IrcVariable irc_variable[] =
 	{ "ALWAYS_SPLIT_BIGGEST",0,	BOOL_TYPE_VAR,	DEFAULT_ALWAYS_SPLIT_BIGGEST, NULL, NULL, 0, 0 },
 	{ "APPEND_LOG",0,		BOOL_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "AUTO_RECONNECT",0,		BOOL_TYPE_VAR,	DEFAULT_AUTO_RECONNECT, NULL, NULL, 0, VIF_BXNAP },
-	{ "AUTO_REJOIN",0,		INT_TYPE_VAR,   1, NULL, NULL, 0, VIF_BXNAP  },
+	{ "AUTO_REJOIN",0,		BOOL_TYPE_VAR,   1, NULL, NULL, 0, VIF_BXNAP  },
 	{ "AUTO_SHARE", 0,		BOOL_TYPE_VAR,	1, NULL, NULL, 0, VIF_BXNAP },
 	{ "BEEP",0,			BOOL_TYPE_VAR,	DEFAULT_BEEP, NULL, NULL, 0, VIF_BXNAP },
 	{ "BEEP_ALWAYS",0,		BOOL_TYPE_VAR,	0, NULL, NULL, 0, VIF_BXNAP },
@@ -157,7 +158,7 @@ static	IrcVariable irc_variable[] =
 	{ "MAX_SENDS_NICK",0,		INT_TYPE_VAR,	4, NULL, NULL, 0, 0 },
 	{ "MAX_SERVER_RECONNECT",0,	INT_TYPE_VAR,	2, NULL, 0, VIF_BXNAP },
 	{ "MAX_URLS",0,			INT_TYPE_VAR,	50,NULL, 0, VIF_BXNAP },
-	{ "META_STATES_VAR",0,		INT_TYPE_VAR,	DEFAULT_META_STATES, NULL, NULL, 0, 0 },
+	{ "META_STATES",0,		INT_TYPE_VAR,	DEFAULT_META_STATES, NULL, NULL, 0, 0 },
 	{ "MOVE_INCOMPLETE",0,		BOOL_TYPE_VAR,	1, NULL, NULL, 0, 0 },
 	{ "NAMES_COLUMNS",0,		INT_TYPE_VAR,	4, NULL, NULL, 0, VIF_BXNAP },
 	{ "NAMES_NICKCOLOR",0,		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
@@ -317,7 +318,7 @@ void init_variables()
 
 	set_string_var(FORMAT_DIRECTORY_VAR, "%d");
 	set_string_var(FORMAT_FILENAME_VAR, "%g4.1L %b %t %f");
-	set_string_var(SEARCH_FORMAT_VAR, "%-40f %b %3.1H %t %g4.1L %N");
+	set_string_var(SEARCH_FORMAT_VAR, "%-40f %b %3.1H %t %g4.1L %N %1.3p");
 	
 	set_string_var(NAMES_NICKCOLOR_VAR, FORMAT_NICKCOLOR);
 	set_string_var(DOWNLOAD_DIRECTORY_VAR, DEFAULT_DOWNLOAD_DIRECTORY);
@@ -778,7 +779,7 @@ static void set_ptest_connect(Window *win, char *unused, int value)
 
 static void set_ptest_timeout(Window *win, char *unused, int value)
 {
-#ifdef WANT_PTEST
+#if defined(WANT_PTEST) && defined(WANT_THREAD)
 	if (value == ON)
 		start_ptest();
 	else if (value == OFF)
@@ -878,3 +879,104 @@ char	*make_string_var(const char *var_name)
 	return ret;
 }
 
+typedef struct	varstacklist
+{
+	int 		which;
+	IrcVariable 	*set;
+	char		*name;
+	int		var_index;
+	struct varstacklist *next;
+}	VarStack;
+
+VarStack *set_stack = NULL;
+
+void do_stack_set(int type, char *args)
+{
+	VarStack *aptr = set_stack;
+	VarStack **aptrptr = &set_stack;
+
+	if (!*aptrptr && (type == STACK_POP || type == STACK_LIST))
+	{
+		say("Set stack is empty!");
+		return;
+	}
+
+	if (STACK_PUSH == type)
+	{
+		enum VAR_TYPES var_index;
+		int cnt = 0;
+
+		upper(args);
+		find_fixed_array_item (irc_variable, sizeof(IrcVariable), NUMBER_OF_VARIABLES, args, &cnt, (int *)&var_index);
+
+		if (cnt < 0 || cnt == 1)
+		{
+			aptr = (VarStack *)new_malloc(sizeof(VarStack));
+			aptr->next = aptrptr ? *aptrptr : NULL;
+			*aptrptr = aptr;
+			aptr->set = (IrcVariable *) new_malloc(sizeof(IrcVariable));
+			memcpy(aptr->set, &irc_variable[var_index], sizeof(IrcVariable));
+			aptr->name = m_strdup(irc_variable[var_index].name);
+			aptr->set->string = m_strdup(irc_variable[var_index].string);
+			aptr->var_index = var_index;
+		}
+		else
+			say("No such Set [%s]", args);
+
+		return;
+	}
+
+	if (STACK_POP == type)
+	{
+		VarStack *prev = NULL;
+		for (aptr = *aptrptr; aptr; prev = aptr, aptr = aptr->next)
+		{
+			/* have we found it on the stack? */
+			if (!my_stricmp(args, aptr->name))
+			{
+				/* remove it from the list */
+				if (prev == NULL)
+					*aptrptr = aptr->next;
+				else
+					prev->next = aptr->next;
+
+				new_free(&(irc_variable[aptr->var_index].string));
+				memcpy(&irc_variable[aptr->var_index], aptr->set, sizeof(IrcVariable));
+				/* free it */
+				new_free((char **)&aptr->name);
+				new_free((char **)&aptr->set);
+				new_free((char **)&aptr);
+				return;
+			}
+		}
+		say("%s is not on the %s stack!", args, "Set");
+		return;
+	}
+	if (STACK_LIST == type)
+	{
+		VarStack *prev = NULL;
+		for (aptr = *aptrptr; aptr; prev = aptr, aptr = aptr->next)
+		{
+			switch(aptr->set->type)
+			{
+				case BOOL_TYPE_VAR:
+					say("Variable [%s] = %s", aptr->set->name, var_settings[aptr->set->integer]);
+					break;
+				case INT_TYPE_VAR:
+					say("Variable [%s] = %d", aptr->set->name, aptr->set->integer);
+					break;
+				case CHAR_TYPE_VAR:
+					say("Variable [%s] = %c", aptr->set->name, aptr->set->integer);
+					break;
+				case STR_TYPE_VAR:
+					say("Variable [%s] = %s", aptr->set->name, aptr->set->string?aptr->set->string:"<Empty String>");
+					break;
+				default:
+					say("Error in do_stack_set: unknown set type");
+			}
+		}
+		return;
+	}
+	say("Unknown STACK type ??");
+}
+
